/****************************************************************************
** Meta object code from reading C++ file 'syncresult.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/syncresult.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncresult.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC10SyncResultE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncResult::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10SyncResultE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncResult",
        "Status",
        "Undefined",
        "NotYetStarted",
        "SyncPrepare",
        "SyncRunning",
        "SyncAbortRequested",
        "Success",
        "Problem",
        "Error",
        "SetupError",
        "Paused"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(1, 1, QMC::EnumFlags{}).add({
            {    2, Status::Undefined },
            {    3, Status::NotYetStarted },
            {    4, Status::SyncPrepare },
            {    5, Status::SyncRunning },
            {    6, Status::SyncAbortRequested },
            {    7, Status::Success },
            {    8, Status::Problem },
            {    9, Status::Error },
            {   10, Status::SetupError },
            {   11, Status::Paused },
        }),
    };
    return QtMocHelpers::metaObjectData<SyncResult, qt_meta_tag_ZN3OCC10SyncResultE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncResult::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncResultE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncResultE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10SyncResultE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
