/****************************************************************************
** Meta object code from reading C++ file 'folderwizard.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/folderwizard.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folderwizard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FormatWarningsWizardPage::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FormatWarningsWizardPage"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FormatWarningsWizardPage, qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FormatWarningsWizardPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t>.metaTypes,
    nullptr
} };

void OCC::FormatWarningsWizardPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FormatWarningsWizardPage *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OCC::FormatWarningsWizardPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FormatWarningsWizardPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC24FormatWarningsWizardPageE_t>.strings))
        return static_cast<void*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int OCC::FormatWarningsWizardPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderWizardLocalPath::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderWizardLocalPath",
        "initialFolderSelectionCanceled",
        "",
        "slotChooseLocalFolder"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'initialFolderSelectionCanceled'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChooseLocalFolder'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderWizardLocalPath, qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderWizardLocalPath::staticMetaObject = { {
    QMetaObject::SuperData::link<FormatWarningsWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t>.metaTypes,
    nullptr
} };

void OCC::FolderWizardLocalPath::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderWizardLocalPath *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->initialFolderSelectionCanceled(); break;
        case 1: _t->slotChooseLocalFolder(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FolderWizardLocalPath::*)()>(_a, &FolderWizardLocalPath::initialFolderSelectionCanceled, 0))
            return;
    }
}

const QMetaObject *OCC::FolderWizardLocalPath::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderWizardLocalPath::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21FolderWizardLocalPathE_t>.strings))
        return static_cast<void*>(this);
    return FormatWarningsWizardPage::qt_metacast(_clname);
}

int OCC::FolderWizardLocalPath::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FormatWarningsWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::FolderWizardLocalPath::initialFolderSelectionCanceled()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderWizardRemotePath::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderWizardRemotePath",
        "showWarn",
        "",
        "slotAddRemoteFolder",
        "slotCreateRemoteFolder",
        "slotCreateRemoteFolderFinished",
        "slotHandleMkdirNetworkError",
        "QNetworkReply*",
        "slotHandleLsColNetworkError",
        "slotUpdateDirectories",
        "slotGatherEncryptedPaths",
        "QMap<QString,QString>",
        "slotRefreshFolders",
        "slotItemExpanded",
        "QTreeWidgetItem*",
        "slotCurrentItemChanged",
        "slotFolderEntryEdited",
        "text",
        "slotLsColFolderEntry",
        "slotTypedPathFound",
        "subpaths",
        "changeStyle"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'showWarn'
        QtMocHelpers::SlotData<void(const QString &) const>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'showWarn'
        QtMocHelpers::SlotData<void() const>(1, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotAddRemoteFolder'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCreateRemoteFolder'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotCreateRemoteFolderFinished'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHandleMkdirNetworkError'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'slotHandleLsColNetworkError'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'slotUpdateDirectories'
        QtMocHelpers::SlotData<void(const QStringList &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QStringList, 2 },
        }}),
        // Slot 'slotGatherEncryptedPaths'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { 0x80000000 | 11, 2 },
        }}),
        // Slot 'slotRefreshFolders'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotItemExpanded'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'slotCurrentItemChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'slotFolderEntryEdited'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'slotLsColFolderEntry'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTypedPathFound'
        QtMocHelpers::SlotData<void(const QStringList &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QStringList, 20 },
        }}),
        // Slot 'changeStyle'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderWizardRemotePath, qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderWizardRemotePath::staticMetaObject = { {
    QMetaObject::SuperData::link<FormatWarningsWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t>.metaTypes,
    nullptr
} };

void OCC::FolderWizardRemotePath::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderWizardRemotePath *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showWarn((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->showWarn(); break;
        case 2: _t->slotAddRemoteFolder(); break;
        case 3: _t->slotCreateRemoteFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->slotCreateRemoteFolderFinished(); break;
        case 5: _t->slotHandleMkdirNetworkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 6: _t->slotHandleLsColNetworkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 7: _t->slotUpdateDirectories((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: _t->slotGatherEncryptedPaths((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 9: _t->slotRefreshFolders(); break;
        case 10: _t->slotItemExpanded((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 11: _t->slotCurrentItemChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 12: _t->slotFolderEntryEdited((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->slotLsColFolderEntry(); break;
        case 14: _t->slotTypedPathFound((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 15: _t->changeStyle(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OCC::FolderWizardRemotePath::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderWizardRemotePath::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC22FolderWizardRemotePathE_t>.strings))
        return static_cast<void*>(this);
    return FormatWarningsWizardPage::qt_metacast(_clname);
}

int OCC::FolderWizardRemotePath::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FormatWarningsWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderWizardSelectiveSync::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderWizardSelectiveSync",
        "virtualFilesCheckboxClicked",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'virtualFilesCheckboxClicked'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderWizardSelectiveSync, qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderWizardSelectiveSync::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t>.metaTypes,
    nullptr
} };

void OCC::FolderWizardSelectiveSync::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderWizardSelectiveSync *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->virtualFilesCheckboxClicked(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *OCC::FolderWizardSelectiveSync::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderWizardSelectiveSync::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC25FolderWizardSelectiveSyncE_t>.strings))
        return static_cast<void*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int OCC::FolderWizardSelectiveSync::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC12FolderWizardE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderWizard::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12FolderWizardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderWizard"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderWizard, qt_meta_tag_ZN3OCC12FolderWizardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderWizard::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizard::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12FolderWizardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12FolderWizardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12FolderWizardE_t>.metaTypes,
    nullptr
} };

void OCC::FolderWizard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderWizard *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OCC::FolderWizard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderWizard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12FolderWizardE_t>.strings))
        return static_cast<void*>(this);
    return QWizard::qt_metacast(_clname);
}

int OCC::FolderWizard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizard::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
