/****************************************************************************
** Meta object code from reading C++ file 'pushnotifications.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/pushnotifications.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pushnotifications.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC17PushNotificationsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PushNotifications::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17PushNotificationsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PushNotifications",
        "ready",
        "",
        "filesChanged",
        "OCC::Account*",
        "account",
        "fileIdsChanged",
        "QList<qint64>",
        "fileIds",
        "activitiesChanged",
        "notificationsChanged",
        "authenticationFailed",
        "connectionLost",
        "onWebSocketConnected",
        "onWebSocketDisconnected",
        "onWebSocketTextMessageReceived",
        "message",
        "onWebSocketError",
        "QAbstractSocket::SocketError",
        "error",
        "onWebSocketSslErrors",
        "QList<QSslError>",
        "errors",
        "onWebSocketPongReceived",
        "elapsedTime",
        "payload",
        "onPingTimedOut"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filesChanged'
        QtMocHelpers::SignalData<void(OCC::Account *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'fileIdsChanged'
        QtMocHelpers::SignalData<void(OCC::Account *, const QList<qint64> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'activitiesChanged'
        QtMocHelpers::SignalData<void(OCC::Account *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'notificationsChanged'
        QtMocHelpers::SignalData<void(OCC::Account *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'authenticationFailed'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'connectionLost'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onWebSocketConnected'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWebSocketDisconnected'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWebSocketTextMessageReceived'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'onWebSocketError'
        QtMocHelpers::SlotData<void(QAbstractSocket::SocketError)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'onWebSocketSslErrors'
        QtMocHelpers::SlotData<void(const QList<QSslError> &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'onWebSocketPongReceived'
        QtMocHelpers::SlotData<void(quint64, const QByteArray &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 24 }, { QMetaType::QByteArray, 25 },
        }}),
        // Slot 'onPingTimedOut'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PushNotifications, qt_meta_tag_ZN3OCC17PushNotificationsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PushNotifications::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17PushNotificationsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17PushNotificationsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17PushNotificationsE_t>.metaTypes,
    nullptr
} };

void OCC::PushNotifications::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PushNotifications *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->filesChanged((*reinterpret_cast<std::add_pointer_t<OCC::Account*>>(_a[1]))); break;
        case 2: _t->fileIdsChanged((*reinterpret_cast<std::add_pointer_t<OCC::Account*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<qint64>>>(_a[2]))); break;
        case 3: _t->activitiesChanged((*reinterpret_cast<std::add_pointer_t<OCC::Account*>>(_a[1]))); break;
        case 4: _t->notificationsChanged((*reinterpret_cast<std::add_pointer_t<OCC::Account*>>(_a[1]))); break;
        case 5: _t->authenticationFailed(); break;
        case 6: _t->connectionLost(); break;
        case 7: _t->onWebSocketConnected(); break;
        case 8: _t->onWebSocketDisconnected(); break;
        case 9: _t->onWebSocketTextMessageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->onWebSocketError((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketError>>(_a[1]))); break;
        case 11: _t->onWebSocketSslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 12: _t->onWebSocketPongReceived((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 13: _t->onPingTimedOut(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<qint64> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)()>(_a, &PushNotifications::ready, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)(OCC::Account * )>(_a, &PushNotifications::filesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)(OCC::Account * , const QList<qint64> & )>(_a, &PushNotifications::fileIdsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)(OCC::Account * )>(_a, &PushNotifications::activitiesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)(OCC::Account * )>(_a, &PushNotifications::notificationsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)()>(_a, &PushNotifications::authenticationFailed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PushNotifications::*)()>(_a, &PushNotifications::connectionLost, 6))
            return;
    }
}

const QMetaObject *OCC::PushNotifications::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PushNotifications::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17PushNotificationsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::PushNotifications::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void OCC::PushNotifications::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::PushNotifications::filesChanged(OCC::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::PushNotifications::fileIdsChanged(OCC::Account * _t1, const QList<qint64> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::PushNotifications::activitiesChanged(OCC::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::PushNotifications::notificationsChanged(OCC::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::PushNotifications::authenticationFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::PushNotifications::connectionLost()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
