/****************************************************************************
** Meta object code from reading C++ file 'syncfileitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/syncfileitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncfileitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12SyncFileItemE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncFileItem::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12SyncFileItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncFileItem",
        "Direction",
        "None",
        "Up",
        "Down",
        "SynchronizationOptions",
        "NormalSynchronization",
        "WantsPermanentDeletion",
        "MoveToClientTrashBin",
        "Status",
        "NoStatus",
        "FatalError",
        "NormalError",
        "SoftError",
        "Conflict",
        "FileIgnored",
        "FileLocked",
        "Restoration",
        "FileNameInvalid",
        "FileNameInvalidOnServer",
        "FileNameClash",
        "DetailError",
        "BlacklistedError",
        "Success"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Direction'
        QtMocHelpers::EnumData<enum Direction>(1, 1, QMC::EnumFlags{}).add({
            {    2, Direction::None },
            {    3, Direction::Up },
            {    4, Direction::Down },
        }),
        // enum 'SynchronizationOptions'
        QtMocHelpers::EnumData<enum SynchronizationOptions>(5, 5, QMC::EnumIsScoped).add({
            {    6, SynchronizationOptions::NormalSynchronization },
            {    7, SynchronizationOptions::WantsPermanentDeletion },
            {    8, SynchronizationOptions::MoveToClientTrashBin },
        }),
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(9, 9, QMC::EnumFlags{}).add({
            {   10, Status::NoStatus },
            {   11, Status::FatalError },
            {   12, Status::NormalError },
            {   13, Status::SoftError },
            {   14, Status::Conflict },
            {   15, Status::FileIgnored },
            {   16, Status::FileLocked },
            {   17, Status::Restoration },
            {   18, Status::FileNameInvalid },
            {   19, Status::FileNameInvalidOnServer },
            {   20, Status::FileNameClash },
            {   21, Status::DetailError },
            {   22, Status::BlacklistedError },
            {   23, Status::Success },
        }),
    };
    return QtMocHelpers::metaObjectData<SyncFileItem, qt_meta_tag_ZN3OCC12SyncFileItemE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncFileItem::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12SyncFileItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12SyncFileItemE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12SyncFileItemE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
