/*
 * SPDX-FileCopyrightText: 2013 ownCloud GmbH
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef MIRALL_CREDS_COMMON_H
#define MIRALL_CREDS_COMMON_H

#include "owncloudlib.h"

namespace OCC {

} // namespace OCC

#endif
