/****************************************************************************
** Meta object code from reading C++ file 'webflowcredentialsdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/creds/webflowcredentialsdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'webflowcredentialsdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::WebFlowCredentialsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::WebFlowCredentialsDialog",
        "urlCatched",
        "",
        "user",
        "pass",
        "host",
        "styleChanged",
        "onActivate",
        "onClose",
        "slotFlow2AuthResult",
        "OCC::Flow2Auth::Result",
        "errorString",
        "appPassword",
        "slotShowSettingsDialog"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'urlCatched'
        QtMocHelpers::SignalData<void(const QString, const QString, const QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'styleChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onActivate'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onClose'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFlow2AuthResult'
        QtMocHelpers::SlotData<void(OCC::Flow2Auth::Result, const QString &, const QString &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 }, { QMetaType::QString, 11 }, { QMetaType::QString, 3 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'slotShowSettingsDialog'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WebFlowCredentialsDialog, qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::WebFlowCredentialsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t>.metaTypes,
    nullptr
} };

void OCC::WebFlowCredentialsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WebFlowCredentialsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->urlCatched((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->styleChanged(); break;
        case 2: _t->onActivate(); break;
        case 3: _t->onClose(); break;
        case 4: _t->slotFlow2AuthResult((*reinterpret_cast<std::add_pointer_t<OCC::Flow2Auth::Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 5: _t->slotShowSettingsDialog(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WebFlowCredentialsDialog::*)(const QString , const QString , const QString )>(_a, &WebFlowCredentialsDialog::urlCatched, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WebFlowCredentialsDialog::*)()>(_a, &WebFlowCredentialsDialog::styleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WebFlowCredentialsDialog::*)()>(_a, &WebFlowCredentialsDialog::onActivate, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WebFlowCredentialsDialog::*)()>(_a, &WebFlowCredentialsDialog::onClose, 3))
            return;
    }
}

const QMetaObject *OCC::WebFlowCredentialsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::WebFlowCredentialsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC24WebFlowCredentialsDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int OCC::WebFlowCredentialsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void OCC::WebFlowCredentialsDialog::urlCatched(const QString _t1, const QString _t2, const QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void OCC::WebFlowCredentialsDialog::styleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::WebFlowCredentialsDialog::onActivate()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::WebFlowCredentialsDialog::onClose()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
