/****************************************************************************
** Meta object code from reading C++ file 'accountsettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/accountsettings.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountsettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC15AccountSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountSettings::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC15AccountSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountSettings",
        "folderChanged",
        "",
        "openFolderAlias",
        "showIssuesList",
        "OCC::AccountState*",
        "account",
        "requestMnemonic",
        "removeAccountFolders",
        "styleChanged",
        "slotOpenOC",
        "slotUpdateQuota",
        "total",
        "used",
        "slotAccountStateChanged",
        "slotStyleChanged",
        "slotHideSelectiveSyncWidget",
        "slotAddFolder",
        "slotEnableCurrentFolder",
        "terminate",
        "slotScheduleCurrentFolder",
        "slotScheduleCurrentFolderForceRemoteDiscovery",
        "slotForceSyncCurrentFolder",
        "slotRemoveCurrentFolder",
        "slotOpenCurrentFolder",
        "slotOpenCurrentLocalSubFolder",
        "slotEditCurrentIgnoredFiles",
        "slotOpenMakeFolderDialog",
        "slotEditCurrentLocalIgnoredFiles",
        "slotEnableVfsCurrentFolder",
        "slotDisableVfsCurrentFolder",
        "slotSetCurrentFolderAvailability",
        "OCC::PinState",
        "state",
        "slotSetSubFolderAvailability",
        "OCC::Folder*",
        "folder",
        "path",
        "slotFolderWizardAccepted",
        "slotFolderWizardRejected",
        "slotToggleSignInState",
        "refreshSelectiveSyncStatus",
        "slotMarkSubfolderEncrypted",
        "OCC::FolderStatusModel::SubFolderInfo*",
        "folderInfo",
        "slotSubfolderContextMenuRequested",
        "QModelIndex",
        "idx",
        "QPoint",
        "point",
        "slotCustomContextMenuRequested",
        "slotFolderListClicked",
        "indx",
        "doExpand",
        "slotLinkActivated",
        "link",
        "slotE2eEncryptionMnemonicReady",
        "slotE2eEncryptionGenerateKeys",
        "slotE2eEncryptionInitializationFinished",
        "isNewMnemonicGenerated",
        "slotEncryptFolderFinished",
        "status",
        "slotSelectiveSyncChanged",
        "topLeft",
        "bottomRight",
        "QList<int>",
        "roles",
        "slotPossiblyUnblacklistE2EeFoldersAndRestartSync",
        "slotE2eEncryptionCertificateNeedMigration",
        "updateBlackListAndScheduleFolderSync",
        "blackList",
        "foldersToRemoveFromBlacklist",
        "folderTerminateSyncAndUpdateBlackList",
        "displayMnemonic",
        "mnemonic",
        "forgetEncryptionOnDeviceForAccount",
        "OCC::AccountPtr",
        "migrateCertificateForAccount",
        "showConnectionLabel",
        "message",
        "errors",
        "openIgnoredFilesDialog",
        "absFolderPath",
        "customizeStyle",
        "setupE2eEncryption",
        "forgetE2eEncryption",
        "checkClientSideEncryptionState",
        "removeActionFromEncryptionMessage",
        "actionId",
        "accountState",
        "AccountState*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'folderChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openFolderAlias'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'showIssuesList'
        QtMocHelpers::SignalData<void(OCC::AccountState *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'requestMnemonic'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'removeAccountFolders'
        QtMocHelpers::SignalData<void(OCC::AccountState *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'styleChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenOC'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateQuota'
        QtMocHelpers::SlotData<void(qint64, qint64)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 12 }, { QMetaType::LongLong, 13 },
        }}),
        // Slot 'slotAccountStateChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStyleChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideSelectiveSyncWidget'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddFolder'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnableCurrentFolder'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'slotEnableCurrentFolder'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotScheduleCurrentFolder'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotScheduleCurrentFolderForceRemoteDiscovery'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotForceSyncCurrentFolder'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveCurrentFolder'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOpenCurrentFolder'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOpenCurrentLocalSubFolder'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditCurrentIgnoredFiles'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOpenMakeFolderDialog'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditCurrentLocalIgnoredFiles'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnableVfsCurrentFolder'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDisableVfsCurrentFolder'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSetCurrentFolderAvailability'
        QtMocHelpers::SlotData<void(OCC::PinState)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotSetSubFolderAvailability'
        QtMocHelpers::SlotData<void(OCC::Folder *, const QString &, OCC::PinState)>(34, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 35, 36 }, { QMetaType::QString, 37 }, { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotFolderWizardAccepted'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFolderWizardRejected'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotToggleSignInState'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'refreshSelectiveSyncStatus'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMarkSubfolderEncrypted'
        QtMocHelpers::SlotData<void(OCC::FolderStatusModel::SubFolderInfo *)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'slotSubfolderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotCustomContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 48, 2 },
        }}),
        // Slot 'slotFolderListClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(51, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 46, 52 },
        }}),
        // Slot 'doExpand'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotLinkActivated'
        QtMocHelpers::SlotData<void(const QString &)>(54, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 55 },
        }}),
        // Slot 'slotE2eEncryptionMnemonicReady'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotE2eEncryptionGenerateKeys'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotE2eEncryptionInitializationFinished'
        QtMocHelpers::SlotData<void(bool)>(58, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 59 },
        }}),
        // Slot 'slotEncryptFolderFinished'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 61 },
        }}),
        // Slot 'slotSelectiveSyncChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &, const QVector<int> &)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 46, 63 }, { 0x80000000 | 46, 64 }, { 0x80000000 | 65, 66 },
        }}),
        // Slot 'slotPossiblyUnblacklistE2EeFoldersAndRestartSync'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotE2eEncryptionCertificateNeedMigration'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateBlackListAndScheduleFolderSync'
        QtMocHelpers::SlotData<void(const QStringList &, OCC::Folder *, const QStringList &) const>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 70 }, { 0x80000000 | 35, 36 }, { QMetaType::QStringList, 71 },
        }}),
        // Slot 'folderTerminateSyncAndUpdateBlackList'
        QtMocHelpers::SlotData<void(const QStringList &, OCC::Folder *, const QStringList &)>(72, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 70 }, { 0x80000000 | 35, 36 }, { QMetaType::QStringList, 71 },
        }}),
        // Slot 'displayMnemonic'
        QtMocHelpers::SlotData<void(const QString &)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 74 },
        }}),
        // Slot 'forgetEncryptionOnDeviceForAccount'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &) const>(75, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 76, 6 },
        }}),
        // Slot 'migrateCertificateForAccount'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &)>(77, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 76, 6 },
        }}),
        // Slot 'showConnectionLabel'
        QtMocHelpers::SlotData<void(const QString &, QStringList)>(78, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 79 }, { QMetaType::QStringList, 80 },
        }}),
        // Slot 'showConnectionLabel'
        QtMocHelpers::SlotData<void(const QString &)>(78, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 79 },
        }}),
        // Slot 'openIgnoredFilesDialog'
        QtMocHelpers::SlotData<void(const QString &)>(81, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 82 },
        }}),
        // Slot 'customizeStyle'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setupE2eEncryption'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'forgetE2eEncryption'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkClientSideEncryptionState'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeActionFromEncryptionMessage'
        QtMocHelpers::SlotData<void(const QString &)>(87, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 88 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accountState'
        QtMocHelpers::PropertyData<AccountState*>(89, 0x80000000 | 90, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountSettings, qt_meta_tag_ZN3OCC15AccountSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::AccountSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.metaTypes,
    nullptr
} };

void OCC::AccountSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->folderChanged(); break;
        case 1: _t->openFolderAlias((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->showIssuesList((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 3: _t->requestMnemonic(); break;
        case 4: _t->removeAccountFolders((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 5: _t->styleChanged(); break;
        case 6: _t->slotOpenOC(); break;
        case 7: _t->slotUpdateQuota((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 8: _t->slotAccountStateChanged(); break;
        case 9: _t->slotStyleChanged(); break;
        case 10: _t->slotHideSelectiveSyncWidget(); break;
        case 11: _t->slotAddFolder(); break;
        case 12: _t->slotEnableCurrentFolder((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotEnableCurrentFolder(); break;
        case 14: _t->slotScheduleCurrentFolder(); break;
        case 15: _t->slotScheduleCurrentFolderForceRemoteDiscovery(); break;
        case 16: _t->slotForceSyncCurrentFolder(); break;
        case 17: _t->slotRemoveCurrentFolder(); break;
        case 18: _t->slotOpenCurrentFolder(); break;
        case 19: _t->slotOpenCurrentLocalSubFolder(); break;
        case 20: _t->slotEditCurrentIgnoredFiles(); break;
        case 21: _t->slotOpenMakeFolderDialog(); break;
        case 22: _t->slotEditCurrentLocalIgnoredFiles(); break;
        case 23: _t->slotEnableVfsCurrentFolder(); break;
        case 24: _t->slotDisableVfsCurrentFolder(); break;
        case 25: _t->slotSetCurrentFolderAvailability((*reinterpret_cast<std::add_pointer_t<OCC::PinState>>(_a[1]))); break;
        case 26: _t->slotSetSubFolderAvailability((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<OCC::PinState>>(_a[3]))); break;
        case 27: _t->slotFolderWizardAccepted(); break;
        case 28: _t->slotFolderWizardRejected(); break;
        case 29: _t->slotToggleSignInState(); break;
        case 30: _t->refreshSelectiveSyncStatus(); break;
        case 31: _t->slotMarkSubfolderEncrypted((*reinterpret_cast<std::add_pointer_t<OCC::FolderStatusModel::SubFolderInfo*>>(_a[1]))); break;
        case 32: _t->slotSubfolderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 33: _t->slotCustomContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 34: _t->slotFolderListClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 35: _t->doExpand(); break;
        case 36: _t->slotLinkActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->slotE2eEncryptionMnemonicReady(); break;
        case 38: _t->slotE2eEncryptionGenerateKeys(); break;
        case 39: _t->slotE2eEncryptionInitializationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->slotEncryptFolderFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 41: _t->slotSelectiveSyncChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 42: _t->slotPossiblyUnblacklistE2EeFoldersAndRestartSync(); break;
        case 43: _t->slotE2eEncryptionCertificateNeedMigration(); break;
        case 44: _t->updateBlackListAndScheduleFolderSync((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 45: _t->folderTerminateSyncAndUpdateBlackList((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 46: _t->displayMnemonic((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 47: _t->forgetEncryptionOnDeviceForAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 48: _t->migrateCertificateForAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 49: _t->showConnectionLabel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 50: _t->showConnectionLabel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 51: _t->openIgnoredFilesDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->customizeStyle(); break;
        case 53: _t->setupE2eEncryption(); break;
        case 54: _t->forgetE2eEncryption(); break;
        case 55: _t->checkClientSideEncryptionState(); break;
        case 56: _t->removeActionFromEncryptionMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::FolderStatusModel::SubFolderInfo* >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 44:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 45:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 47:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountPtr >(); break;
            }
            break;
        case 48:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::folderChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)(const QString & )>(_a, &AccountSettings::openFolderAlias, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)(OCC::AccountState * )>(_a, &AccountSettings::showIssuesList, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::requestMnemonic, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)(OCC::AccountState * )>(_a, &AccountSettings::removeAccountFolders, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::styleChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AccountState* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountState**>(_v) = _t->_accountState; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->_accountState, *reinterpret_cast<AccountState**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::AccountSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int OCC::AccountSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OCC::AccountSettings::folderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::AccountSettings::openFolderAlias(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::AccountSettings::showIssuesList(OCC::AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::AccountSettings::requestMnemonic()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::AccountSettings::removeAccountFolders(OCC::AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::AccountSettings::styleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
