/****************************************************************************
** Meta object code from reading C++ file 'caseclashconflictsolver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/caseclashconflictsolver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'caseclashconflictsolver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::CaseClashConflictSolver::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::CaseClashConflictSolver",
        "allowedToRenameChanged",
        "",
        "errorStringChanged",
        "done",
        "failed",
        "solveConflict",
        "newFilename",
        "checkIfAllowedToRename",
        "onRemoteDestinationFileAlreadyExists",
        "onRemoteDestinationFileDoesNotExist",
        "onPropfindPermissionSuccess",
        "QVariantMap",
        "values",
        "onPropfindPermissionError",
        "QNetworkReply*",
        "reply",
        "onRemoteSourceFileAlreadyExists",
        "onRemoteSourceFileDoesNotExist",
        "onMoveJobFinished",
        "allowedToRename",
        "errorString"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'allowedToRenameChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorStringChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'done'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'failed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'solveConflict'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'checkIfAllowedToRename'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRemoteDestinationFileAlreadyExists'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onRemoteDestinationFileDoesNotExist'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPropfindPermissionSuccess'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onPropfindPermissionError'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'onRemoteSourceFileAlreadyExists'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onRemoteSourceFileDoesNotExist'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onMoveJobFinished'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'allowedToRename'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CaseClashConflictSolver, qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::CaseClashConflictSolver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t>.metaTypes,
    nullptr
} };

void OCC::CaseClashConflictSolver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CaseClashConflictSolver *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->allowedToRenameChanged(); break;
        case 1: _t->errorStringChanged(); break;
        case 2: _t->done(); break;
        case 3: _t->failed(); break;
        case 4: _t->solveConflict((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->checkIfAllowedToRename(); break;
        case 6: _t->onRemoteDestinationFileAlreadyExists(); break;
        case 7: _t->onRemoteDestinationFileDoesNotExist(); break;
        case 8: _t->onPropfindPermissionSuccess((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 9: _t->onPropfindPermissionError((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 10: _t->onRemoteSourceFileAlreadyExists(); break;
        case 11: _t->onRemoteSourceFileDoesNotExist(); break;
        case 12: _t->onMoveJobFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CaseClashConflictSolver::*)()>(_a, &CaseClashConflictSolver::allowedToRenameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CaseClashConflictSolver::*)()>(_a, &CaseClashConflictSolver::errorStringChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CaseClashConflictSolver::*)()>(_a, &CaseClashConflictSolver::done, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CaseClashConflictSolver::*)()>(_a, &CaseClashConflictSolver::failed, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->allowedToRename(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::CaseClashConflictSolver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::CaseClashConflictSolver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC23CaseClashConflictSolverE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::CaseClashConflictSolver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::CaseClashConflictSolver::allowedToRenameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::CaseClashConflictSolver::errorStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::CaseClashConflictSolver::done()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::CaseClashConflictSolver::failed()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
