/****************************************************************************
** Meta object code from reading C++ file 'clientsideencryptiontokenselector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/clientsideencryptiontokenselector.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'clientsideencryptiontokenselector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ClientSideEncryptionTokenSelector::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ClientSideEncryptionTokenSelector",
        "isSetupChanged",
        "",
        "sha256FingerprintChanged",
        "failedToInitialize",
        "OCC::AccountPtr",
        "account",
        "searchForCertificates",
        "QFuture<void>",
        "setSha256Fingerprint",
        "sha256Fingerprint",
        "isSetup"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'isSetupChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sha256FingerprintChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'failedToInitialize'
        QtMocHelpers::SignalData<void(const OCC::AccountPtr &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'searchForCertificates'
        QtMocHelpers::SlotData<QFuture<void>(const OCC::AccountPtr &)>(7, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setSha256Fingerprint'
        QtMocHelpers::SlotData<void(const QByteArray &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isSetup'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'sha256Fingerprint'
        QtMocHelpers::PropertyData<QByteArray>(10, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ClientSideEncryptionTokenSelector, qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ClientSideEncryptionTokenSelector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t>.metaTypes,
    nullptr
} };

void OCC::ClientSideEncryptionTokenSelector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ClientSideEncryptionTokenSelector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->isSetupChanged(); break;
        case 1: _t->sha256FingerprintChanged(); break;
        case 2: _t->failedToInitialize((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 3: { QFuture<void> _r = _t->searchForCertificates((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QFuture<void>*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setSha256Fingerprint((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryptionTokenSelector::*)()>(_a, &ClientSideEncryptionTokenSelector::isSetupChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryptionTokenSelector::*)()>(_a, &ClientSideEncryptionTokenSelector::sha256FingerprintChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryptionTokenSelector::*)(const OCC::AccountPtr & )>(_a, &ClientSideEncryptionTokenSelector::failedToInitialize, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isSetup(); break;
        case 1: *reinterpret_cast<QByteArray*>(_v) = _t->sha256Fingerprint(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSha256Fingerprint(*reinterpret_cast<QByteArray*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::ClientSideEncryptionTokenSelector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ClientSideEncryptionTokenSelector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC33ClientSideEncryptionTokenSelectorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ClientSideEncryptionTokenSelector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::ClientSideEncryptionTokenSelector::isSetupChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ClientSideEncryptionTokenSelector::sha256FingerprintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::ClientSideEncryptionTokenSelector::failedToInitialize(const OCC::AccountPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
