/****************************************************************************
** Meta object code from reading C++ file 'owncloudwizard.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/wizard/owncloudwizard.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'owncloudwizard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14OwncloudWizardE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::OwncloudWizard::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14OwncloudWizardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::OwncloudWizard",
        "clearPendingRequests",
        "",
        "determineAuthType",
        "QUrl",
        "serverURL",
        "OCC::WizardProxySettingsDialog::WizardProxySettings",
        "proxySettings",
        "connectToOCUrl",
        "createLocalAndRemoteFolders",
        "basicSetupFinished",
        "skipFolderConfiguration",
        "needCertificate",
        "styleChanged",
        "onActivate",
        "wizardClosed",
        "setAuthType",
        "OCC::DetermineAuthTypeJob::AuthType",
        "type",
        "setRemoteFolder",
        "appendToConfigurationLog",
        "msg",
        "OCC::OwncloudWizard::LogType",
        "slotCurrentPageChanged",
        "successfulStep",
        "slotCustomButtonClicked",
        "which",
        "adjustWizardSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clearPendingRequests'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'determineAuthType'
        QtMocHelpers::SignalData<void(const QUrl &, const OCC::WizardProxySettingsDialog::WizardProxySettings &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'connectToOCUrl'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'createLocalAndRemoteFolders'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'basicSetupFinished'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'skipFolderConfiguration'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'needCertificate'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'styleChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onActivate'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wizardClosed'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setAuthType'
        QtMocHelpers::SlotData<void(OCC::DetermineAuthTypeJob::AuthType)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'setRemoteFolder'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'appendToConfigurationLog'
        QtMocHelpers::SlotData<void(const QString &, OCC::OwncloudWizard::LogType)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { 0x80000000 | 22, 18 },
        }}),
        // Slot 'appendToConfigurationLog'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'slotCurrentPageChanged'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'successfulStep'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCustomButtonClicked'
        QtMocHelpers::SlotData<void(const int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'adjustWizardSize'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OwncloudWizard, qt_meta_tag_ZN3OCC14OwncloudWizardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::OwncloudWizard::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizard::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14OwncloudWizardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14OwncloudWizardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14OwncloudWizardE_t>.metaTypes,
    nullptr
} };

void OCC::OwncloudWizard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OwncloudWizard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearPendingRequests(); break;
        case 1: _t->determineAuthType((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::WizardProxySettingsDialog::WizardProxySettings>>(_a[2]))); break;
        case 2: _t->connectToOCUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->createLocalAndRemoteFolders((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->basicSetupFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->skipFolderConfiguration(); break;
        case 6: _t->needCertificate(); break;
        case 7: _t->styleChanged(); break;
        case 8: _t->onActivate(); break;
        case 9: _t->wizardClosed(); break;
        case 10: _t->setAuthType((*reinterpret_cast<std::add_pointer_t<OCC::DetermineAuthTypeJob::AuthType>>(_a[1]))); break;
        case 11: _t->setRemoteFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->appendToConfigurationLog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::OwncloudWizard::LogType>>(_a[2]))); break;
        case 13: _t->appendToConfigurationLog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->slotCurrentPageChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->successfulStep(); break;
        case 16: _t->slotCustomButtonClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->adjustWizardSize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)()>(_a, &OwncloudWizard::clearPendingRequests, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)(const QUrl & , const OCC::WizardProxySettingsDialog::WizardProxySettings & )>(_a, &OwncloudWizard::determineAuthType, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)(const QString & )>(_a, &OwncloudWizard::connectToOCUrl, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)(const QString & , const QString & )>(_a, &OwncloudWizard::createLocalAndRemoteFolders, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)(int )>(_a, &OwncloudWizard::basicSetupFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)()>(_a, &OwncloudWizard::skipFolderConfiguration, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)()>(_a, &OwncloudWizard::needCertificate, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)()>(_a, &OwncloudWizard::styleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)()>(_a, &OwncloudWizard::onActivate, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (OwncloudWizard::*)()>(_a, &OwncloudWizard::wizardClosed, 9))
            return;
    }
}

const QMetaObject *OCC::OwncloudWizard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::OwncloudWizard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14OwncloudWizardE_t>.strings))
        return static_cast<void*>(this);
    return QWizard::qt_metacast(_clname);
}

int OCC::OwncloudWizard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizard::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void OCC::OwncloudWizard::clearPendingRequests()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::OwncloudWizard::determineAuthType(const QUrl & _t1, const OCC::WizardProxySettingsDialog::WizardProxySettings & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::OwncloudWizard::connectToOCUrl(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::OwncloudWizard::createLocalAndRemoteFolders(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::OwncloudWizard::basicSetupFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::OwncloudWizard::skipFolderConfiguration()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::OwncloudWizard::needCertificate()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::OwncloudWizard::styleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OCC::OwncloudWizard::onActivate()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OCC::OwncloudWizard::wizardClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
