/****************************************************************************
** Meta object code from reading C++ file 'syncengine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/syncengine.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC10SyncEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncEngine::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10SyncEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncEngine",
        "rootEtag",
        "",
        "rootFileIdReceived",
        "fileId",
        "aboutToPropagate",
        "OCC::SyncFileItemVector&",
        "itemCompleted",
        "OCC::SyncFileItemPtr",
        "item",
        "OCC::ErrorCategory",
        "category",
        "transmissionProgress",
        "OCC::ProgressInfo",
        "progress",
        "itemDiscovered",
        "syncError",
        "message",
        "addErrorToGui",
        "OCC::SyncFileItem::Status",
        "status",
        "errorMessage",
        "subject",
        "finished",
        "success",
        "started",
        "aboutToRemoveAllFiles",
        "OCC::SyncFileItem::Direction",
        "direction",
        "std::function<void(bool)>",
        "f",
        "newBigFolder",
        "folder",
        "isExternal",
        "existingFolderNowBig",
        "seenLockedFile",
        "fileName",
        "lockFileDetected",
        "lockFile",
        "setSingleItemDiscoveryOptions",
        "OCC::SyncEngine::SingleItemDiscoveryOptions",
        "singleItemDiscoveryOptions",
        "startSync",
        "abort",
        "setNetworkLimits",
        "upload",
        "download",
        "setSyncOptions",
        "OCC::SyncOptions",
        "options",
        "setIgnoreHiddenFiles",
        "ignore",
        "setLocalDiscoveryOptions",
        "OCC::LocalDiscoveryEnums::LocalDiscoveryStyle",
        "style",
        "std::set<QString>",
        "paths",
        "addAcceptedInvalidFileName",
        "filePath",
        "setLocalDiscoveryEnforceWindowsFileNameCompatibility",
        "value",
        "slotFolderDiscovered",
        "local",
        "slotRootEtagReceived",
        "time",
        "slotRootFileIdReceived",
        "slotItemDiscovered",
        "slotNewItem",
        "slotItemCompleted",
        "slotDiscoveryFinished",
        "slotPropagationFinished",
        "slotProgress",
        "OCC::SyncFileItem",
        "current",
        "slotCleanPollsJobAborted",
        "error",
        "detectFileLock",
        "slotAddTouchedFile",
        "fn",
        "slotClearTouchedFiles",
        "slotSummaryError",
        "slotInsufficientLocalStorage",
        "slotInsufficientRemoteStorage",
        "slotScheduleFilesDelayedSync",
        "slotUnscheduleFilesDelayedSync",
        "slotCleanupScheduledSyncTimers",
        "remnantReadOnlyFolderDiscovered"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rootEtag'
        QtMocHelpers::SignalData<void(const QByteArray &, const QDateTime &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QDateTime, 2 },
        }}),
        // Signal 'rootFileIdReceived'
        QtMocHelpers::SignalData<void(qint64)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 4 },
        }}),
        // Signal 'aboutToPropagate'
        QtMocHelpers::SignalData<void(OCC::SyncFileItemVector &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'itemCompleted'
        QtMocHelpers::SignalData<void(const OCC::SyncFileItemPtr &, const OCC::ErrorCategory)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'transmissionProgress'
        QtMocHelpers::SignalData<void(const OCC::ProgressInfo &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'itemDiscovered'
        QtMocHelpers::SignalData<void(const OCC::SyncFileItemPtr &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'syncError'
        QtMocHelpers::SignalData<void(const QString &, const OCC::ErrorCategory)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'addErrorToGui'
        QtMocHelpers::SignalData<void(const OCC::SyncFileItem::Status, const QString &, const QString &, const OCC::ErrorCategory)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 }, { QMetaType::QString, 22 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToRemoveAllFiles'
        QtMocHelpers::SignalData<void(OCC::SyncFileItem::Direction, std::function<void(bool)>)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Signal 'newBigFolder'
        QtMocHelpers::SignalData<void(const QString &, bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 }, { QMetaType::Bool, 33 },
        }}),
        // Signal 'existingFolderNowBig'
        QtMocHelpers::SignalData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Signal 'seenLockedFile'
        QtMocHelpers::SignalData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
        // Signal 'lockFileDetected'
        QtMocHelpers::SignalData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Slot 'setSingleItemDiscoveryOptions'
        QtMocHelpers::SlotData<void(const OCC::SyncEngine::SingleItemDiscoveryOptions &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'startSync'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'abort'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setNetworkLimits'
        QtMocHelpers::SlotData<void(int, int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 45 }, { QMetaType::Int, 46 },
        }}),
        // Slot 'setSyncOptions'
        QtMocHelpers::SlotData<void(const OCC::SyncOptions &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'setIgnoreHiddenFiles'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'setLocalDiscoveryOptions'
        QtMocHelpers::SlotData<void(OCC::LocalDiscoveryEnums::LocalDiscoveryStyle, std::set<QString>)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 }, { 0x80000000 | 55, 56 },
        }}),
        // Slot 'setLocalDiscoveryOptions'
        QtMocHelpers::SlotData<void(OCC::LocalDiscoveryEnums::LocalDiscoveryStyle)>(52, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'addAcceptedInvalidFileName'
        QtMocHelpers::SlotData<void(const QString &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 58 },
        }}),
        // Slot 'setLocalDiscoveryEnforceWindowsFileNameCompatibility'
        QtMocHelpers::SlotData<void(bool)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotFolderDiscovered'
        QtMocHelpers::SlotData<void(bool, const QString &)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 62 }, { QMetaType::QString, 32 },
        }}),
        // Slot 'slotRootEtagReceived'
        QtMocHelpers::SlotData<void(const QByteArray &, const QDateTime &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QDateTime, 64 },
        }}),
        // Slot 'slotRootFileIdReceived'
        QtMocHelpers::SlotData<void(qint64)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 4 },
        }}),
        // Slot 'slotItemDiscovered'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotNewItem'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &, const OCC::ErrorCategory)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotDiscoveryFinished'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPropagationFinished'
        QtMocHelpers::SlotData<void(OCC::SyncFileItem::Status)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotProgress'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItem &, qint64)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 72, 9 }, { QMetaType::LongLong, 73 },
        }}),
        // Slot 'slotCleanPollsJobAborted'
        QtMocHelpers::SlotData<void(const QString &, const OCC::ErrorCategory)>(74, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 75 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'detectFileLock'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(76, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotAddTouchedFile'
        QtMocHelpers::SlotData<void(const QString &)>(77, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 78 },
        }}),
        // Slot 'slotClearTouchedFiles'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSummaryError'
        QtMocHelpers::SlotData<void(const QString &)>(80, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'slotInsufficientLocalStorage'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInsufficientRemoteStorage'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotScheduleFilesDelayedSync'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUnscheduleFilesDelayedSync'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCleanupScheduledSyncTimers'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'remnantReadOnlyFolderDiscovered'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(86, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SyncEngine, qt_meta_tag_ZN3OCC10SyncEngineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.metaTypes,
    nullptr
} };

void OCC::SyncEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rootEtag((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 1: _t->rootFileIdReceived((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 2: _t->aboutToPropagate((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemVector&>>(_a[1]))); break;
        case 3: _t->itemCompleted((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 4: _t->transmissionProgress((*reinterpret_cast<std::add_pointer_t<OCC::ProgressInfo>>(_a[1]))); break;
        case 5: _t->itemDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 6: _t->syncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 7: _t->addErrorToGui((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[4]))); break;
        case 8: _t->finished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->started(); break;
        case 10: _t->aboutToRemoveAllFiles((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Direction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::function<void(bool)>>>(_a[2]))); break;
        case 11: _t->newBigFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->existingFolderNowBig((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->seenLockedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->lockFileDetected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->setSingleItemDiscoveryOptions((*reinterpret_cast<std::add_pointer_t<OCC::SyncEngine::SingleItemDiscoveryOptions>>(_a[1]))); break;
        case 16: _t->startSync(); break;
        case 17: _t->abort(); break;
        case 18: _t->setNetworkLimits((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 19: _t->setSyncOptions((*reinterpret_cast<std::add_pointer_t<OCC::SyncOptions>>(_a[1]))); break;
        case 20: _t->setIgnoreHiddenFiles((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setLocalDiscoveryOptions((*reinterpret_cast<std::add_pointer_t<OCC::LocalDiscoveryEnums::LocalDiscoveryStyle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::set<QString>>>(_a[2]))); break;
        case 22: _t->setLocalDiscoveryOptions((*reinterpret_cast<std::add_pointer_t<OCC::LocalDiscoveryEnums::LocalDiscoveryStyle>>(_a[1]))); break;
        case 23: _t->addAcceptedInvalidFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->setLocalDiscoveryEnforceWindowsFileNameCompatibility((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->slotFolderDiscovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 26: _t->slotRootEtagReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 27: _t->slotRootFileIdReceived((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 28: _t->slotItemDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 29: _t->slotNewItem((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 30: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 31: _t->slotDiscoveryFinished(); break;
        case 32: _t->slotPropagationFinished((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[1]))); break;
        case 33: _t->slotProgress((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 34: _t->slotCleanPollsJobAborted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 35: _t->detectFileLock((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 36: _t->slotAddTouchedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->slotClearTouchedFiles(); break;
        case 38: _t->slotSummaryError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->slotInsufficientLocalStorage(); break;
        case 40: _t->slotInsufficientRemoteStorage(); break;
        case 41: _t->slotScheduleFilesDelayedSync(); break;
        case 42: _t->slotUnscheduleFilesDelayedSync(); break;
        case 43: _t->slotCleanupScheduledSyncTimers(); break;
        case 44: _t->remnantReadOnlyFolderDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItem >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 44:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QByteArray & , const QDateTime & )>(_a, &SyncEngine::rootEtag, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(qint64 )>(_a, &SyncEngine::rootFileIdReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(OCC::SyncFileItemVector & )>(_a, &SyncEngine::aboutToPropagate, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const OCC::SyncFileItemPtr & , const OCC::ErrorCategory )>(_a, &SyncEngine::itemCompleted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const OCC::ProgressInfo & )>(_a, &SyncEngine::transmissionProgress, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const OCC::SyncFileItemPtr & )>(_a, &SyncEngine::itemDiscovered, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & , const OCC::ErrorCategory )>(_a, &SyncEngine::syncError, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const OCC::SyncFileItem::Status , const QString & , const QString & , const OCC::ErrorCategory )>(_a, &SyncEngine::addErrorToGui, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(bool )>(_a, &SyncEngine::finished, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)()>(_a, &SyncEngine::started, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(OCC::SyncFileItem::Direction , std::function<void(bool)> )>(_a, &SyncEngine::aboutToRemoveAllFiles, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & , bool )>(_a, &SyncEngine::newBigFolder, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & )>(_a, &SyncEngine::existingFolderNowBig, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & )>(_a, &SyncEngine::seenLockedFile, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncEngine::*)(const QString & )>(_a, &SyncEngine::lockFileDetected, 14))
            return;
    }
}

const QMetaObject *OCC::SyncEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::SyncEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10SyncEngineE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::SyncEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    return _id;
}

// SIGNAL 0
void OCC::SyncEngine::rootEtag(const QByteArray & _t1, const QDateTime & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::SyncEngine::rootFileIdReceived(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::SyncEngine::aboutToPropagate(OCC::SyncFileItemVector & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::SyncEngine::itemCompleted(const OCC::SyncFileItemPtr & _t1, const OCC::ErrorCategory _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::SyncEngine::transmissionProgress(const OCC::ProgressInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::SyncEngine::itemDiscovered(const OCC::SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void OCC::SyncEngine::syncError(const QString & _t1, const OCC::ErrorCategory _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void OCC::SyncEngine::addErrorToGui(const OCC::SyncFileItem::Status _t1, const QString & _t2, const QString & _t3, const OCC::ErrorCategory _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 8
void OCC::SyncEngine::finished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void OCC::SyncEngine::started()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void OCC::SyncEngine::aboutToRemoveAllFiles(OCC::SyncFileItem::Direction _t1, std::function<void(bool)> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void OCC::SyncEngine::newBigFolder(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void OCC::SyncEngine::existingFolderNowBig(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void OCC::SyncEngine::seenLockedFile(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void OCC::SyncEngine::lockFileDetected(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}
QT_WARNING_POP
