/****************************************************************************
** Meta object code from reading C++ file 'syncconflictsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/syncconflictsmodel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncconflictsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC18SyncConflictsModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncConflictsModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC18SyncConflictsModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncConflictsModel",
        "conflictActivitiesChanged",
        "",
        "allExistingsSelectedChanged",
        "allConflictingSelectedChanged",
        "setConflictActivities",
        "OCC::ActivityList",
        "conflicts",
        "selectAllExisting",
        "selected",
        "selectAllConflicting",
        "applySolution",
        "conflictActivities",
        "allExistingsSelected",
        "allConflictingSelected",
        "SyncConflictRoles",
        "ExistingFileName",
        "ExistingSize",
        "ConflictSize",
        "ExistingDate",
        "ConflictDate",
        "ExistingSelected",
        "ConflictSelected",
        "ExistingPreviewUrl",
        "ConflictPreviewUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'conflictActivitiesChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allExistingsSelectedChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allConflictingSelectedChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setConflictActivities'
        QtMocHelpers::SlotData<void(OCC::ActivityList)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'selectAllExisting'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'selectAllConflicting'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'applySolution'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'conflictActivities'
        QtMocHelpers::PropertyData<OCC::ActivityList>(12, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'allExistingsSelected'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'allConflictingSelected'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SyncConflictRoles'
        QtMocHelpers::EnumData<enum SyncConflictRoles>(15, 15, QMC::EnumIsScoped).add({
            {   16, SyncConflictRoles::ExistingFileName },
            {   17, SyncConflictRoles::ExistingSize },
            {   18, SyncConflictRoles::ConflictSize },
            {   19, SyncConflictRoles::ExistingDate },
            {   20, SyncConflictRoles::ConflictDate },
            {   21, SyncConflictRoles::ExistingSelected },
            {   22, SyncConflictRoles::ConflictSelected },
            {   23, SyncConflictRoles::ExistingPreviewUrl },
            {   24, SyncConflictRoles::ConflictPreviewUrl },
        }),
    };
    return QtMocHelpers::metaObjectData<SyncConflictsModel, qt_meta_tag_ZN3OCC18SyncConflictsModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC18SyncConflictsModelE[] = {
    QMetaObject::SuperData::link<OCC::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::SyncConflictsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18SyncConflictsModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18SyncConflictsModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC18SyncConflictsModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC18SyncConflictsModelE_t>.metaTypes,
    nullptr
} };

void OCC::SyncConflictsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncConflictsModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->conflictActivitiesChanged(); break;
        case 1: _t->allExistingsSelectedChanged(); break;
        case 2: _t->allConflictingSelectedChanged(); break;
        case 3: _t->setConflictActivities((*reinterpret_cast<std::add_pointer_t<OCC::ActivityList>>(_a[1]))); break;
        case 4: _t->selectAllExisting((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->selectAllConflicting((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->applySolution(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ActivityList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SyncConflictsModel::*)()>(_a, &SyncConflictsModel::conflictActivitiesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncConflictsModel::*)()>(_a, &SyncConflictsModel::allExistingsSelectedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncConflictsModel::*)()>(_a, &SyncConflictsModel::allConflictingSelectedChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< OCC::ActivityList >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<OCC::ActivityList*>(_v) = _t->conflictActivities(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->allExistingsSelected(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->allConflictingSelected(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setConflictActivities(*reinterpret_cast<OCC::ActivityList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::SyncConflictsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::SyncConflictsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18SyncConflictsModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::SyncConflictsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void OCC::SyncConflictsModel::conflictActivitiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::SyncConflictsModel::allExistingsSelectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::SyncConflictsModel::allConflictingSelectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
