/****************************************************************************
** Meta object code from reading C++ file 'folderman.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/folderman.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folderman.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC9FolderManE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderMan::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9FolderManE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderMan",
        "folderSyncStateChange",
        "",
        "OCC::Folder*",
        "scheduleQueueChanged",
        "folderListChanged",
        "OCC::Folder::Map",
        "wipeDone",
        "OCC::AccountState*",
        "account",
        "success",
        "slotAccountStateChanged",
        "slotScheduleAppRestart",
        "slotSyncOnceFileUnlocks",
        "path",
        "slotScheduleETagJob",
        "alias",
        "OCC::RequestEtagJob*",
        "job",
        "slotWipeFolderForAccount",
        "accountState",
        "forceSyncForFolder",
        "folder",
        "removeE2eFiles",
        "OCC::AccountPtr",
        "slotFolderSyncPaused",
        "paused",
        "slotFolderCanSyncChanged",
        "slotFolderSyncStarted",
        "slotFolderSyncFinished",
        "OCC::SyncResult",
        "slotRunOneEtagJob",
        "slotEtagJobDestroyed",
        "slotStartScheduledFolderSync",
        "slotEtagPollTimerTimeout",
        "slotAccountRemoved",
        "slotRemoveFoldersForAccount",
        "slotForwardFolderSyncStateChange",
        "slotServerVersionChanged",
        "slotWatchedFileUnlocked",
        "slotScheduleFolderByTime",
        "slotSetupPushNotifications",
        "slotProcessFilesPushNotification",
        "OCC::Account*",
        "slotProcessFileIdsPushNotification",
        "QList<qint64>",
        "fileIds",
        "slotConnectToPushNotifications",
        "slotLeaveShare",
        "localFile",
        "folderToken"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'folderSyncStateChange'
        QtMocHelpers::SignalData<void(OCC::Folder *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'scheduleQueueChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'folderListChanged'
        QtMocHelpers::SignalData<void(const OCC::Folder::Map &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'wipeDone'
        QtMocHelpers::SignalData<void(OCC::AccountState *, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Slot 'slotAccountStateChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotScheduleAppRestart'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncOnceFileUnlocks'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'slotScheduleETagJob'
        QtMocHelpers::SlotData<void(const QString &, OCC::RequestEtagJob *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotWipeFolderForAccount'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 20 },
        }}),
        // Slot 'forceSyncForFolder'
        QtMocHelpers::SlotData<void(OCC::Folder *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'removeE2eFiles'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &) const>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 9 },
        }}),
        // Slot 'slotFolderSyncPaused'
        QtMocHelpers::SlotData<void(OCC::Folder *, bool)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 26 },
        }}),
        // Slot 'slotFolderCanSyncChanged'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFolderSyncStarted'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFolderSyncFinished'
        QtMocHelpers::SlotData<void(const OCC::SyncResult &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 2 },
        }}),
        // Slot 'slotRunOneEtagJob'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEtagJobDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 2 },
        }}),
        // Slot 'slotStartScheduledFolderSync'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEtagPollTimerTimeout'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAccountRemoved'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 20 },
        }}),
        // Slot 'slotRemoveFoldersForAccount'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 20 },
        }}),
        // Slot 'slotForwardFolderSyncStateChange'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotServerVersionChanged'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 9 },
        }}),
        // Slot 'slotWatchedFileUnlocked'
        QtMocHelpers::SlotData<void(const QString &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'slotScheduleFolderByTime'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetupPushNotifications'
        QtMocHelpers::SlotData<void(const OCC::Folder::Map &)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotProcessFilesPushNotification'
        QtMocHelpers::SlotData<void(OCC::Account *)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 9 },
        }}),
        // Slot 'slotProcessFileIdsPushNotification'
        QtMocHelpers::SlotData<void(OCC::Account *, const QList<qint64> &)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 9 }, { 0x80000000 | 45, 46 },
        }}),
        // Slot 'slotConnectToPushNotifications'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 9 },
        }}),
        // Slot 'slotLeaveShare'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 49 }, { QMetaType::QByteArray, 50 },
        }}),
        // Slot 'slotLeaveShare'
        QtMocHelpers::SlotData<void(const QString &)>(48, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderMan, qt_meta_tag_ZN3OCC9FolderManE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderMan::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9FolderManE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9FolderManE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9FolderManE_t>.metaTypes,
    nullptr
} };

void OCC::FolderMan::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderMan *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->folderSyncStateChange((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1]))); break;
        case 1: _t->scheduleQueueChanged(); break;
        case 2: _t->folderListChanged((*reinterpret_cast<std::add_pointer_t<OCC::Folder::Map>>(_a[1]))); break;
        case 3: _t->wipeDone((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->slotAccountStateChanged(); break;
        case 5: _t->slotScheduleAppRestart(); break;
        case 6: _t->slotSyncOnceFileUnlocks((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->slotScheduleETagJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::RequestEtagJob*>>(_a[2]))); break;
        case 8: _t->slotWipeFolderForAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 9: _t->forceSyncForFolder((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1]))); break;
        case 10: _t->removeE2eFiles((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 11: _t->slotFolderSyncPaused((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->slotFolderCanSyncChanged(); break;
        case 13: _t->slotFolderSyncStarted(); break;
        case 14: _t->slotFolderSyncFinished((*reinterpret_cast<std::add_pointer_t<OCC::SyncResult>>(_a[1]))); break;
        case 15: _t->slotRunOneEtagJob(); break;
        case 16: _t->slotEtagJobDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 17: _t->slotStartScheduledFolderSync(); break;
        case 18: _t->slotEtagPollTimerTimeout(); break;
        case 19: _t->slotAccountRemoved((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 20: _t->slotRemoveFoldersForAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 21: _t->slotForwardFolderSyncStateChange(); break;
        case 22: _t->slotServerVersionChanged((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 23: _t->slotWatchedFileUnlocked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->slotScheduleFolderByTime(); break;
        case 25: _t->slotSetupPushNotifications((*reinterpret_cast<std::add_pointer_t<OCC::Folder::Map>>(_a[1]))); break;
        case 26: _t->slotProcessFilesPushNotification((*reinterpret_cast<std::add_pointer_t<OCC::Account*>>(_a[1]))); break;
        case 27: _t->slotProcessFileIdsPushNotification((*reinterpret_cast<std::add_pointer_t<OCC::Account*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<qint64>>>(_a[2]))); break;
        case 28: _t->slotConnectToPushNotifications((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 29: _t->slotLeaveShare((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 30: _t->slotLeaveShare((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::RequestEtagJob* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<qint64> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(OCC::Folder * )>(_a, &FolderMan::folderSyncStateChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)()>(_a, &FolderMan::scheduleQueueChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(const OCC::Folder::Map & )>(_a, &FolderMan::folderListChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderMan::*)(OCC::AccountState * , bool )>(_a, &FolderMan::wipeDone, 3))
            return;
    }
}

const QMetaObject *OCC::FolderMan::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderMan::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9FolderManE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::FolderMan::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void OCC::FolderMan::folderSyncStateChange(OCC::Folder * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::FolderMan::scheduleQueueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::FolderMan::folderListChanged(const OCC::Folder::Map & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::FolderMan::wipeDone(OCC::AccountState * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
