/****************************************************************************
** Meta object code from reading C++ file 'shareemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/filedetails/shareemodel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shareemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC11ShareeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ShareeModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC11ShareeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ShareeModel",
        "accountStateChanged",
        "",
        "shareItemIsFolderChanged",
        "searchStringChanged",
        "fetchOngoingChanged",
        "lookupModeChanged",
        "shareeBlocklistChanged",
        "shareesReady",
        "displayErrorMessage",
        "code",
        "message",
        "setAccountState",
        "OCC::AccountState*",
        "accountState",
        "setShareItemIsFolder",
        "shareItemIsFolder",
        "setSearchString",
        "searchString",
        "setLookupMode",
        "OCC::ShareeModel::LookupMode",
        "lookupMode",
        "setShareeBlocklist",
        "QVariantList",
        "shareeBlocklist",
        "searchGlobally",
        "fetch",
        "shareesFetched",
        "QJsonDocument",
        "reply",
        "insertSearchGloballyItem",
        "QList<OCC::ShareePtr>",
        "newShareesFetched",
        "filterSharees",
        "slotDarkModeChanged",
        "AccountState*",
        "fetchOngoing",
        "LookupMode",
        "LocalSearch",
        "GlobalSearch",
        "Roles",
        "ShareeRole",
        "AutoCompleterStringMatchRole",
        "TypeRole",
        "IconRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountStateChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareItemIsFolderChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'searchStringChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fetchOngoingChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lookupModeChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareeBlocklistChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareesReady'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displayErrorMessage'
        QtMocHelpers::SignalData<void(const int, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'setAccountState'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'setShareItemIsFolder'
        QtMocHelpers::SlotData<void(const bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setSearchString'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'setLookupMode'
        QtMocHelpers::SlotData<void(const OCC::ShareeModel::LookupMode)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'setShareeBlocklist'
        QtMocHelpers::SlotData<void(const QVariantList)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'searchGlobally'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetch'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shareesFetched'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'insertSearchGloballyItem'
        QtMocHelpers::SlotData<void(const QVector<OCC::ShareePtr> &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'filterSharees'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDarkModeChanged'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accountState'
        QtMocHelpers::PropertyData<AccountState*>(14, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'shareItemIsFolder'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'searchString'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'fetchOngoing'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'lookupMode'
        QtMocHelpers::PropertyData<enum LookupMode>(21, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'shareeBlocklist'
        QtMocHelpers::PropertyData<QVariantList>(24, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LookupMode'
        QtMocHelpers::EnumData<enum LookupMode>(37, 37, QMC::EnumIsScoped).add({
            {   38, LookupMode::LocalSearch },
            {   39, LookupMode::GlobalSearch },
        }),
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(40, 40, QMC::EnumFlags{}).add({
            {   41, Roles::ShareeRole },
            {   42, Roles::AutoCompleterStringMatchRole },
            {   43, Roles::TypeRole },
            {   44, Roles::IconRole },
        }),
    };
    return QtMocHelpers::metaObjectData<ShareeModel, qt_meta_tag_ZN3OCC11ShareeModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ShareeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ShareeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ShareeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC11ShareeModelE_t>.metaTypes,
    nullptr
} };

void OCC::ShareeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShareeModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountStateChanged(); break;
        case 1: _t->shareItemIsFolderChanged(); break;
        case 2: _t->searchStringChanged(); break;
        case 3: _t->fetchOngoingChanged(); break;
        case 4: _t->lookupModeChanged(); break;
        case 5: _t->shareeBlocklistChanged(); break;
        case 6: _t->shareesReady(); break;
        case 7: _t->displayErrorMessage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->setAccountState((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 9: _t->setShareItemIsFolder((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setSearchString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->setLookupMode((*reinterpret_cast<std::add_pointer_t<OCC::ShareeModel::LookupMode>>(_a[1]))); break;
        case 12: _t->setShareeBlocklist((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 13: _t->searchGlobally(); break;
        case 14: _t->fetch(); break;
        case 15: _t->shareesFetched((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        case 16: _t->insertSearchGloballyItem((*reinterpret_cast<std::add_pointer_t<QList<OCC::ShareePtr>>>(_a[1]))); break;
        case 17: _t->filterSharees(); break;
        case 18: _t->slotDarkModeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::ShareePtr> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::accountStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::shareItemIsFolderChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::searchStringChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::fetchOngoingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::lookupModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::shareeBlocklistChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)()>(_a, &ShareeModel::shareesReady, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareeModel::*)(const int , const QString & )>(_a, &ShareeModel::displayErrorMessage, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AccountState* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountState**>(_v) = _t->accountState(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->shareItemIsFolder(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->searchString(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->fetchOngoing(); break;
        case 4: *reinterpret_cast<enum LookupMode*>(_v) = _t->lookupMode(); break;
        case 5: *reinterpret_cast<QVariantList*>(_v) = _t->shareeBlocklist(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAccountState(*reinterpret_cast<AccountState**>(_v)); break;
        case 1: _t->setShareItemIsFolder(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setSearchString(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setLookupMode(*reinterpret_cast<enum LookupMode*>(_v)); break;
        case 5: _t->setShareeBlocklist(*reinterpret_cast<QVariantList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::ShareeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ShareeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ShareeModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::ShareeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void OCC::ShareeModel::accountStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ShareeModel::shareItemIsFolderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::ShareeModel::searchStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::ShareeModel::fetchOngoingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::ShareeModel::lookupModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::ShareeModel::shareeBlocklistChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::ShareeModel::shareesReady()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::ShareeModel::displayErrorMessage(const int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
