/****************************************************************************
** Meta object code from reading C++ file 'discoveryphase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/discoveryphase.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'discoveryphase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::DiscoverySingleLocalDirectoryJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::DiscoverySingleLocalDirectoryJob",
        "finished",
        "",
        "QList<OCC::LocalInfo>",
        "result",
        "finishedFatalError",
        "errorString",
        "finishedNonFatalError",
        "itemDiscovered",
        "OCC::SyncFileItemPtr",
        "item",
        "childIgnored",
        "b"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void(QVector<OCC::LocalInfo>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'finishedFatalError'
        QtMocHelpers::SignalData<void(QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'finishedNonFatalError'
        QtMocHelpers::SignalData<void(QString)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'itemDiscovered'
        QtMocHelpers::SignalData<void(OCC::SyncFileItemPtr)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'childIgnored'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiscoverySingleLocalDirectoryJob, qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::DiscoverySingleLocalDirectoryJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.metaTypes,
    nullptr
} };

void OCC::DiscoverySingleLocalDirectoryJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiscoverySingleLocalDirectoryJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast<std::add_pointer_t<QList<OCC::LocalInfo>>>(_a[1]))); break;
        case 1: _t->finishedFatalError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->finishedNonFatalError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->itemDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 4: _t->childIgnored((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(QVector<OCC::LocalInfo> )>(_a, &DiscoverySingleLocalDirectoryJob::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(QString )>(_a, &DiscoverySingleLocalDirectoryJob::finishedFatalError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(QString )>(_a, &DiscoverySingleLocalDirectoryJob::finishedNonFatalError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(OCC::SyncFileItemPtr )>(_a, &DiscoverySingleLocalDirectoryJob::itemDiscovered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleLocalDirectoryJob::*)(bool )>(_a, &DiscoverySingleLocalDirectoryJob::childIgnored, 4))
            return;
    }
}

const QMetaObject *OCC::DiscoverySingleLocalDirectoryJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::DiscoverySingleLocalDirectoryJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32DiscoverySingleLocalDirectoryJobE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QRunnable"))
        return static_cast< QRunnable*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::DiscoverySingleLocalDirectoryJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::DiscoverySingleLocalDirectoryJob::finished(QVector<OCC::LocalInfo> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::DiscoverySingleLocalDirectoryJob::finishedFatalError(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::DiscoverySingleLocalDirectoryJob::finishedNonFatalError(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::DiscoverySingleLocalDirectoryJob::itemDiscovered(OCC::SyncFileItemPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::DiscoverySingleLocalDirectoryJob::childIgnored(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::DiscoverySingleDirectoryJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::DiscoverySingleDirectoryJob",
        "firstDirectoryPermissions",
        "",
        "OCC::RemotePermissions",
        "etag",
        "time",
        "finished",
        "OCC::HttpResult<QList<OCC::RemoteInfo>>",
        "result",
        "setfolderQuota",
        "OCC::FolderQuota",
        "folderQuota",
        "firstDirectoryFileId",
        "fileId",
        "directoryListingIteratedSlot",
        "QMap<QString,QString>",
        "lsJobFinishedWithoutErrorSlot",
        "lsJobFinishedWithErrorSlot",
        "QNetworkReply*",
        "reply",
        "fetchE2eMetadata",
        "metadataReceived",
        "QJsonDocument",
        "json",
        "statusCode",
        "metadataError",
        "httpReturnCode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'firstDirectoryPermissions'
        QtMocHelpers::SignalData<void(OCC::RemotePermissions)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'etag'
        QtMocHelpers::SignalData<void(const QByteArray &, const QDateTime &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QDateTime, 5 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(const OCC::HttpResult<QVector<OCC::RemoteInfo>> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'setfolderQuota'
        QtMocHelpers::SignalData<void(const OCC::FolderQuota &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'firstDirectoryFileId'
        QtMocHelpers::SignalData<void(qint64)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 13 },
        }}),
        // Slot 'directoryListingIteratedSlot'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { 0x80000000 | 15, 2 },
        }}),
        // Slot 'lsJobFinishedWithoutErrorSlot'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'lsJobFinishedWithErrorSlot'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'fetchE2eMetadata'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'metadataReceived'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'metadataError'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 13 }, { QMetaType::Int, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiscoverySingleDirectoryJob, qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::DiscoverySingleDirectoryJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.metaTypes,
    nullptr
} };

void OCC::DiscoverySingleDirectoryJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiscoverySingleDirectoryJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->firstDirectoryPermissions((*reinterpret_cast<std::add_pointer_t<OCC::RemotePermissions>>(_a[1]))); break;
        case 1: _t->etag((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 2: _t->finished((*reinterpret_cast<std::add_pointer_t<OCC::HttpResult<QList<OCC::RemoteInfo>>>>(_a[1]))); break;
        case 3: _t->setfolderQuota((*reinterpret_cast<std::add_pointer_t<OCC::FolderQuota>>(_a[1]))); break;
        case 4: _t->firstDirectoryFileId((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 5: _t->directoryListingIteratedSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 6: _t->lsJobFinishedWithoutErrorSlot(); break;
        case 7: _t->lsJobFinishedWithErrorSlot((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 8: _t->fetchE2eMetadata(); break;
        case 9: _t->metadataReceived((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->metadataError((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::RemotePermissions >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(OCC::RemotePermissions )>(_a, &DiscoverySingleDirectoryJob::firstDirectoryPermissions, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(const QByteArray & , const QDateTime & )>(_a, &DiscoverySingleDirectoryJob::etag, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(const OCC::HttpResult<QVector<OCC::RemoteInfo>> & )>(_a, &DiscoverySingleDirectoryJob::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(const OCC::FolderQuota & )>(_a, &DiscoverySingleDirectoryJob::setfolderQuota, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoverySingleDirectoryJob::*)(qint64 )>(_a, &DiscoverySingleDirectoryJob::firstDirectoryFileId, 4))
            return;
    }
}

const QMetaObject *OCC::DiscoverySingleDirectoryJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::DiscoverySingleDirectoryJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC27DiscoverySingleDirectoryJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::DiscoverySingleDirectoryJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OCC::DiscoverySingleDirectoryJob::firstDirectoryPermissions(OCC::RemotePermissions _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::DiscoverySingleDirectoryJob::etag(const QByteArray & _t1, const QDateTime & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::DiscoverySingleDirectoryJob::finished(const OCC::HttpResult<QVector<OCC::RemoteInfo>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::DiscoverySingleDirectoryJob::setfolderQuota(const OCC::FolderQuota & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::DiscoverySingleDirectoryJob::firstDirectoryFileId(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::DiscoveryPhase::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::DiscoveryPhase",
        "fatalError",
        "",
        "errorString",
        "OCC::ErrorCategory",
        "errorCategory",
        "itemDiscovered",
        "OCC::SyncFileItemPtr",
        "item",
        "finished",
        "newBigFolder",
        "folder",
        "isExternal",
        "existingFolderNowBig",
        "silentlyExcluded",
        "folderPath",
        "addErrorToGui",
        "OCC::SyncFileItem::Status",
        "status",
        "errorMessage",
        "subject",
        "category",
        "remnantReadOnlyFolderDiscovered",
        "slotItemDiscovered"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fatalError'
        QtMocHelpers::SignalData<void(const QString &, const OCC::ErrorCategory)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'itemDiscovered'
        QtMocHelpers::SignalData<void(const OCC::SyncFileItemPtr &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newBigFolder'
        QtMocHelpers::SignalData<void(const QString &, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'existingFolderNowBig'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'silentlyExcluded'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'addErrorToGui'
        QtMocHelpers::SignalData<void(const OCC::SyncFileItem::Status, const QString &, const QString &, const OCC::ErrorCategory)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 20 }, { 0x80000000 | 4, 21 },
        }}),
        // Signal 'remnantReadOnlyFolderDiscovered'
        QtMocHelpers::SignalData<void(const OCC::SyncFileItemPtr &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotItemDiscovered'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiscoveryPhase, qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::DiscoveryPhase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.metaTypes,
    nullptr
} };

void OCC::DiscoveryPhase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiscoveryPhase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fatalError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 1: _t->itemDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 2: _t->finished(); break;
        case 3: _t->newBigFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->existingFolderNowBig((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->silentlyExcluded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->addErrorToGui((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[4]))); break;
        case 7: _t->remnantReadOnlyFolderDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 8: _t->slotItemDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & , const OCC::ErrorCategory )>(_a, &DiscoveryPhase::fatalError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const OCC::SyncFileItemPtr & )>(_a, &DiscoveryPhase::itemDiscovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)()>(_a, &DiscoveryPhase::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & , bool )>(_a, &DiscoveryPhase::newBigFolder, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & )>(_a, &DiscoveryPhase::existingFolderNowBig, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const QString & )>(_a, &DiscoveryPhase::silentlyExcluded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const OCC::SyncFileItem::Status , const QString & , const QString & , const OCC::ErrorCategory )>(_a, &DiscoveryPhase::addErrorToGui, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiscoveryPhase::*)(const OCC::SyncFileItemPtr & )>(_a, &DiscoveryPhase::remnantReadOnlyFolderDiscovered, 7))
            return;
    }
}

const QMetaObject *OCC::DiscoveryPhase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::DiscoveryPhase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14DiscoveryPhaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::DiscoveryPhase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void OCC::DiscoveryPhase::fatalError(const QString & _t1, const OCC::ErrorCategory _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::DiscoveryPhase::itemDiscovered(const OCC::SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::DiscoveryPhase::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::DiscoveryPhase::newBigFolder(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::DiscoveryPhase::existingFolderNowBig(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::DiscoveryPhase::silentlyExcluded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void OCC::DiscoveryPhase::addErrorToGui(const OCC::SyncFileItem::Status _t1, const QString & _t2, const QString & _t3, const OCC::ErrorCategory _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 7
void OCC::DiscoveryPhase::remnantReadOnlyFolderDiscovered(const OCC::SyncFileItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3OCC19LocalDiscoveryEnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::LocalDiscoveryEnums::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19LocalDiscoveryEnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::LocalDiscoveryEnums",
        "LocalDiscoveryStyle",
        "FilesystemOnly",
        "DatabaseAndFilesystem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LocalDiscoveryStyle'
        QtMocHelpers::EnumData<LocalDiscoveryStyle>(1, 1, QMC::EnumIsScoped).add({
            {    2, LocalDiscoveryStyle::FilesystemOnly },
            {    3, LocalDiscoveryStyle::DatabaseAndFilesystem },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3OCC19LocalDiscoveryEnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN3OCC19LocalDiscoveryEnumsE =
    OCC::LocalDiscoveryEnums::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19LocalDiscoveryEnumsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3OCC19LocalDiscoveryEnumsE =
    qt_staticMetaObjectContent_ZN3OCC19LocalDiscoveryEnumsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3OCC19LocalDiscoveryEnumsE =
    qt_staticMetaObjectContent_ZN3OCC19LocalDiscoveryEnumsE.relocatingData;

Q_CONSTINIT const QMetaObject OCC::LocalDiscoveryEnums::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3OCC19LocalDiscoveryEnumsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3OCC19LocalDiscoveryEnumsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3OCC19LocalDiscoveryEnumsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
