/****************************************************************************
** Meta object code from reading C++ file 'folder.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/folder.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folder.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC6FolderE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Folder::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC6FolderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Folder",
        "syncStateChange",
        "",
        "syncStarted",
        "syncFinished",
        "OCC::SyncResult",
        "result",
        "progressInfo",
        "OCC::ProgressInfo",
        "progress",
        "newBigFolderDiscovered",
        "syncPausedChanged",
        "OCC::Folder*",
        "paused",
        "canSyncChanged",
        "watchedFileChangedExternally",
        "path",
        "slotTerminateSync",
        "slotAboutToRemoveAllFiles",
        "OCC::SyncFileItem::Direction",
        "std::function<void(bool)>",
        "callback",
        "startSync",
        "pathList",
        "slotDiscardDownloadProgress",
        "downloadInfoCount",
        "slotWipeErrorBlacklist",
        "errorBlackListEntryCount",
        "slotWatchedPathChanged",
        "QStringView",
        "OCC::Folder::ChangeReason",
        "reason",
        "slotFilesLockReleased",
        "QSet<QString>",
        "files",
        "slotFilesLockImposed",
        "slotLockedFilesFound",
        "implicitlyHydrateFile",
        "relativepath",
        "schedulePathForLocalDiscovery",
        "relativePath",
        "slotNextSyncFullLocalDiscovery",
        "setSilenceErrorsUntilNextSync",
        "silenceErrors",
        "removeLocalE2eFiles",
        "slotSyncStarted",
        "slotSyncFinished",
        "disconnectFolderWatcher",
        "slotSyncError",
        "message",
        "OCC::ErrorCategory",
        "category",
        "slotAddErrorToGui",
        "OCC::SyncFileItem::Status",
        "status",
        "errorMessage",
        "subject",
        "slotTransmissionProgress",
        "pi",
        "slotItemCompleted",
        "OCC::SyncFileItemPtr",
        "errorCategory",
        "slotRunEtagJob",
        "etagRetrieved",
        "tp",
        "etagRetrievedFromSyncEngine",
        "time",
        "rootFileIdReceivedFromSyncEngine",
        "fileId",
        "slotEmitFinishedDelayed",
        "slotNewBigFolderDiscovered",
        "isExternal",
        "slotExistingFolderNowBig",
        "folderPath",
        "slotLogPropagationStart",
        "slotScheduleThisFolder",
        "slotFolderConflicts",
        "folder",
        "conflictPaths",
        "warnOnNewExcludedItem",
        "OCC::SyncJournalFileRecord",
        "record",
        "slotWatcherUnreliable",
        "slotHydrationStarts",
        "slotHydrationDone",
        "slotHydrationFailed",
        "errorCode",
        "statusCode",
        "errorString",
        "fileName",
        "slotCapabilitiesChanged",
        "ChangeReason",
        "Other",
        "UnLock"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'syncStateChange'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncStarted'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncFinished'
        QtMocHelpers::SignalData<void(const OCC::SyncResult &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'progressInfo'
        QtMocHelpers::SignalData<void(const OCC::ProgressInfo &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'newBigFolderDiscovered'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'syncPausedChanged'
        QtMocHelpers::SignalData<void(OCC::Folder *, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 }, { QMetaType::Bool, 13 },
        }}),
        // Signal 'canSyncChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'watchedFileChangedExternally'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'slotTerminateSync'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAboutToRemoveAllFiles'
        QtMocHelpers::SlotData<void(OCC::SyncFileItem::Direction, std::function<void(bool)>)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'startSync'
        QtMocHelpers::SlotData<void(const QStringList &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 23 },
        }}),
        // Slot 'startSync'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotDiscardDownloadProgress'
        QtMocHelpers::SlotData<int()>(24, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'downloadInfoCount'
        QtMocHelpers::SlotData<int()>(25, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'slotWipeErrorBlacklist'
        QtMocHelpers::SlotData<int()>(26, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'errorBlackListEntryCount'
        QtMocHelpers::SlotData<int()>(27, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'slotWatchedPathChanged'
        QtMocHelpers::SlotData<void(const QStringView &, const OCC::Folder::ChangeReason)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 16 }, { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotFilesLockReleased'
        QtMocHelpers::SlotData<void(const QSet<QString> &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'slotFilesLockImposed'
        QtMocHelpers::SlotData<void(const QSet<QString> &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'slotLockedFilesFound'
        QtMocHelpers::SlotData<void(const QSet<QString> &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'implicitlyHydrateFile'
        QtMocHelpers::SlotData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Slot 'schedulePathForLocalDiscovery'
        QtMocHelpers::SlotData<void(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Slot 'slotNextSyncFullLocalDiscovery'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSilenceErrorsUntilNextSync'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'removeLocalE2eFiles'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncStarted'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSyncFinished'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'disconnectFolderWatcher'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSyncError'
        QtMocHelpers::SlotData<void(const QString &, OCC::ErrorCategory)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 49 }, { 0x80000000 | 50, 51 },
        }}),
        // Slot 'slotAddErrorToGui'
        QtMocHelpers::SlotData<void(OCC::SyncFileItem::Status, const QString &, const QString &, OCC::ErrorCategory)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 53, 54 }, { QMetaType::QString, 55 }, { QMetaType::QString, 56 }, { 0x80000000 | 50, 51 },
        }}),
        // Slot 'slotTransmissionProgress'
        QtMocHelpers::SlotData<void(const OCC::ProgressInfo &)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 58 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &, OCC::ErrorCategory)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 60, 2 }, { 0x80000000 | 50, 61 },
        }}),
        // Slot 'slotRunEtagJob'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'etagRetrieved'
        QtMocHelpers::SlotData<void(const QByteArray &, const QDateTime &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QDateTime, 64 },
        }}),
        // Slot 'etagRetrievedFromSyncEngine'
        QtMocHelpers::SlotData<void(const QByteArray &, const QDateTime &)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QDateTime, 66 },
        }}),
        // Slot 'rootFileIdReceivedFromSyncEngine'
        QtMocHelpers::SlotData<void(qint64)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 68 },
        }}),
        // Slot 'slotEmitFinishedDelayed'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewBigFolderDiscovered'
        QtMocHelpers::SlotData<void(const QString &, bool)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Bool, 71 },
        }}),
        // Slot 'slotExistingFolderNowBig'
        QtMocHelpers::SlotData<void(const QString &)>(72, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 73 },
        }}),
        // Slot 'slotLogPropagationStart'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotScheduleThisFolder'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFolderConflicts'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &)>(76, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 77 }, { QMetaType::QStringList, 78 },
        }}),
        // Slot 'warnOnNewExcludedItem'
        QtMocHelpers::SlotData<void(const OCC::SyncJournalFileRecord &, const QStringView &)>(79, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 80, 81 }, { 0x80000000 | 29, 16 },
        }}),
        // Slot 'slotWatcherUnreliable'
        QtMocHelpers::SlotData<void(const QString &)>(82, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'slotHydrationStarts'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHydrationDone'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHydrationFailed'
        QtMocHelpers::SlotData<void(int, int, const QString &, const QString &)>(85, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 86 }, { QMetaType::Int, 87 }, { QMetaType::QString, 88 }, { QMetaType::QString, 89 },
        }}),
        // Slot 'slotCapabilitiesChanged'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ChangeReason'
        QtMocHelpers::EnumData<enum ChangeReason>(91, 91, QMC::EnumIsScoped).add({
            {   92, ChangeReason::Other },
            {   93, ChangeReason::UnLock },
        }),
    };
    return QtMocHelpers::metaObjectData<Folder, qt_meta_tag_ZN3OCC6FolderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Folder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC6FolderE_t>.metaTypes,
    nullptr
} };

void OCC::Folder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Folder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->syncStateChange(); break;
        case 1: _t->syncStarted(); break;
        case 2: _t->syncFinished((*reinterpret_cast<std::add_pointer_t<OCC::SyncResult>>(_a[1]))); break;
        case 3: _t->progressInfo((*reinterpret_cast<std::add_pointer_t<OCC::ProgressInfo>>(_a[1]))); break;
        case 4: _t->newBigFolderDiscovered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->syncPausedChanged((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->canSyncChanged(); break;
        case 7: _t->watchedFileChangedExternally((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->slotTerminateSync(); break;
        case 9: _t->slotAboutToRemoveAllFiles((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Direction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::function<void(bool)>>>(_a[2]))); break;
        case 10: _t->startSync((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 11: _t->startSync(); break;
        case 12: { int _r = _t->slotDiscardDownloadProgress();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 13: { int _r = _t->downloadInfoCount();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->slotWipeErrorBlacklist();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: { int _r = _t->errorBlackListEntryCount();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->slotWatchedPathChanged((*reinterpret_cast<std::add_pointer_t<QStringView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::Folder::ChangeReason>>(_a[2]))); break;
        case 17: _t->slotFilesLockReleased((*reinterpret_cast<std::add_pointer_t<QSet<QString>>>(_a[1]))); break;
        case 18: _t->slotFilesLockImposed((*reinterpret_cast<std::add_pointer_t<QSet<QString>>>(_a[1]))); break;
        case 19: _t->slotLockedFilesFound((*reinterpret_cast<std::add_pointer_t<QSet<QString>>>(_a[1]))); break;
        case 20: _t->implicitlyHydrateFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->schedulePathForLocalDiscovery((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->slotNextSyncFullLocalDiscovery(); break;
        case 23: _t->setSilenceErrorsUntilNextSync((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->removeLocalE2eFiles(); break;
        case 25: _t->slotSyncStarted(); break;
        case 26: _t->slotSyncFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->disconnectFolderWatcher(); break;
        case 28: _t->slotSyncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 29: _t->slotAddErrorToGui((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[4]))); break;
        case 30: _t->slotTransmissionProgress((*reinterpret_cast<std::add_pointer_t<OCC::ProgressInfo>>(_a[1]))); break;
        case 31: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[2]))); break;
        case 32: _t->slotRunEtagJob(); break;
        case 33: _t->etagRetrieved((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 34: _t->etagRetrievedFromSyncEngine((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 35: _t->rootFileIdReceivedFromSyncEngine((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 36: _t->slotEmitFinishedDelayed(); break;
        case 37: _t->slotNewBigFolderDiscovered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 38: _t->slotExistingFolderNowBig((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->slotLogPropagationStart(); break;
        case 40: _t->slotScheduleThisFolder(); break;
        case 41: _t->slotFolderConflicts((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 42: _t->warnOnNewExcludedItem((*reinterpret_cast<std::add_pointer_t<OCC::SyncJournalFileRecord>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringView>>(_a[2]))); break;
        case 43: _t->slotWatcherUnreliable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 44: _t->slotHydrationStarts(); break;
        case 45: _t->slotHydrationDone(); break;
        case 46: _t->slotHydrationFailed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 47: _t->slotCapabilitiesChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QString> >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QString> >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QString> >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::syncStateChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::syncStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const OCC::SyncResult & )>(_a, &Folder::syncFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const OCC::ProgressInfo & )>(_a, &Folder::progressInfo, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const QString & )>(_a, &Folder::newBigFolderDiscovered, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(OCC::Folder * , bool )>(_a, &Folder::syncPausedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::canSyncChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const QString & )>(_a, &Folder::watchedFileChangedExternally, 7))
            return;
    }
}

const QMetaObject *OCC::Folder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Folder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Folder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    return _id;
}

// SIGNAL 0
void OCC::Folder::syncStateChange()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Folder::syncStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Folder::syncFinished(const OCC::SyncResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::Folder::progressInfo(const OCC::ProgressInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::Folder::newBigFolderDiscovered(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OCC::Folder::syncPausedChanged(OCC::Folder * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void OCC::Folder::canSyncChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::Folder::watchedFileChangedExternally(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
