/*
 * SPDX-FileCopyrightText: 2021 Nextcloud GmbH and Nextcloud contributors
 * SPDX-FileCopyrightText: 2013 ownCloud GmbH
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#include <QList>
#include <QString>
#include <QSslCertificate>
#include <QSslConfiguration>

#include "creds/credentialscommon.h"

#include "common/utility.h"
#include "account.h"
#include "syncengine.h"

namespace OCC {


} // namespace OCC
