/****************************************************************************
** Meta object code from reading C++ file 'csync.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/csync/csync.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'csync.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC21EncryptionStatusEnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::EncryptionStatusEnums::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC21EncryptionStatusEnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::EncryptionStatusEnums",
        "ItemEncryptionStatus",
        "NotEncrypted",
        "Encrypted",
        "EncryptedMigratedV1_2",
        "EncryptedMigratedV2_0",
        "JournalDbEncryptionStatus",
        "EncryptedMigratedV1_2Invalid"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ItemEncryptionStatus'
        QtMocHelpers::EnumData<ItemEncryptionStatus>(1, 1, QMC::EnumIsScoped).add({
            {    2, ItemEncryptionStatus::NotEncrypted },
            {    3, ItemEncryptionStatus::Encrypted },
            {    4, ItemEncryptionStatus::EncryptedMigratedV1_2 },
            {    5, ItemEncryptionStatus::EncryptedMigratedV2_0 },
        }),
        // enum 'JournalDbEncryptionStatus'
        QtMocHelpers::EnumData<JournalDbEncryptionStatus>(6, 6, QMC::EnumIsScoped).add({
            {    2, JournalDbEncryptionStatus::NotEncrypted },
            {    3, JournalDbEncryptionStatus::Encrypted },
            {    7, JournalDbEncryptionStatus::EncryptedMigratedV1_2Invalid },
            {    4, JournalDbEncryptionStatus::EncryptedMigratedV1_2 },
            {    5, JournalDbEncryptionStatus::EncryptedMigratedV2_0 },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3OCC21EncryptionStatusEnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN3OCC21EncryptionStatusEnumsE =
    OCC::EncryptionStatusEnums::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC21EncryptionStatusEnumsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3OCC21EncryptionStatusEnumsE =
    qt_staticMetaObjectContent_ZN3OCC21EncryptionStatusEnumsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3OCC21EncryptionStatusEnumsE =
    qt_staticMetaObjectContent_ZN3OCC21EncryptionStatusEnumsE.relocatingData;

Q_CONSTINIT const QMetaObject OCC::EncryptionStatusEnums::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3OCC21EncryptionStatusEnumsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3OCC21EncryptionStatusEnumsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3OCC21EncryptionStatusEnumsE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10CSyncEnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto CSyncEnums::qt_create_metaobjectdata<qt_meta_tag_ZN10CSyncEnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CSyncEnums",
        "csync_status_codes_e",
        "CSYNC_STATUS_OK",
        "CSYNC_STATUS_ERROR",
        "CSYNC_STATUS_UNSUCCESSFUL",
        "CSYNC_STATUS_STATEDB_LOAD_ERROR",
        "CSYNC_STATUS_UPDATE_ERROR",
        "CSYNC_STATUS_TIMEOUT",
        "CSYNC_STATUS_HTTP_ERROR",
        "CSYNC_STATUS_PERMISSION_DENIED",
        "CSYNC_STATUS_NOT_FOUND",
        "CSYNC_STATUS_FILE_EXISTS",
        "CSYNC_STATUS_OUT_OF_SPACE",
        "CSYNC_STATUS_SERVICE_UNAVAILABLE",
        "CSYNC_STATUS_STORAGE_UNAVAILABLE",
        "CSYNC_STATUS_FILE_SIZE_ERROR",
        "CSYNC_STATUS_OPENDIR_ERROR",
        "CSYNC_STATUS_READDIR_ERROR",
        "CSYNC_STATUS_OPEN_ERROR",
        "CSYNC_STATUS_ABORTED",
        "CSYNC_STATUS_INDIVIDUAL_IS_SYMLINK",
        "CSYNC_STATUS_INDIVIDUAL_IGNORE_LIST",
        "CSYNC_STATUS_INDIVIDUAL_IS_INVALID_CHARS",
        "CSYNC_STATUS_INDIVIDUAL_TRAILING_SPACE",
        "CSYNC_STATUS_INDIVIDUAL_EXCLUDE_LONG_FILENAME",
        "CSYNC_STATUS_INDIVIDUAL_EXCLUDE_HIDDEN",
        "CSYNC_STATUS_INVALID_CHARACTERS",
        "CSYNC_STATUS_INDIVIDUAL_STAT_FAILED",
        "CSYNC_STATUS_FORBIDDEN",
        "CSYNC_STATUS_INDIVIDUAL_TOO_DEEP",
        "CSYNC_STATUS_INDIVIDUAL_IS_CONFLICT_FILE",
        "CSYNC_STATUS_INDIVIDUAL_CANNOT_ENCODE",
        "SyncInstructions",
        "CSYNC_INSTRUCTION_NONE",
        "CSYNC_INSTRUCTION_EVAL",
        "CSYNC_INSTRUCTION_REMOVE",
        "CSYNC_INSTRUCTION_RENAME",
        "CSYNC_INSTRUCTION_EVAL_RENAME",
        "CSYNC_INSTRUCTION_NEW",
        "CSYNC_INSTRUCTION_CONFLICT",
        "CSYNC_INSTRUCTION_IGNORE",
        "CSYNC_INSTRUCTION_SYNC",
        "CSYNC_INSTRUCTION_STAT_ERROR",
        "CSYNC_INSTRUCTION_ERROR",
        "CSYNC_INSTRUCTION_TYPE_CHANGE",
        "CSYNC_INSTRUCTION_UPDATE_METADATA",
        "CSYNC_INSTRUCTION_CASE_CLASH_CONFLICT",
        "CSYNC_INSTRUCTION_UPDATE_VFS_METADATA",
        "CSYNC_INSTRUCTION_UPDATE_ENCRYPTION_METADATA",
        "ItemType",
        "ItemTypeFile",
        "ItemTypeSoftLink",
        "ItemTypeDirectory",
        "ItemTypeSkip",
        "ItemTypeVirtualFile",
        "ItemTypeVirtualFileDownload",
        "ItemTypeVirtualFileDehydration",
        "ItemTypeVirtualDirectory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'csync_status_codes_e'
        QtMocHelpers::EnumData<csync_status_codes_e>(1, 1, QMC::EnumFlags{}).add({
            {    2, csync_status_codes_e::CSYNC_STATUS_OK },
            {    3, csync_status_codes_e::CSYNC_STATUS_ERROR },
            {    4, csync_status_codes_e::CSYNC_STATUS_UNSUCCESSFUL },
            {    5, csync_status_codes_e::CSYNC_STATUS_STATEDB_LOAD_ERROR },
            {    6, csync_status_codes_e::CSYNC_STATUS_UPDATE_ERROR },
            {    7, csync_status_codes_e::CSYNC_STATUS_TIMEOUT },
            {    8, csync_status_codes_e::CSYNC_STATUS_HTTP_ERROR },
            {    9, csync_status_codes_e::CSYNC_STATUS_PERMISSION_DENIED },
            {   10, csync_status_codes_e::CSYNC_STATUS_NOT_FOUND },
            {   11, csync_status_codes_e::CSYNC_STATUS_FILE_EXISTS },
            {   12, csync_status_codes_e::CSYNC_STATUS_OUT_OF_SPACE },
            {   13, csync_status_codes_e::CSYNC_STATUS_SERVICE_UNAVAILABLE },
            {   14, csync_status_codes_e::CSYNC_STATUS_STORAGE_UNAVAILABLE },
            {   15, csync_status_codes_e::CSYNC_STATUS_FILE_SIZE_ERROR },
            {   16, csync_status_codes_e::CSYNC_STATUS_OPENDIR_ERROR },
            {   17, csync_status_codes_e::CSYNC_STATUS_READDIR_ERROR },
            {   18, csync_status_codes_e::CSYNC_STATUS_OPEN_ERROR },
            {   19, csync_status_codes_e::CSYNC_STATUS_ABORTED },
            {   20, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_IS_SYMLINK },
            {   21, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_IGNORE_LIST },
            {   22, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_IS_INVALID_CHARS },
            {   23, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_TRAILING_SPACE },
            {   24, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_EXCLUDE_LONG_FILENAME },
            {   25, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_EXCLUDE_HIDDEN },
            {   26, csync_status_codes_e::CSYNC_STATUS_INVALID_CHARACTERS },
            {   27, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_STAT_FAILED },
            {   28, csync_status_codes_e::CSYNC_STATUS_FORBIDDEN },
            {   29, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_TOO_DEEP },
            {   30, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_IS_CONFLICT_FILE },
            {   31, csync_status_codes_e::CSYNC_STATUS_INDIVIDUAL_CANNOT_ENCODE },
        }),
        // enum 'SyncInstructions'
        QtMocHelpers::EnumData<SyncInstructions>(32, 32, QMC::EnumFlags{}).add({
            {   33, SyncInstructions::CSYNC_INSTRUCTION_NONE },
            {   34, SyncInstructions::CSYNC_INSTRUCTION_EVAL },
            {   35, SyncInstructions::CSYNC_INSTRUCTION_REMOVE },
            {   36, SyncInstructions::CSYNC_INSTRUCTION_RENAME },
            {   37, SyncInstructions::CSYNC_INSTRUCTION_EVAL_RENAME },
            {   38, SyncInstructions::CSYNC_INSTRUCTION_NEW },
            {   39, SyncInstructions::CSYNC_INSTRUCTION_CONFLICT },
            {   40, SyncInstructions::CSYNC_INSTRUCTION_IGNORE },
            {   41, SyncInstructions::CSYNC_INSTRUCTION_SYNC },
            {   42, SyncInstructions::CSYNC_INSTRUCTION_STAT_ERROR },
            {   43, SyncInstructions::CSYNC_INSTRUCTION_ERROR },
            {   44, SyncInstructions::CSYNC_INSTRUCTION_TYPE_CHANGE },
            {   45, SyncInstructions::CSYNC_INSTRUCTION_UPDATE_METADATA },
            {   46, SyncInstructions::CSYNC_INSTRUCTION_CASE_CLASH_CONFLICT },
            {   47, SyncInstructions::CSYNC_INSTRUCTION_UPDATE_VFS_METADATA },
            {   48, SyncInstructions::CSYNC_INSTRUCTION_UPDATE_ENCRYPTION_METADATA },
        }),
        // enum 'ItemType'
        QtMocHelpers::EnumData<ItemType>(49, 49, QMC::EnumFlags{}).add({
            {   50, ItemType::ItemTypeFile },
            {   51, ItemType::ItemTypeSoftLink },
            {   52, ItemType::ItemTypeDirectory },
            {   53, ItemType::ItemTypeSkip },
            {   54, ItemType::ItemTypeVirtualFile },
            {   55, ItemType::ItemTypeVirtualFileDownload },
            {   56, ItemType::ItemTypeVirtualFileDehydration },
            {   57, ItemType::ItemTypeVirtualDirectory },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10CSyncEnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10CSyncEnumsE =
    CSyncEnums::qt_create_metaobjectdata<qt_meta_tag_ZN10CSyncEnumsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10CSyncEnumsE =
    qt_staticMetaObjectContent_ZN10CSyncEnumsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10CSyncEnumsE =
    qt_staticMetaObjectContent_ZN10CSyncEnumsE.relocatingData;

Q_CONSTINIT const QMetaObject CSyncEnums::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10CSyncEnumsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10CSyncEnumsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10CSyncEnumsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
