/****************************************************************************
** Meta object code from reading C++ file 'filetagmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/filetagmodel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filetagmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12FileTagModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FileTagModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12FileTagModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FileTagModel",
        "serverRelativePathChanged",
        "",
        "accountChanged",
        "maxTagsChanged",
        "totalTagsChanged",
        "overflowTagsStringChanged",
        "setServerRelativePath",
        "serverRelativePath",
        "setAccount",
        "OCC::AccountPtr",
        "account",
        "setMaxTags",
        "maxTags",
        "updateOverflowTagsString",
        "resetForNewFile",
        "fetchFileTags",
        "processFileTagRequestFinished",
        "QVariantMap",
        "result",
        "processFileTagRequestFinishedWithError",
        "AccountPtr",
        "totalTags",
        "overflowTagsString"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serverRelativePathChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maxTagsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'totalTagsChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overflowTagsStringChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setServerRelativePath'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'setAccount'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setMaxTags'
        QtMocHelpers::SlotData<void(const int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'updateOverflowTagsString'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetForNewFile'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchFileTags'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'processFileTagRequestFinished'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'processFileTagRequestFinishedWithError'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'serverRelativePath'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'account'
        QtMocHelpers::PropertyData<AccountPtr>(11, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'maxTags'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'totalTags'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags, 3),
        // property 'overflowTagsString'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileTagModel, qt_meta_tag_ZN3OCC12FileTagModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FileTagModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12FileTagModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12FileTagModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12FileTagModelE_t>.metaTypes,
    nullptr
} };

void OCC::FileTagModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileTagModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverRelativePathChanged(); break;
        case 1: _t->accountChanged(); break;
        case 2: _t->maxTagsChanged(); break;
        case 3: _t->totalTagsChanged(); break;
        case 4: _t->overflowTagsStringChanged(); break;
        case 5: _t->setServerRelativePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->setAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 7: _t->setMaxTags((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->updateOverflowTagsString(); break;
        case 9: _t->resetForNewFile(); break;
        case 10: _t->fetchFileTags(); break;
        case 11: _t->processFileTagRequestFinished((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 12: _t->processFileTagRequestFinishedWithError(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileTagModel::*)()>(_a, &FileTagModel::serverRelativePathChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileTagModel::*)()>(_a, &FileTagModel::accountChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileTagModel::*)()>(_a, &FileTagModel::maxTagsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileTagModel::*)()>(_a, &FileTagModel::totalTagsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileTagModel::*)()>(_a, &FileTagModel::overflowTagsStringChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->serverRelativePath(); break;
        case 1: *reinterpret_cast<AccountPtr*>(_v) = _t->account(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->maxTags(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->totalTags(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->overflowTagsString(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServerRelativePath(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setAccount(*reinterpret_cast<AccountPtr*>(_v)); break;
        case 2: _t->setMaxTags(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::FileTagModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FileTagModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12FileTagModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::FileTagModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::FileTagModel::serverRelativePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::FileTagModel::accountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::FileTagModel::maxTagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::FileTagModel::totalTagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::FileTagModel::overflowTagsStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
