/****************************************************************************
** Meta object code from reading C++ file 'sharemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/sharemanager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sharemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC5ShareE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Share::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC5ShareE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Share",
        "permissionsSet",
        "",
        "shareDeleted",
        "serverError",
        "code",
        "message",
        "passwordSet",
        "hideDownloadSet",
        "passwordSetError",
        "statusCode",
        "deleteShare",
        "setPermissions",
        "OCC::Share::Permissions",
        "permissions",
        "setPassword",
        "password",
        "slotOcsError",
        "slotPasswordSet",
        "QJsonDocument",
        "QVariant",
        "value",
        "slotSetPasswordError",
        "slotDeleted",
        "slotPermissionsSet",
        "account",
        "AccountPtr",
        "path",
        "id",
        "uidOwner",
        "ownerDisplayName",
        "shareType",
        "ShareType",
        "shareWith",
        "ShareePtr",
        "Permissions",
        "isPasswordSet",
        "TypeSecureFileDropPlaceholderLink",
        "TypeInternalLink",
        "TypePlaceholderLink",
        "TypeUser",
        "TypeGroup",
        "TypeLink",
        "TypeEmail",
        "TypeRemote",
        "TypeTeam",
        "TypeRoom"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'permissionsSet'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareDeleted'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverError'
        QtMocHelpers::SignalData<void(int, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'passwordSet'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideDownloadSet'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'passwordSetError'
        QtMocHelpers::SignalData<void(int, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'deleteShare'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPermissions'
        QtMocHelpers::SlotData<void(OCC::Share::Permissions)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'setPassword'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'slotOcsError'
        QtMocHelpers::SlotData<void(int, const QString &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'slotPasswordSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 19, 2 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotSetPasswordError'
        QtMocHelpers::SlotData<void(int, const QString &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'slotDeleted'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPermissionsSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 2 }, { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'account'
        QtMocHelpers::PropertyData<AccountPtr>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'path'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'id'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'uidOwner'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'ownerDisplayName'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'shareType'
        QtMocHelpers::PropertyData<enum ShareType>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'shareWith'
        QtMocHelpers::PropertyData<ShareePtr>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'permissions'
        QtMocHelpers::PropertyData<Permissions>(14, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'isPasswordSet'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ShareType'
        QtMocHelpers::EnumData<enum ShareType>(32, 32, QMC::EnumFlags{}).add({
            {   37, ShareType::TypeSecureFileDropPlaceholderLink },
            {   38, ShareType::TypeInternalLink },
            {   39, ShareType::TypePlaceholderLink },
            {   40, ShareType::TypeUser },
            {   41, ShareType::TypeGroup },
            {   42, ShareType::TypeLink },
            {   43, ShareType::TypeEmail },
            {   44, ShareType::TypeRemote },
            {   45, ShareType::TypeTeam },
            {   46, ShareType::TypeRoom },
        }),
    };
    return QtMocHelpers::metaObjectData<Share, qt_meta_tag_ZN3OCC5ShareE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Share::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ShareE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ShareE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC5ShareE_t>.metaTypes,
    nullptr
} };

void OCC::Share::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Share *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->permissionsSet(); break;
        case 1: _t->shareDeleted(); break;
        case 2: _t->serverError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->passwordSet(); break;
        case 4: _t->hideDownloadSet(); break;
        case 5: _t->passwordSetError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->deleteShare(); break;
        case 7: _t->setPermissions((*reinterpret_cast<std::add_pointer_t<OCC::Share::Permissions>>(_a[1]))); break;
        case 8: _t->setPassword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->slotOcsError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->slotPasswordSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 11: _t->slotSetPasswordError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->slotDeleted(); break;
        case 13: _t->slotPermissionsSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Share::*)()>(_a, &Share::permissionsSet, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Share::*)()>(_a, &Share::shareDeleted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Share::*)(int , const QString & )>(_a, &Share::serverError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Share::*)()>(_a, &Share::passwordSet, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Share::*)()>(_a, &Share::hideDownloadSet, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Share::*)(int , const QString & )>(_a, &Share::passwordSetError, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountPtr*>(_v) = _t->account(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->path(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getId(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->getUidOwner(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getOwnerDisplayName(); break;
        case 5: *reinterpret_cast<enum ShareType*>(_v) = _t->getShareType(); break;
        case 6: *reinterpret_cast<ShareePtr*>(_v) = _t->getShareWith(); break;
        case 7: *reinterpret_cast<Permissions*>(_v) = _t->getPermissions(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isPasswordSet(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 7: _t->setPermissions(*reinterpret_cast<Permissions*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Share::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Share::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ShareE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Share::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void OCC::Share::permissionsSet()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Share::shareDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Share::serverError(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::Share::passwordSet()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::Share::hideDownloadSet()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Share::passwordSetError(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN3OCC9LinkShareE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::LinkShare::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9LinkShareE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::LinkShare",
        "expireDateSet",
        "",
        "noteSet",
        "nameSet",
        "labelSet",
        "setName",
        "name",
        "setNote",
        "note",
        "setExpireDate",
        "expireDate",
        "setLabel",
        "label",
        "setHideDownload",
        "hideDownload",
        "slotNoteSet",
        "QJsonDocument",
        "QVariant",
        "value",
        "slotExpireDateSet",
        "reply",
        "slotNameSet",
        "slotLabelSet",
        "slotHideDownloadSet",
        "jsonDoc",
        "hideDownloadSet",
        "link",
        "QUrl",
        "directDownloadLink",
        "publicCanUpload",
        "publicCanReadDirectory",
        "token"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'expireDateSet'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noteSet'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameSet'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelSet'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setName'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setNote'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'setExpireDate'
        QtMocHelpers::SlotData<void(const QDate &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 11 },
        }}),
        // Slot 'setLabel'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'setHideDownload'
        QtMocHelpers::SlotData<void(const bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotNoteSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 2 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotExpireDateSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 21 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotNameSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 2 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotLabelSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 2 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotHideDownloadSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 25 }, { 0x80000000 | 18, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'link'
        QtMocHelpers::PropertyData<QUrl>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'directDownloadLink'
        QtMocHelpers::PropertyData<QUrl>(29, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'publicCanUpload'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'publicCanReadDirectory'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'note'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'hideDownload'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0x70000000 | 26),
        // property 'expireDate'
        QtMocHelpers::PropertyData<QDate>(11, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'token'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LinkShare, qt_meta_tag_ZN3OCC9LinkShareE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::LinkShare::staticMetaObject = { {
    QMetaObject::SuperData::link<Share::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9LinkShareE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9LinkShareE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9LinkShareE_t>.metaTypes,
    nullptr
} };

void OCC::LinkShare::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LinkShare *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->expireDateSet(); break;
        case 1: _t->noteSet(); break;
        case 2: _t->nameSet(); break;
        case 3: _t->labelSet(); break;
        case 4: _t->setName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->setNote((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->setExpireDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 7: _t->setLabel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->setHideDownload((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotNoteSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 10: _t->slotExpireDateSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 11: _t->slotNameSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 12: _t->slotLabelSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 13: _t->slotHideDownloadSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LinkShare::*)()>(_a, &LinkShare::expireDateSet, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LinkShare::*)()>(_a, &LinkShare::noteSet, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LinkShare::*)()>(_a, &LinkShare::nameSet, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LinkShare::*)()>(_a, &LinkShare::labelSet, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->getLink(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->getDirectDownloadLink(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getPublicUpload(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getShowFileListing(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getNote(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getLabel(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getHideDownload(); break;
        case 8: *reinterpret_cast<QDate*>(_v) = _t->getExpireDate(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getToken(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setNote(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setHideDownload(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setExpireDate(*reinterpret_cast<QDate*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::LinkShare::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::LinkShare::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9LinkShareE_t>.strings))
        return static_cast<void*>(this);
    return Share::qt_metacast(_clname);
}

int OCC::LinkShare::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Share::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void OCC::LinkShare::expireDateSet()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::LinkShare::noteSet()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::LinkShare::nameSet()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::LinkShare::labelSet()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace CheckNotifySignalValidity_ZN3OCC9LinkShareE {
template<typename T> using has_nullary_hideDownloadSet = decltype(std::declval<T>().hideDownloadSet());
template<typename T> using has_unary_hideDownloadSet = decltype(std::declval<T>().hideDownloadSet(std::declval<bool>()));
static_assert(qxp::is_detected_v<has_nullary_hideDownloadSet, OCC::LinkShare> || qxp::is_detected_v<has_unary_hideDownloadSet, OCC::LinkShare>,
              "NOTIFY signal hideDownloadSet does not exist in class (or is private in its parent)");
}
namespace {
struct qt_meta_tag_ZN3OCC14UserGroupShareE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UserGroupShare::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14UserGroupShareE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UserGroupShare",
        "noteSet",
        "",
        "noteSetError",
        "expireDateSet",
        "setNote",
        "note",
        "setExpireDate",
        "date",
        "slotNoteSet",
        "QJsonDocument",
        "json",
        "QVariant",
        "slotExpireDateSet",
        "reply",
        "value",
        "expireDate"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'noteSet'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noteSetError'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expireDateSet'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setNote'
        QtMocHelpers::SlotData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'setExpireDate'
        QtMocHelpers::SlotData<void(const QDate &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 8 },
        }}),
        // Slot 'slotNoteSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 6 },
        }}),
        // Slot 'slotExpireDateSet'
        QtMocHelpers::SlotData<void(const QJsonDocument &, const QVariant &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 14 }, { 0x80000000 | 12, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'note'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'expireDate'
        QtMocHelpers::PropertyData<QDate>(16, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UserGroupShare, qt_meta_tag_ZN3OCC14UserGroupShareE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::UserGroupShare::staticMetaObject = { {
    QMetaObject::SuperData::link<Share::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14UserGroupShareE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14UserGroupShareE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14UserGroupShareE_t>.metaTypes,
    nullptr
} };

void OCC::UserGroupShare::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UserGroupShare *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->noteSet(); break;
        case 1: _t->noteSetError(); break;
        case 2: _t->expireDateSet(); break;
        case 3: _t->setNote((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->setExpireDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 5: _t->slotNoteSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 6: _t->slotExpireDateSet((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UserGroupShare::*)()>(_a, &UserGroupShare::noteSet, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserGroupShare::*)()>(_a, &UserGroupShare::noteSetError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserGroupShare::*)()>(_a, &UserGroupShare::expireDateSet, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getNote(); break;
        case 1: *reinterpret_cast<QDate*>(_v) = _t->getExpireDate(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNote(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setExpireDate(*reinterpret_cast<QDate*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::UserGroupShare::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::UserGroupShare::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14UserGroupShareE_t>.strings))
        return static_cast<void*>(this);
    return Share::qt_metacast(_clname);
}

int OCC::UserGroupShare::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Share::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::UserGroupShare::noteSet()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::UserGroupShare::noteSetError()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::UserGroupShare::expireDateSet()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC12ShareManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ShareManager::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12ShareManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ShareManager",
        "shareCreated",
        "",
        "OCC::SharePtr",
        "share",
        "linkShareCreated",
        "QSharedPointer<OCC::LinkShare>",
        "sharesFetched",
        "QList<OCC::SharePtr>",
        "shares",
        "sharedWithMeFetched",
        "serverError",
        "code",
        "message",
        "linkShareRequiresPassword",
        "slotSharesFetched",
        "QJsonDocument",
        "reply",
        "slotSharedWithMeFetched",
        "slotLinkShareCreated",
        "slotShareCreated",
        "slotOcsError",
        "statusCode",
        "slotCreateE2eeShareJobFinised"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'shareCreated'
        QtMocHelpers::SignalData<void(const OCC::SharePtr &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'linkShareCreated'
        QtMocHelpers::SignalData<void(const QSharedPointer<OCC::LinkShare> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Signal 'sharesFetched'
        QtMocHelpers::SignalData<void(const QList<OCC::SharePtr> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'sharedWithMeFetched'
        QtMocHelpers::SignalData<void(const QList<OCC::SharePtr> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'serverError'
        QtMocHelpers::SignalData<void(int, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'linkShareRequiresPassword'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'slotSharesFetched'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotSharedWithMeFetched'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotLinkShareCreated'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotShareCreated'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotOcsError'
        QtMocHelpers::SlotData<void(int, const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 22 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'slotCreateE2eeShareJobFinised'
        QtMocHelpers::SlotData<void(int, const QString &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 22 }, { QMetaType::QString, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ShareManager, qt_meta_tag_ZN3OCC12ShareManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ShareManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ShareManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ShareManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12ShareManagerE_t>.metaTypes,
    nullptr
} };

void OCC::ShareManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShareManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->shareCreated((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1]))); break;
        case 1: _t->linkShareCreated((*reinterpret_cast<std::add_pointer_t<QSharedPointer<OCC::LinkShare>>>(_a[1]))); break;
        case 2: _t->sharesFetched((*reinterpret_cast<std::add_pointer_t<QList<OCC::SharePtr>>>(_a[1]))); break;
        case 3: _t->sharedWithMeFetched((*reinterpret_cast<std::add_pointer_t<QList<OCC::SharePtr>>>(_a[1]))); break;
        case 4: _t->serverError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->linkShareRequiresPassword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotSharesFetched((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        case 7: _t->slotSharedWithMeFetched((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        case 8: _t->slotLinkShareCreated((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        case 9: _t->slotShareCreated((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        case 10: _t->slotOcsError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->slotCreateE2eeShareJobFinised((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<OCC::LinkShare> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::SharePtr> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::SharePtr> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShareManager::*)(const OCC::SharePtr & )>(_a, &ShareManager::shareCreated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareManager::*)(const QSharedPointer<OCC::LinkShare> & )>(_a, &ShareManager::linkShareCreated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareManager::*)(const QList<OCC::SharePtr> & )>(_a, &ShareManager::sharesFetched, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareManager::*)(const QList<OCC::SharePtr> & )>(_a, &ShareManager::sharedWithMeFetched, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareManager::*)(int , const QString & )>(_a, &ShareManager::serverError, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareManager::*)(const QString & )>(_a, &ShareManager::linkShareRequiresPassword, 5))
            return;
    }
}

const QMetaObject *OCC::ShareManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ShareManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ShareManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ShareManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void OCC::ShareManager::shareCreated(const OCC::SharePtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::ShareManager::linkShareCreated(const QSharedPointer<OCC::LinkShare> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::ShareManager::sharesFetched(const QList<OCC::SharePtr> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::ShareManager::sharedWithMeFetched(const QList<OCC::SharePtr> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::ShareManager::serverError(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void OCC::ShareManager::linkShareRequiresPassword(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
