/****************************************************************************
** Meta object code from reading C++ file 'owncloudgui.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/owncloudgui.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'owncloudgui.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC11ownCloudGuiE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ownCloudGui::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ownCloudGui",
        "setupProxy",
        "",
        "serverError",
        "code",
        "message",
        "isShowingSettingsDialog",
        "slotComputeOverallSyncStatus",
        "slotShowTrayMessage",
        "title",
        "msg",
        "slotShowTrayUpdateMessage",
        "QUrl",
        "webUrl",
        "slotFolderOpenAction",
        "alias",
        "slotUpdateProgress",
        "folder",
        "OCC::ProgressInfo",
        "progress",
        "slotShowGuiMessage",
        "slotShowSettings",
        "slotShowSyncProtocol",
        "slotShutdown",
        "slotSyncStateChange",
        "OCC::Folder*",
        "slotTrayClicked",
        "QSystemTrayIcon::ActivationReason",
        "reason",
        "slotToggleLogBrowser",
        "slotOpenOwnCloud",
        "slotOpenSettingsDialog",
        "slotOpenMainDialog",
        "slotSettingsDialogActivated",
        "slotHelp",
        "slotOpenPath",
        "path",
        "slotTrayMessageIfServerUnsupported",
        "OCC::AccountPtr",
        "account",
        "slotNeedToAcceptTermsOfService",
        "OCC::AccountState::State",
        "state",
        "slotShowShareDialog",
        "localPath",
        "slotShowFileActivityDialog",
        "slotShowFileActionsDialog",
        "slotNewAccountWizard",
        "slotLogin",
        "slotLogout"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setupProxy'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverError'
        QtMocHelpers::SignalData<void(int, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'isShowingSettingsDialog'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotComputeOverallSyncStatus'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowTrayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotShowTrayUpdateMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QUrl &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotFolderOpenAction'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'slotUpdateProgress'
        QtMocHelpers::SlotData<void(const QString &, const OCC::ProgressInfo &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotShowGuiMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'slotShowSettings'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowSyncProtocol'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShutdown'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncStateChange'
        QtMocHelpers::SlotData<void(OCC::Folder *)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 2 },
        }}),
        // Slot 'slotTrayClicked'
        QtMocHelpers::SlotData<void(QSystemTrayIcon::ActivationReason)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'slotToggleLogBrowser'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenOwnCloud'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenSettingsDialog'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenMainDialog'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSettingsDialogActivated'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHelp'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenPath'
        QtMocHelpers::SlotData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
        // Slot 'slotTrayMessageIfServerUnsupported'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'slotNeedToAcceptTermsOfService'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &, OCC::AccountState::State)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'slotShowShareDialog'
        QtMocHelpers::SlotData<void(const QString &) const>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Slot 'slotShowFileActivityDialog'
        QtMocHelpers::SlotData<void(const QString &) const>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Slot 'slotShowFileActionsDialog'
        QtMocHelpers::SlotData<void(const QString &) const>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Slot 'slotNewAccountWizard'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLogin'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLogout'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ownCloudGui, qt_meta_tag_ZN3OCC11ownCloudGuiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ownCloudGui::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.metaTypes,
    nullptr
} };

void OCC::ownCloudGui::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ownCloudGui *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setupProxy(); break;
        case 1: _t->serverError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->isShowingSettingsDialog(); break;
        case 3: _t->slotComputeOverallSyncStatus(); break;
        case 4: _t->slotShowTrayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->slotShowTrayUpdateMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 6: _t->slotFolderOpenAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->slotUpdateProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ProgressInfo>>(_a[2]))); break;
        case 8: _t->slotShowGuiMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->slotShowSettings(); break;
        case 10: _t->slotShowSyncProtocol(); break;
        case 11: _t->slotShutdown(); break;
        case 12: _t->slotSyncStateChange((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1]))); break;
        case 13: _t->slotTrayClicked((*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::ActivationReason>>(_a[1]))); break;
        case 14: _t->slotToggleLogBrowser(); break;
        case 15: _t->slotOpenOwnCloud(); break;
        case 16: _t->slotOpenSettingsDialog(); break;
        case 17: _t->slotOpenMainDialog(); break;
        case 18: _t->slotSettingsDialogActivated(); break;
        case 19: _t->slotHelp(); break;
        case 20: _t->slotOpenPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->slotTrayMessageIfServerUnsupported((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 22: _t->slotNeedToAcceptTermsOfService((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::AccountState::State>>(_a[2]))); break;
        case 23: _t->slotShowShareDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->slotShowFileActivityDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->slotShowFileActionsDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->slotNewAccountWizard(); break;
        case 27: _t->slotLogin(); break;
        case 28: _t->slotLogout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Folder* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountPtr >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ownCloudGui::*)()>(_a, &ownCloudGui::setupProxy, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ownCloudGui::*)(int , const QString & )>(_a, &ownCloudGui::serverError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ownCloudGui::*)()>(_a, &ownCloudGui::isShowingSettingsDialog, 2))
            return;
    }
}

const QMetaObject *OCC::ownCloudGui::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ownCloudGui::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11ownCloudGuiE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ownCloudGui::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void OCC::ownCloudGui::setupProxy()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ownCloudGui::serverError(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::ownCloudGui::isShowingSettingsDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
