/****************************************************************************
** Meta object code from reading C++ file 'pinstate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/common/pinstate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pinstate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCCE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::qt_create_metaobjectdata<qt_meta_tag_ZN3OCCE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC",
        "PinState",
        "Inherited",
        "AlwaysLocal",
        "OnlineOnly",
        "Unspecified",
        "Excluded",
        "VfsItemAvailability",
        "AllHydrated",
        "Mixed",
        "AllDehydrated"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PinState'
        QtMocHelpers::EnumData<PinState>(1, 1, QMC::EnumIsScoped).add({
            {    2, PinState::Inherited },
            {    3, PinState::AlwaysLocal },
            {    4, PinState::OnlineOnly },
            {    5, PinState::Unspecified },
            {    6, PinState::Excluded },
        }),
        // enum 'VfsItemAvailability'
        QtMocHelpers::EnumData<VfsItemAvailability>(7, 7, QMC::EnumIsScoped).add({
            {    3, VfsItemAvailability::AlwaysLocal },
            {    8, VfsItemAvailability::AllHydrated },
            {    9, VfsItemAvailability::Mixed },
            {   10, VfsItemAvailability::AllDehydrated },
            {    4, VfsItemAvailability::OnlineOnly },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3OCCE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN3OCCE =
    OCC::qt_create_metaobjectdata<qt_meta_tag_ZN3OCCE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3OCCE =
    qt_staticMetaObjectContent_ZN3OCCE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3OCCE =
    qt_staticMetaObjectContent_ZN3OCCE.relocatingData;

Q_CONSTINIT const QMetaObject OCC::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3OCCE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3OCCE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3OCCE.metaTypes,
    nullptr
} };

QT_WARNING_POP
