/*
 * SPDX-FileCopyrightText: 2014 ownCloud GmbH
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#pragma once

#include <QString>

namespace OCC {
/**
 * @brief Open the file manager with the specified file pre-selected
 * @ingroup gui
 */
void showInFileManager(const QString &localPath);
}
