/****************************************************************************
** Meta object code from reading C++ file 'connectionvalidator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/connectionvalidator.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'connectionvalidator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::TermsOfServiceChecker::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::TermsOfServiceChecker",
        "needToSignChanged",
        "",
        "done",
        "start",
        "slotServerTermsOfServiceRecieved",
        "QJsonDocument",
        "reply",
        "needToSign"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'needToSignChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'done'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotServerTermsOfServiceRecieved'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'needToSign'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TermsOfServiceChecker, qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::TermsOfServiceChecker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t>.metaTypes,
    nullptr
} };

void OCC::TermsOfServiceChecker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TermsOfServiceChecker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->needToSignChanged(); break;
        case 1: _t->done(); break;
        case 2: _t->start(); break;
        case 3: _t->slotServerTermsOfServiceRecieved((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TermsOfServiceChecker::*)()>(_a, &TermsOfServiceChecker::needToSignChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TermsOfServiceChecker::*)()>(_a, &TermsOfServiceChecker::done, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->needToSign(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::TermsOfServiceChecker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::TermsOfServiceChecker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21TermsOfServiceCheckerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::TermsOfServiceChecker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OCC::TermsOfServiceChecker::needToSignChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::TermsOfServiceChecker::done()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC19ConnectionValidatorE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ConnectionValidator::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ConnectionValidator",
        "connectionResult",
        "",
        "OCC::ConnectionValidator::Status",
        "status",
        "errors",
        "checkServerAndAuth",
        "systemProxyLookupDone",
        "QNetworkProxy",
        "proxy",
        "checkAuthentication",
        "slotCheckRedirectCostFreeUrl",
        "slotCheckServerAndAuth",
        "slotCheckRedirectCostFreeUrlFinished",
        "statusCode",
        "slotStatusFound",
        "QUrl",
        "url",
        "QJsonObject",
        "info",
        "slotNoStatusFound",
        "QNetworkReply*",
        "reply",
        "slotJobTimeout",
        "slotAuthFailed",
        "slotAuthSuccess",
        "slotCapabilitiesRecieved",
        "QJsonDocument",
        "slotUserFetched",
        "OCC::UserInfo*",
        "userInfo",
        "termsOfServiceCheckDone",
        "Status",
        "Undefined",
        "Connected",
        "NotConfigured",
        "ServerVersionMismatch",
        "CredentialsNotReady",
        "CredentialsWrong",
        "SslError",
        "StatusNotFound",
        "StatusRedirect",
        "ServiceUnavailable",
        "MaintenanceMode",
        "Timeout",
        "NeedToSignTermsOfService"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectionResult'
        QtMocHelpers::SignalData<void(OCC::ConnectionValidator::Status, const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QStringList, 5 },
        }}),
        // Slot 'checkServerAndAuth'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'systemProxyLookupDone'
        QtMocHelpers::SlotData<void(const QNetworkProxy &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'checkAuthentication'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCheckRedirectCostFreeUrl'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCheckServerAndAuth'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCheckRedirectCostFreeUrlFinished'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'slotStatusFound'
        QtMocHelpers::SlotData<void(const QUrl &, const QJsonObject &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotNoStatusFound'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotJobTimeout'
        QtMocHelpers::SlotData<void(const QUrl &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotAuthFailed'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotAuthSuccess'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCapabilitiesRecieved'
        QtMocHelpers::SlotData<void(const QJsonDocument &)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 27, 2 },
        }}),
        // Slot 'slotUserFetched'
        QtMocHelpers::SlotData<void(OCC::UserInfo *)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'termsOfServiceCheckDone'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(32, 32, QMC::EnumFlags{}).add({
            {   33, Status::Undefined },
            {   34, Status::Connected },
            {   35, Status::NotConfigured },
            {   36, Status::ServerVersionMismatch },
            {   37, Status::CredentialsNotReady },
            {   38, Status::CredentialsWrong },
            {   39, Status::SslError },
            {   40, Status::StatusNotFound },
            {   41, Status::StatusRedirect },
            {   42, Status::ServiceUnavailable },
            {   43, Status::MaintenanceMode },
            {   44, Status::Timeout },
            {   45, Status::NeedToSignTermsOfService },
        }),
    };
    return QtMocHelpers::metaObjectData<ConnectionValidator, qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ConnectionValidator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.metaTypes,
    nullptr
} };

void OCC::ConnectionValidator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConnectionValidator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectionResult((*reinterpret_cast<std::add_pointer_t<OCC::ConnectionValidator::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 1: _t->checkServerAndAuth(); break;
        case 2: _t->systemProxyLookupDone((*reinterpret_cast<std::add_pointer_t<QNetworkProxy>>(_a[1]))); break;
        case 3: _t->checkAuthentication(); break;
        case 4: _t->slotCheckRedirectCostFreeUrl(); break;
        case 5: _t->slotCheckServerAndAuth(); break;
        case 6: _t->slotCheckRedirectCostFreeUrlFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->slotStatusFound((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJsonObject>>(_a[2]))); break;
        case 8: _t->slotNoStatusFound((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 9: _t->slotJobTimeout((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 10: _t->slotAuthFailed((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 11: _t->slotAuthSuccess(); break;
        case 12: _t->slotCapabilitiesRecieved((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1]))); break;
        case 13: _t->slotUserFetched((*reinterpret_cast<std::add_pointer_t<OCC::UserInfo*>>(_a[1]))); break;
        case 14: _t->termsOfServiceCheckDone(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConnectionValidator::*)(OCC::ConnectionValidator::Status , const QStringList & )>(_a, &ConnectionValidator::connectionResult, 0))
            return;
    }
}

const QMetaObject *OCC::ConnectionValidator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ConnectionValidator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ConnectionValidatorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ConnectionValidator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void OCC::ConnectionValidator::connectionResult(OCC::ConnectionValidator::Status _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
