/********************************************************************************
** Form generated from reading UI file 'termsofservicecheckwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TERMSOFSERVICECHECKWIDGET_H
#define UI_TERMSOFSERVICECHECKWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_TermsOfServiceCheckWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *verticalSpacer_6;
    QVBoxLayout *verticalLayout;
    QLabel *logoLabel;
    QSpacerItem *verticalSpacer;
    QLabel *label;
    QSpacerItem *verticalSpacer_4;
    QLabel *statusLabel;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *progressLayout;
    QSpacerItem *verticalSpacer_5;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QPushButton *copyLinkButton;
    QPushButton *openLinkButton;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *TermsOfServiceCheckWidget)
    {
        if (TermsOfServiceCheckWidget->objectName().isEmpty())
            TermsOfServiceCheckWidget->setObjectName("TermsOfServiceCheckWidget");
        TermsOfServiceCheckWidget->resize(597, 387);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TermsOfServiceCheckWidget->sizePolicy().hasHeightForWidth());
        TermsOfServiceCheckWidget->setSizePolicy(sizePolicy);
        TermsOfServiceCheckWidget->setMinimumSize(QSize(500, 280));
        verticalLayout_2 = new QVBoxLayout(TermsOfServiceCheckWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_6);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        logoLabel = new QLabel(TermsOfServiceCheckWidget);
        logoLabel->setObjectName("logoLabel");
        logoLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(logoLabel);

        verticalSpacer = new QSpacerItem(20, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        label = new QLabel(TermsOfServiceCheckWidget);
        label->setObjectName("label");
        QFont font;
        font.setPointSize(12);
        font.setBold(true);
        label->setFont(font);
        label->setAlignment(Qt::AlignmentFlag::AlignCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        verticalSpacer_4 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_4);

        statusLabel = new QLabel(TermsOfServiceCheckWidget);
        statusLabel->setObjectName("statusLabel");
        statusLabel->setText(QString::fromUtf8("Status"));
        statusLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        statusLabel->setMargin(0);

        verticalLayout->addWidget(statusLabel);

        verticalSpacer_2 = new QSpacerItem(20, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        progressLayout = new QHBoxLayout();
        progressLayout->setObjectName("progressLayout");

        verticalLayout->addLayout(progressLayout);

        verticalSpacer_5 = new QSpacerItem(20, 64, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_5);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        copyLinkButton = new QPushButton(TermsOfServiceCheckWidget);
        copyLinkButton->setObjectName("copyLinkButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(copyLinkButton->sizePolicy().hasHeightForWidth());
        copyLinkButton->setSizePolicy(sizePolicy1);
        QFont font1;
        font1.setPointSize(9);
        copyLinkButton->setFont(font1);
        copyLinkButton->setText(QString::fromUtf8("copyLinkButton"));
        copyLinkButton->setAutoDefault(true);

        horizontalLayout->addWidget(copyLinkButton);

        openLinkButton = new QPushButton(TermsOfServiceCheckWidget);
        openLinkButton->setObjectName("openLinkButton");
        sizePolicy1.setHeightForWidth(openLinkButton->sizePolicy().hasHeightForWidth());
        openLinkButton->setSizePolicy(sizePolicy1);
        openLinkButton->setFont(font1);
        openLinkButton->setText(QString::fromUtf8("openLinkButton"));
        openLinkButton->setAutoDefault(true);

        horizontalLayout->addWidget(openLinkButton);


        horizontalLayout_2->addLayout(horizontalLayout);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_2->addLayout(verticalLayout);

        verticalSpacer_3 = new QSpacerItem(20, 60, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);


        retranslateUi(TermsOfServiceCheckWidget);

        copyLinkButton->setDefault(false);
        openLinkButton->setDefault(false);


        QMetaObject::connectSlotsByName(TermsOfServiceCheckWidget);
    } // setupUi

    void retranslateUi(QWidget *TermsOfServiceCheckWidget)
    {
        TermsOfServiceCheckWidget->setWindowTitle(QCoreApplication::translate("TermsOfServiceCheckWidget", "Terms of Service", nullptr));
        logoLabel->setText(QCoreApplication::translate("TermsOfServiceCheckWidget", "Logo", nullptr));
        label->setText(QCoreApplication::translate("TermsOfServiceCheckWidget", "Switch to your browser to accept the terms of service", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TermsOfServiceCheckWidget: public Ui_TermsOfServiceCheckWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TERMSOFSERVICECHECKWIDGET_H
