/****************************************************************************
** Meta object code from reading C++ file 'fileactionsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/integration/fileactionsmodel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileactionsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC16FileActionsModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FileActionsModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC16FileActionsModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FileActionsModel",
        "accountStateChanged",
        "",
        "fileChanged",
        "responseChanged",
        "fileActionModelChanged",
        "createRequest",
        "row",
        "processRequest",
        "QJsonDocument",
        "json",
        "statusCode",
        "accountState",
        "AccountState*",
        "localPath",
        "fileId",
        "remoteItemPath",
        "fileIcon",
        "responseLabel",
        "responseUrl",
        "DataRole",
        "FileActionIconRole",
        "FileActionNameRole",
        "FileActionUrlRole",
        "FileActionMethodRole",
        "FileActionParamsRole",
        "FileActionResponseLabelRole",
        "FileActionResponseUrlRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountStateChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'responseChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileActionModelChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createRequest'
        QtMocHelpers::SlotData<void(const int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'processRequest'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accountState'
        QtMocHelpers::PropertyData<AccountState*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'localPath'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'fileId'
        QtMocHelpers::PropertyData<QByteArray>(15, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'remoteItemPath'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'fileIcon'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'responseLabel'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'responseUrl'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DataRole'
        QtMocHelpers::EnumData<enum DataRole>(20, 20, QMC::EnumFlags{}).add({
            {   21, DataRole::FileActionIconRole },
            {   22, DataRole::FileActionNameRole },
            {   23, DataRole::FileActionUrlRole },
            {   24, DataRole::FileActionMethodRole },
            {   25, DataRole::FileActionParamsRole },
            {   26, DataRole::FileActionResponseLabelRole },
            {   27, DataRole::FileActionResponseUrlRole },
        }),
    };
    return QtMocHelpers::metaObjectData<FileActionsModel, qt_meta_tag_ZN3OCC16FileActionsModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FileActionsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16FileActionsModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16FileActionsModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC16FileActionsModelE_t>.metaTypes,
    nullptr
} };

void OCC::FileActionsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileActionsModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountStateChanged(); break;
        case 1: _t->fileChanged(); break;
        case 2: _t->responseChanged(); break;
        case 3: _t->fileActionModelChanged(); break;
        case 4: _t->createRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->processRequest((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileActionsModel::*)()>(_a, &FileActionsModel::accountStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileActionsModel::*)()>(_a, &FileActionsModel::fileChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileActionsModel::*)()>(_a, &FileActionsModel::responseChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileActionsModel::*)()>(_a, &FileActionsModel::fileActionModelChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AccountState* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountState**>(_v) = _t->accountState(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->localPath(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->fileId(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->remoteItemPath(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->fileIcon(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->responseLabel(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->responseUrl(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAccountState(*reinterpret_cast<AccountState**>(_v)); break;
        case 1: _t->setLocalPath(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setFileId(*reinterpret_cast<QByteArray*>(_v)); break;
        case 3: _t->setRemoteItemPath(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setResponseLabel(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setResponseUrl(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::FileActionsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FileActionsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16FileActionsModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::FileActionsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OCC::FileActionsModel::accountStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::FileActionsModel::fileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::FileActionsModel::responseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::FileActionsModel::fileActionModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
