/****************************************************************************
** Meta object code from reading C++ file 'flow2auth.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/creds/flow2auth.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'flow2auth.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC9Flow2AuthE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Flow2Auth::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9Flow2AuthE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Flow2Auth",
        "result",
        "",
        "OCC::Flow2Auth::Result",
        "errorString",
        "user",
        "appPassword",
        "statusChanged",
        "OCC::Flow2Auth::PollStatus",
        "status",
        "secondsLeft",
        "slotPollNow",
        "slotPollTimerTimeout",
        "Result",
        "NotSupported",
        "LoggedIn",
        "Error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'result'
        QtMocHelpers::SignalData<void(OCC::Flow2Auth::Result, const QString &, const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 1 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'result'
        QtMocHelpers::SignalData<void(OCC::Flow2Auth::Result, const QString &, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 1 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'result'
        QtMocHelpers::SignalData<void(OCC::Flow2Auth::Result, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 1 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'result'
        QtMocHelpers::SignalData<void(OCC::Flow2Auth::Result)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 1 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(const OCC::Flow2Auth::PollStatus, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'slotPollNow'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPollTimerTimeout'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Result'
        QtMocHelpers::EnumData<enum Result>(13, 13, QMC::EnumFlags{}).add({
            {   14, Result::NotSupported },
            {   15, Result::LoggedIn },
            {   16, Result::Error },
        }),
    };
    return QtMocHelpers::metaObjectData<Flow2Auth, qt_meta_tag_ZN3OCC9Flow2AuthE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Flow2Auth::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Flow2AuthE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Flow2AuthE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9Flow2AuthE_t>.metaTypes,
    nullptr
} };

void OCC::Flow2Auth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Flow2Auth *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->result((*reinterpret_cast<std::add_pointer_t<OCC::Flow2Auth::Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 1: _t->result((*reinterpret_cast<std::add_pointer_t<OCC::Flow2Auth::Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->result((*reinterpret_cast<std::add_pointer_t<OCC::Flow2Auth::Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->result((*reinterpret_cast<std::add_pointer_t<OCC::Flow2Auth::Result>>(_a[1]))); break;
        case 4: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<OCC::Flow2Auth::PollStatus>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->slotPollNow(); break;
        case 6: _t->slotPollTimerTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Flow2Auth::*)(OCC::Flow2Auth::Result , const QString & , const QString & , const QString & )>(_a, &Flow2Auth::result, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Flow2Auth::*)(const OCC::Flow2Auth::PollStatus , int )>(_a, &Flow2Auth::statusChanged, 4))
            return;
    }
}

const QMetaObject *OCC::Flow2Auth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Flow2Auth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Flow2AuthE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Flow2Auth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OCC::Flow2Auth::result(OCC::Flow2Auth::Result _t1, const QString & _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void OCC::Flow2Auth::statusChanged(const OCC::Flow2Auth::PollStatus _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
