/*
 * SPDX-FileCopyrightText: 2021 Nextcloud GmbH and Nextcloud contributors
 * SPDX-FileCopyrightText: 2017 ownCloud, Inc.
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#ifndef VERSION_H
#define VERSION_H

#define GIT_SHA1 "2bb7ca6f2bba603ce27105b3b80dc9d0e226c636"

#define MIRALL_STRINGIFY(s) MIRALL_TOSTRING(s)
#define MIRALL_TOSTRING(s) #s

/* MIRALL version */
constexpr auto MIRALL_VERSION_MAJOR = 33;
constexpr auto MIRALL_VERSION_MINOR = 0;
constexpr auto MIRALL_VERSION_PATCH = 5;
constexpr auto MIRALL_VERSION_BUILD = 0;
constexpr auto MIRALL_VERSION_SUFFIX = "daily";

#define MIRALL_VERSION       33.0.5
#define MIRALL_VERSION_FULL  33.0.5.0

#define MIRALL_VERSION_STRING "33.0.5daily"
#define MIRALL_HUMAN_VERSION_STRING "33.0.5daily"

constexpr int NEXTCLOUD_SERVER_VERSION_MIN_SUPPORTED_MAJOR = 20;
constexpr int NEXTCLOUD_SERVER_VERSION_MIN_SUPPORTED_MINOR = 0;
constexpr int NEXTCLOUD_SERVER_VERSION_MIN_SUPPORTED_PATCH = 0;

constexpr int NEXTCLOUD_SERVER_VERSION_SECURE_FILEDROP_MIN_SUPPORTED_MAJOR = 26;
constexpr int NEXTCLOUD_SERVER_VERSION_SECURE_FILEDROP_MIN_SUPPORTED_MINOR = 0;
constexpr int NEXTCLOUD_SERVER_VERSION_SECURE_FILEDROP_MIN_SUPPORTED_PATCH = 0;

constexpr int NEXTCLOUD_SERVER_VERSION_MOUNT_ROOT_PROPERTY_SUPPORTED_MAJOR = 28;
constexpr int NEXTCLOUD_SERVER_VERSION_MOUNT_ROOT_PROPERTY_SUPPORTED_MINOR = 0;
constexpr int NEXTCLOUD_SERVER_VERSION_MOUNT_ROOT_PROPERTY_SUPPORTED_PATCH = 3;

#endif // VERSION_H
