/****************************************************************************
** Meta object code from reading C++ file 'sharemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/filedetails/sharemodel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sharemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC10ShareModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ShareModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10ShareModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ShareModel",
        "localPathChanged",
        "",
        "accountStateChanged",
        "accountConnectedChanged",
        "sharingEnabledChanged",
        "publicLinkSharesEnabledChanged",
        "userGroupSharingEnabledChanged",
        "sharePermissionsChanged",
        "isShareDisabledEncryptedFolderChanged",
        "lockExpireStringChanged",
        "fetchOngoingChanged",
        "hasInitialShareFetchCompletedChanged",
        "shareesChanged",
        "internalLinkReady",
        "serverAllowsResharingChanged",
        "displayShareOwnerChanged",
        "shareOwnerDisplayNameChanged",
        "shareOwnerAvatarChanged",
        "sharedWithMeExpiresChanged",
        "sharedWithMeRemainingTimeStringChanged",
        "serverError",
        "code",
        "message",
        "passwordSetError",
        "shareId",
        "requestPasswordForLinkShare",
        "requestPasswordForEmailSharee",
        "OCC::ShareePtr",
        "sharee",
        "sharesChanged",
        "setAccountState",
        "OCC::AccountState*",
        "accountState",
        "setLocalPath",
        "localPath",
        "createNewLinkShare",
        "createNewLinkShareWithPassword",
        "password",
        "createNewUserGroupShare",
        "createNewUserGroupShareFromQml",
        "QVariant",
        "createNewUserGroupShareWithPassword",
        "createNewUserGroupShareWithPasswordFromQml",
        "deleteShare",
        "OCC::SharePtr",
        "share",
        "deleteShareFromQml",
        "toggleHideDownloadFromQml",
        "enable",
        "toggleShareAllowEditing",
        "toggleShareAllowEditingFromQml",
        "toggleShareAllowResharing",
        "toggleShareAllowResharingFromQml",
        "toggleSharePasswordProtect",
        "toggleSharePasswordProtectFromQml",
        "toggleShareExpirationDate",
        "toggleShareExpirationDateFromQml",
        "changePermissionModeFromQml",
        "OCC::ShareModel::SharePermissionsMode",
        "permissionMode",
        "setLinkShareLabel",
        "QSharedPointer<OCC::LinkShare>",
        "linkShare",
        "label",
        "setLinkShareLabelFromQml",
        "setShareExpireDate",
        "milliseconds",
        "setShareExpireDateFromQml",
        "setSharePassword",
        "setSharePasswordFromQml",
        "setShareNote",
        "note",
        "setShareNoteFromQml",
        "resetData",
        "updateData",
        "initShareManager",
        "handlePlaceholderLinkShare",
        "handleSecureFileDropLinkShare",
        "handleLinkShare",
        "setupInternalLinkShare",
        "setSharePermissionChangeInProgress",
        "isInProgress",
        "setHideDownloadEnabledChangeInProgress",
        "slotPropfindReceived",
        "QVariantMap",
        "result",
        "slotAddShare",
        "slotRemoveShareWithId",
        "slotSharesFetched",
        "QList<OCC::SharePtr>",
        "shares",
        "slotSharedWithMeFetched",
        "slotAddSharee",
        "slotRemoveSharee",
        "slotSharePermissionsSet",
        "slotSharePasswordSet",
        "slotShareNoteSet",
        "slotHideDownloadSet",
        "slotShareNameSet",
        "slotShareLabelSet",
        "slotShareExpireDateSet",
        "slotDeleteE2EeShare",
        "generatePassword",
        "AccountState*",
        "accountConnected",
        "sharingEnabled",
        "publicLinkSharesEnabled",
        "userGroupSharingEnabled",
        "canShare",
        "isShareDisabledEncryptedFolder",
        "fetchOngoing",
        "hasInitialShareFetchCompleted",
        "serverAllowsResharing",
        "sharees",
        "QVariantList",
        "displayShareOwner",
        "shareOwnerDisplayName",
        "shareOwnerAvatar",
        "sharedWithMeExpires",
        "sharedWithMeRemainingTimeString",
        "Roles",
        "ShareRole",
        "ShareTypeRole",
        "ShareIdRole",
        "IconUrlRole",
        "AvatarUrlRole",
        "LinkRole",
        "LinkShareNameRole",
        "LinkShareLabelRole",
        "NoteEnabledRole",
        "NoteRole",
        "ExpireDateEnabledRole",
        "ExpireDateEnforcedRole",
        "ExpireDateRole",
        "EnforcedMaximumExpireDateRole",
        "PasswordProtectEnabledRole",
        "PasswordRole",
        "PasswordEnforcedRole",
        "EditingAllowedRole",
        "CurrentPermissionModeRole",
        "SharedItemTypeRole",
        "IsSharePermissionsChangeInProgress",
        "HideDownloadEnabledRole",
        "IsHideDownloadEnabledChangeInProgress",
        "ResharingAllowedRole",
        "ShareType",
        "ShareTypeUser",
        "ShareTypeGroup",
        "ShareTypeLink",
        "ShareTypeEmail",
        "ShareTypeRemote",
        "ShareTypeTeam",
        "ShareTypeRoom",
        "ShareTypePlaceholderLink",
        "ShareTypeInternalLink",
        "ShareTypeSecureFileDropPlaceholderLink",
        "SharedItemType",
        "SharedItemTypeUndefined",
        "SharedItemTypeFile",
        "SharedItemTypeFolder",
        "SharedItemTypeEncryptedFile",
        "SharedItemTypeEncryptedFolder",
        "SharedItemTypeEncryptedTopLevelFolder",
        "SharePermissionsMode",
        "ModeViewOnly",
        "ModeUploadAndEditing",
        "ModeFileDropOnly"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'localPathChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountStateChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountConnectedChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sharingEnabledChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'publicLinkSharesEnabledChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userGroupSharingEnabledChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sharePermissionsChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isShareDisabledEncryptedFolderChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lockExpireStringChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fetchOngoingChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasInitialShareFetchCompletedChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareesChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'internalLinkReady'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverAllowsResharingChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displayShareOwnerChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareOwnerDisplayNameChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareOwnerAvatarChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sharedWithMeExpiresChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sharedWithMeRemainingTimeStringChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverError'
        QtMocHelpers::SignalData<void(const int, const QString &) const>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 22 }, { QMetaType::QString, 23 },
        }}),
        // Signal 'passwordSetError'
        QtMocHelpers::SignalData<void(const QString &, const int, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::Int, 22 }, { QMetaType::QString, 23 },
        }}),
        // Signal 'requestPasswordForLinkShare'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestPasswordForEmailSharee'
        QtMocHelpers::SignalData<void(const OCC::ShareePtr &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'sharesChanged'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setAccountState'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'setLocalPath'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 },
        }}),
        // Slot 'createNewLinkShare'
        QtMocHelpers::SlotData<void() const>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createNewLinkShareWithPassword'
        QtMocHelpers::SlotData<void(const QString &) const>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Slot 'createNewUserGroupShare'
        QtMocHelpers::SlotData<void(const OCC::ShareePtr &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'createNewUserGroupShareFromQml'
        QtMocHelpers::SlotData<void(const QVariant &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 29 },
        }}),
        // Slot 'createNewUserGroupShareWithPassword'
        QtMocHelpers::SlotData<void(const OCC::ShareePtr &, const QString &) const>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'createNewUserGroupShareWithPasswordFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const QString &) const>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 29 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'deleteShare'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &) const>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'deleteShareFromQml'
        QtMocHelpers::SlotData<void(const QVariant &) const>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 },
        }}),
        // Slot 'toggleHideDownloadFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleShareAllowEditing'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleShareAllowEditingFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleShareAllowResharing'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleShareAllowResharingFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const bool)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleSharePasswordProtect'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleSharePasswordProtectFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleShareExpirationDate'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const bool) const>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'toggleShareExpirationDateFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const bool) const>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'changePermissionModeFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const OCC::ShareModel::SharePermissionsMode)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { 0x80000000 | 59, 60 },
        }}),
        // Slot 'setLinkShareLabel'
        QtMocHelpers::SlotData<void(const QSharedPointer<OCC::LinkShare> &, const QString &) const>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 63 }, { QMetaType::QString, 64 },
        }}),
        // Slot 'setLinkShareLabelFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const QString &) const>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 63 }, { QMetaType::QString, 64 },
        }}),
        // Slot 'setShareExpireDate'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const qint64) const>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::LongLong, 67 },
        }}),
        // Slot 'setShareExpireDateFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const QVariant) const>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { 0x80000000 | 41, 67 },
        }}),
        // Slot 'setSharePassword'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const QString &)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'setSharePasswordFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const QString &)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'setShareNote'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &, const QString &) const>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::QString, 72 },
        }}),
        // Slot 'setShareNoteFromQml'
        QtMocHelpers::SlotData<void(const QVariant &, const QString &) const>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 46 }, { QMetaType::QString, 72 },
        }}),
        // Slot 'resetData'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateData'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'initShareManager'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handlePlaceholderLinkShare'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleSecureFileDropLinkShare'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleLinkShare'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setupInternalLinkShare'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setSharePermissionChangeInProgress'
        QtMocHelpers::SlotData<void(const QString &, const bool)>(81, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 82 },
        }}),
        // Slot 'setHideDownloadEnabledChangeInProgress'
        QtMocHelpers::SlotData<void(const QString &, const bool)>(83, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 82 },
        }}),
        // Slot 'slotPropfindReceived'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(84, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 85, 86 },
        }}),
        // Slot 'slotAddShare'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &)>(87, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'slotRemoveShareWithId'
        QtMocHelpers::SlotData<void(const QString &)>(88, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotSharesFetched'
        QtMocHelpers::SlotData<void(const QList<OCC::SharePtr> &)>(89, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 90, 91 },
        }}),
        // Slot 'slotSharedWithMeFetched'
        QtMocHelpers::SlotData<void(const QList<OCC::SharePtr> &)>(92, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 90, 91 },
        }}),
        // Slot 'slotAddSharee'
        QtMocHelpers::SlotData<void(const OCC::ShareePtr &)>(93, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotRemoveSharee'
        QtMocHelpers::SlotData<void(const OCC::ShareePtr &)>(94, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotSharePermissionsSet'
        QtMocHelpers::SlotData<void(const QString &)>(95, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotSharePasswordSet'
        QtMocHelpers::SlotData<void(const QString &)>(96, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotShareNoteSet'
        QtMocHelpers::SlotData<void(const QString &)>(97, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotHideDownloadSet'
        QtMocHelpers::SlotData<void(const QString &)>(98, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotShareNameSet'
        QtMocHelpers::SlotData<void(const QString &)>(99, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotShareLabelSet'
        QtMocHelpers::SlotData<void(const QString &)>(100, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotShareExpireDateSet'
        QtMocHelpers::SlotData<void(const QString &)>(101, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotDeleteE2EeShare'
        QtMocHelpers::SlotData<void(const OCC::SharePtr &) const>(102, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Method 'generatePassword'
        QtMocHelpers::MethodData<QString()>(103, 2, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accountState'
        QtMocHelpers::PropertyData<AccountState*>(33, 0x80000000 | 104, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'localPath'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'accountConnected'
        QtMocHelpers::PropertyData<bool>(105, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'sharingEnabled'
        QtMocHelpers::PropertyData<bool>(106, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'publicLinkSharesEnabled'
        QtMocHelpers::PropertyData<bool>(107, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'userGroupSharingEnabled'
        QtMocHelpers::PropertyData<bool>(108, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'canShare'
        QtMocHelpers::PropertyData<bool>(109, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'isShareDisabledEncryptedFolder'
        QtMocHelpers::PropertyData<bool>(110, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'fetchOngoing'
        QtMocHelpers::PropertyData<bool>(111, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
        // property 'hasInitialShareFetchCompleted'
        QtMocHelpers::PropertyData<bool>(112, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
        // property 'serverAllowsResharing'
        QtMocHelpers::PropertyData<bool>(113, QMetaType::Bool, QMC::DefaultPropertyFlags, 13),
        // property 'sharees'
        QtMocHelpers::PropertyData<QVariantList>(114, 0x80000000 | 115, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 11),
        // property 'displayShareOwner'
        QtMocHelpers::PropertyData<bool>(116, QMetaType::Bool, QMC::DefaultPropertyFlags, 14),
        // property 'shareOwnerDisplayName'
        QtMocHelpers::PropertyData<QString>(117, QMetaType::QString, QMC::DefaultPropertyFlags, 15),
        // property 'shareOwnerAvatar'
        QtMocHelpers::PropertyData<QString>(118, QMetaType::QString, QMC::DefaultPropertyFlags, 16),
        // property 'sharedWithMeExpires'
        QtMocHelpers::PropertyData<bool>(119, QMetaType::Bool, QMC::DefaultPropertyFlags, 17),
        // property 'sharedWithMeRemainingTimeString'
        QtMocHelpers::PropertyData<QString>(120, QMetaType::QString, QMC::DefaultPropertyFlags, 18),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(121, 121, QMC::EnumFlags{}).add({
            {  122, Roles::ShareRole },
            {  123, Roles::ShareTypeRole },
            {  124, Roles::ShareIdRole },
            {  125, Roles::IconUrlRole },
            {  126, Roles::AvatarUrlRole },
            {  127, Roles::LinkRole },
            {  128, Roles::LinkShareNameRole },
            {  129, Roles::LinkShareLabelRole },
            {  130, Roles::NoteEnabledRole },
            {  131, Roles::NoteRole },
            {  132, Roles::ExpireDateEnabledRole },
            {  133, Roles::ExpireDateEnforcedRole },
            {  134, Roles::ExpireDateRole },
            {  135, Roles::EnforcedMaximumExpireDateRole },
            {  136, Roles::PasswordProtectEnabledRole },
            {  137, Roles::PasswordRole },
            {  138, Roles::PasswordEnforcedRole },
            {  139, Roles::EditingAllowedRole },
            {  140, Roles::CurrentPermissionModeRole },
            {  141, Roles::SharedItemTypeRole },
            {  142, Roles::IsSharePermissionsChangeInProgress },
            {  143, Roles::HideDownloadEnabledRole },
            {  144, Roles::IsHideDownloadEnabledChangeInProgress },
            {  145, Roles::ResharingAllowedRole },
        }),
        // enum 'ShareType'
        QtMocHelpers::EnumData<enum ShareType>(146, 146, QMC::EnumFlags{}).add({
            {  147, ShareType::ShareTypeUser },
            {  148, ShareType::ShareTypeGroup },
            {  149, ShareType::ShareTypeLink },
            {  150, ShareType::ShareTypeEmail },
            {  151, ShareType::ShareTypeRemote },
            {  152, ShareType::ShareTypeTeam },
            {  153, ShareType::ShareTypeRoom },
            {  154, ShareType::ShareTypePlaceholderLink },
            {  155, ShareType::ShareTypeInternalLink },
            {  156, ShareType::ShareTypeSecureFileDropPlaceholderLink },
        }),
        // enum 'SharedItemType'
        QtMocHelpers::EnumData<enum SharedItemType>(157, 157, QMC::EnumIsScoped).add({
            {  158, SharedItemType::SharedItemTypeUndefined },
            {  159, SharedItemType::SharedItemTypeFile },
            {  160, SharedItemType::SharedItemTypeFolder },
            {  161, SharedItemType::SharedItemTypeEncryptedFile },
            {  162, SharedItemType::SharedItemTypeEncryptedFolder },
            {  163, SharedItemType::SharedItemTypeEncryptedTopLevelFolder },
        }),
        // enum 'SharePermissionsMode'
        QtMocHelpers::EnumData<enum SharePermissionsMode>(164, 164, QMC::EnumIsScoped).add({
            {  165, SharePermissionsMode::ModeViewOnly },
            {  166, SharePermissionsMode::ModeUploadAndEditing },
            {  167, SharePermissionsMode::ModeFileDropOnly },
        }),
    };
    return QtMocHelpers::metaObjectData<ShareModel, qt_meta_tag_ZN3OCC10ShareModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ShareModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10ShareModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10ShareModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10ShareModelE_t>.metaTypes,
    nullptr
} };

void OCC::ShareModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShareModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->localPathChanged(); break;
        case 1: _t->accountStateChanged(); break;
        case 2: _t->accountConnectedChanged(); break;
        case 3: _t->sharingEnabledChanged(); break;
        case 4: _t->publicLinkSharesEnabledChanged(); break;
        case 5: _t->userGroupSharingEnabledChanged(); break;
        case 6: _t->sharePermissionsChanged(); break;
        case 7: _t->isShareDisabledEncryptedFolderChanged(); break;
        case 8: _t->lockExpireStringChanged(); break;
        case 9: _t->fetchOngoingChanged(); break;
        case 10: _t->hasInitialShareFetchCompletedChanged(); break;
        case 11: _t->shareesChanged(); break;
        case 12: _t->internalLinkReady(); break;
        case 13: _t->serverAllowsResharingChanged(); break;
        case 14: _t->displayShareOwnerChanged(); break;
        case 15: _t->shareOwnerDisplayNameChanged(); break;
        case 16: _t->shareOwnerAvatarChanged(); break;
        case 17: _t->sharedWithMeExpiresChanged(); break;
        case 18: _t->sharedWithMeRemainingTimeStringChanged(); break;
        case 19: _t->serverError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->passwordSetError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 21: _t->requestPasswordForLinkShare(); break;
        case 22: _t->requestPasswordForEmailSharee((*reinterpret_cast<std::add_pointer_t<OCC::ShareePtr>>(_a[1]))); break;
        case 23: _t->sharesChanged(); break;
        case 24: _t->setAccountState((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 25: _t->setLocalPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->createNewLinkShare(); break;
        case 27: _t->createNewLinkShareWithPassword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->createNewUserGroupShare((*reinterpret_cast<std::add_pointer_t<OCC::ShareePtr>>(_a[1]))); break;
        case 29: _t->createNewUserGroupShareFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 30: _t->createNewUserGroupShareWithPassword((*reinterpret_cast<std::add_pointer_t<OCC::ShareePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 31: _t->createNewUserGroupShareWithPasswordFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 32: _t->deleteShare((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1]))); break;
        case 33: _t->deleteShareFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 34: _t->toggleHideDownloadFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 35: _t->toggleShareAllowEditing((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 36: _t->toggleShareAllowEditingFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 37: _t->toggleShareAllowResharing((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 38: _t->toggleShareAllowResharingFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 39: _t->toggleSharePasswordProtect((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 40: _t->toggleSharePasswordProtectFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 41: _t->toggleShareExpirationDate((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 42: _t->toggleShareExpirationDateFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 43: _t->changePermissionModeFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ShareModel::SharePermissionsMode>>(_a[2]))); break;
        case 44: _t->setLinkShareLabel((*reinterpret_cast<std::add_pointer_t<QSharedPointer<OCC::LinkShare>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 45: _t->setLinkShareLabelFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 46: _t->setShareExpireDate((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 47: _t->setShareExpireDateFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 48: _t->setSharePassword((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 49: _t->setSharePasswordFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 50: _t->setShareNote((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 51: _t->setShareNoteFromQml((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 52: _t->resetData(); break;
        case 53: _t->updateData(); break;
        case 54: _t->initShareManager(); break;
        case 55: _t->handlePlaceholderLinkShare(); break;
        case 56: _t->handleSecureFileDropLinkShare(); break;
        case 57: _t->handleLinkShare(); break;
        case 58: _t->setupInternalLinkShare(); break;
        case 59: _t->setSharePermissionChangeInProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 60: _t->setHideDownloadEnabledChangeInProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 61: _t->slotPropfindReceived((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 62: _t->slotAddShare((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1]))); break;
        case 63: _t->slotRemoveShareWithId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 64: _t->slotSharesFetched((*reinterpret_cast<std::add_pointer_t<QList<OCC::SharePtr>>>(_a[1]))); break;
        case 65: _t->slotSharedWithMeFetched((*reinterpret_cast<std::add_pointer_t<QList<OCC::SharePtr>>>(_a[1]))); break;
        case 66: _t->slotAddSharee((*reinterpret_cast<std::add_pointer_t<OCC::ShareePtr>>(_a[1]))); break;
        case 67: _t->slotRemoveSharee((*reinterpret_cast<std::add_pointer_t<OCC::ShareePtr>>(_a[1]))); break;
        case 68: _t->slotSharePermissionsSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 69: _t->slotSharePasswordSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 70: _t->slotShareNoteSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 71: _t->slotHideDownloadSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 72: _t->slotShareNameSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 73: _t->slotShareLabelSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 74: _t->slotShareExpireDateSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 75: _t->slotDeleteE2EeShare((*reinterpret_cast<std::add_pointer_t<OCC::SharePtr>>(_a[1]))); break;
        case 76: { QString _r = _t->generatePassword();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ShareePtr >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ShareePtr >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ShareePtr >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 44:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<OCC::LinkShare> >(); break;
            }
            break;
        case 46:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 48:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 50:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 62:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        case 64:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::SharePtr> >(); break;
            }
            break;
        case 65:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::SharePtr> >(); break;
            }
            break;
        case 66:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ShareePtr >(); break;
            }
            break;
        case 67:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ShareePtr >(); break;
            }
            break;
        case 75:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SharePtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::localPathChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::accountStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::accountConnectedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::sharingEnabledChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::publicLinkSharesEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::userGroupSharingEnabledChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::sharePermissionsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::isShareDisabledEncryptedFolderChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::lockExpireStringChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::fetchOngoingChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::hasInitialShareFetchCompletedChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::shareesChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::internalLinkReady, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::serverAllowsResharingChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::displayShareOwnerChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::shareOwnerDisplayNameChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::shareOwnerAvatarChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::sharedWithMeExpiresChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::sharedWithMeRemainingTimeStringChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)(const int , const QString & ) const>(_a, &ShareModel::serverError, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)(const QString & , const int , const QString & )>(_a, &ShareModel::passwordSetError, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::requestPasswordForLinkShare, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)(const OCC::ShareePtr & )>(_a, &ShareModel::requestPasswordForEmailSharee, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShareModel::*)()>(_a, &ShareModel::sharesChanged, 23))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AccountState* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountState**>(_v) = _t->accountState(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->localPath(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->accountConnected(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->sharingEnabled(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->publicLinkSharesEnabled(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->userGroupSharingEnabled(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->canShare(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isShareDisabledEncryptedFolder(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->fetchOngoing(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->hasInitialShareFetchCompleted(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->serverAllowsResharing(); break;
        case 11: *reinterpret_cast<QVariantList*>(_v) = _t->sharees(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->displayShareOwner(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->shareOwnerDisplayName(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->shareOwnerAvatar(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->sharedWithMeExpires(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->sharedWithMeRemainingTimeString(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAccountState(*reinterpret_cast<AccountState**>(_v)); break;
        case 1: _t->setLocalPath(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::ShareModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ShareModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10ShareModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::ShareModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 77)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 77;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 77)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 77;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void OCC::ShareModel::localPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ShareModel::accountStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::ShareModel::accountConnectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::ShareModel::sharingEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::ShareModel::publicLinkSharesEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::ShareModel::userGroupSharingEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::ShareModel::sharePermissionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::ShareModel::isShareDisabledEncryptedFolderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OCC::ShareModel::lockExpireStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OCC::ShareModel::fetchOngoingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void OCC::ShareModel::hasInitialShareFetchCompletedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OCC::ShareModel::shareesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void OCC::ShareModel::internalLinkReady()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void OCC::ShareModel::serverAllowsResharingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void OCC::ShareModel::displayShareOwnerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void OCC::ShareModel::shareOwnerDisplayNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void OCC::ShareModel::shareOwnerAvatarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void OCC::ShareModel::sharedWithMeExpiresChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void OCC::ShareModel::sharedWithMeRemainingTimeStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void OCC::ShareModel::serverError(const int _t1, const QString & _t2)const
{
    QMetaObject::activate<void>(const_cast< OCC::ShareModel *>(this), &staticMetaObject, 19, nullptr, _t1, _t2);
}

// SIGNAL 20
void OCC::ShareModel::passwordSetError(const QString & _t1, const int _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1, _t2, _t3);
}

// SIGNAL 21
void OCC::ShareModel::requestPasswordForLinkShare()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void OCC::ShareModel::requestPasswordForEmailSharee(const OCC::ShareePtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void OCC::ShareModel::sharesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}
QT_WARNING_POP
