/****************************************************************************
** Meta object code from reading C++ file 'propagateupload.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/propagateupload.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propagateupload.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12UploadDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UploadDevice::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12UploadDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UploadDevice",
        "slotJobUploadProgress",
        "",
        "sent",
        "t"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotJobUploadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 3 }, { QMetaType::LongLong, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UploadDevice, qt_meta_tag_ZN3OCC12UploadDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::UploadDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QIODevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12UploadDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12UploadDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12UploadDeviceE_t>.metaTypes,
    nullptr
} };

void OCC::UploadDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UploadDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotJobUploadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::UploadDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::UploadDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12UploadDeviceE_t>.strings))
        return static_cast<void*>(this);
    return QIODevice::qt_metacast(_clname);
}

int OCC::UploadDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QIODevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC10PUTFileJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PUTFileJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10PUTFileJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PUTFileJob",
        "finishedSignal",
        "",
        "uploadProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finishedSignal'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'uploadProgress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PUTFileJob, qt_meta_tag_ZN3OCC10PUTFileJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PUTFileJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractNetworkJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10PUTFileJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10PUTFileJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10PUTFileJobE_t>.metaTypes,
    nullptr
} };

void OCC::PUTFileJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PUTFileJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finishedSignal(); break;
        case 1: _t->uploadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PUTFileJob::*)()>(_a, &PUTFileJob::finishedSignal, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PUTFileJob::*)(qint64 , qint64 )>(_a, &PUTFileJob::uploadProgress, 1))
            return;
    }
}

const QMetaObject *OCC::PUTFileJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PUTFileJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10PUTFileJobE_t>.strings))
        return static_cast<void*>(this);
    return AbstractNetworkJob::qt_metacast(_clname);
}

int OCC::PUTFileJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractNetworkJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::PUTFileJob::finishedSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::PUTFileJob::uploadProgress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN3OCC7PollJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PollJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC7PollJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PollJob",
        "finishedSignal",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finishedSignal'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PollJob, qt_meta_tag_ZN3OCC7PollJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PollJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractNetworkJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7PollJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7PollJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC7PollJobE_t>.metaTypes,
    nullptr
} };

void OCC::PollJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PollJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finishedSignal(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PollJob::*)()>(_a, &PollJob::finishedSignal, 0))
            return;
    }
}

const QMetaObject *OCC::PollJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PollJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7PollJobE_t>.strings))
        return static_cast<void*>(this);
    return AbstractNetworkJob::qt_metacast(_clname);
}

int OCC::PollJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractNetworkJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OCC::PollJob::finishedSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateUploadFileCommon::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateUploadFileCommon",
        "slotComputeContentChecksum",
        "",
        "slotComputeTransmissionChecksum",
        "contentChecksumType",
        "contentChecksum",
        "slotStartUpload",
        "transmissionChecksumType",
        "transmissionChecksum",
        "slotFolderUnlocked",
        "folderId",
        "httpReturnCode",
        "slotOnErrorStartFolderUnlock",
        "OCC::SyncFileItem::Status",
        "status",
        "errorString",
        "slotJobDestroyed",
        "job",
        "slotPollFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotComputeContentChecksum'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotComputeTransmissionChecksum'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Slot 'slotStartUpload'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 }, { QMetaType::QByteArray, 8 },
        }}),
        // Slot 'slotFolderUnlocked'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'slotOnErrorStartFolderUnlock'
        QtMocHelpers::SlotData<void(OCC::SyncFileItem::Status, const QString &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'slotJobDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 17 },
        }}),
        // Slot 'slotPollFinished'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateUploadFileCommon, qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateUploadFileCommon::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateItemJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateUploadFileCommon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateUploadFileCommon *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotComputeContentChecksum(); break;
        case 1: _t->slotComputeTransmissionChecksum((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 2: _t->slotStartUpload((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 3: _t->slotFolderUnlocked((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->slotOnErrorStartFolderUnlock((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->slotJobDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 6: _t->slotPollFinished(); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::PropagateUploadFileCommon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateUploadFileCommon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC25PropagateUploadFileCommonE_t>.strings))
        return static_cast<void*>(this);
    return PropagateItemJob::qt_metacast(_clname);
}

int OCC::PropagateUploadFileCommon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateItemJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateUploadFileV1::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateUploadFileV1",
        "abort",
        "",
        "OCC::PropagatorJob::AbortType",
        "abortType",
        "startNextChunk",
        "slotPutFinished",
        "slotUploadProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'abort'
        QtMocHelpers::SlotData<void(OCC::PropagatorJob::AbortType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'startNextChunk'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPutFinished'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUploadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateUploadFileV1, qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateUploadFileV1::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateUploadFileCommon::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t>.metaTypes,
    nullptr
} };

void OCC::PropagateUploadFileV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateUploadFileV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->abort((*reinterpret_cast<std::add_pointer_t<OCC::PropagatorJob::AbortType>>(_a[1]))); break;
        case 1: _t->startNextChunk(); break;
        case 2: _t->slotPutFinished(); break;
        case 3: _t->slotUploadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::PropagateUploadFileV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateUploadFileV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21PropagateUploadFileV1E_t>.strings))
        return static_cast<void*>(this);
    return PropagateUploadFileCommon::qt_metacast(_clname);
}

int OCC::PropagateUploadFileV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateUploadFileCommon::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PropagateUploadFileNG::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PropagateUploadFileNG",
        "abort",
        "",
        "OCC::PropagatorJob::AbortType",
        "abortType",
        "slotPropfindFinished",
        "slotPropfindFinishedWithError",
        "slotPropfindIterate",
        "name",
        "QMap<QString,QString>",
        "properties",
        "slotDeleteJobFinished",
        "slotMkColFinished",
        "slotPutFinished",
        "slotMoveJobFinished",
        "slotUploadProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'abort'
        QtMocHelpers::SlotData<void(OCC::PropagatorJob::AbortType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotPropfindFinished'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPropfindFinishedWithError'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPropfindIterate'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotDeleteJobFinished'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMkColFinished'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPutFinished'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveJobFinished'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUploadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropagateUploadFileNG, qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PropagateUploadFileNG::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagateUploadFileCommon::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t>.metaTypes,
    nullptr
} };

void OCC::PropagateUploadFileNG::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropagateUploadFileNG *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->abort((*reinterpret_cast<std::add_pointer_t<OCC::PropagatorJob::AbortType>>(_a[1]))); break;
        case 1: _t->slotPropfindFinished(); break;
        case 2: _t->slotPropfindFinishedWithError(); break;
        case 3: _t->slotPropfindIterate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 4: _t->slotDeleteJobFinished(); break;
        case 5: _t->slotMkColFinished(); break;
        case 6: _t->slotPutFinished(); break;
        case 7: _t->slotMoveJobFinished(); break;
        case 8: _t->slotUploadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::PropagateUploadFileNG::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::PropagateUploadFileNG::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC21PropagateUploadFileNGE_t>.strings))
        return static_cast<void*>(this);
    return PropagateUploadFileCommon::qt_metacast(_clname);
}

int OCC::PropagateUploadFileNG::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagateUploadFileCommon::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
