/****************************************************************************
** Meta object code from reading C++ file 'theme.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/theme.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'theme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC5ThemeE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Theme::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC5ThemeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Theme",
        "systrayUseMonoIconsChanged",
        "",
        "systemPaletteChanged",
        "QPalette",
        "palette",
        "darkModeChanged",
        "overrideServerUrlChanged",
        "forceOverrideServerUrlChanged",
        "vfsEnabledChanged",
        "startLoginFlowAutomaticallyChanged",
        "setOverrideServerUrl",
        "overrideServerUrl",
        "setForceOverrideServerUrl",
        "forceOverride",
        "setVfsEnabled",
        "enabled",
        "setStartLoginFlowAutomatically",
        "startLoginFlowAuto",
        "systemPaletteHasChanged",
        "getColorDarkness",
        "QColor",
        "color",
        "isDarkColor",
        "branded",
        "appNameGUI",
        "appName",
        "stateOnlineImageSource",
        "QUrl",
        "stateOfflineImageSource",
        "statusOnlineImageSource",
        "statusDoNotDisturbImageSource",
        "statusAwayImageSource",
        "statusBusyImageSource",
        "statusInvisibleImageSource",
        "folderOfflineIcon",
        "QIcon",
        "applicationIcon",
        "version",
        "helpUrl",
        "conflictHelpUrl",
        "forceOverrideServerUrl",
        "isVfsEnabled",
        "startLoginFlowAutomatically",
        "wizardHeaderTitleColor",
        "wizardHeaderBackgroundColor",
        "updateCheckUrl",
        "defaultColor",
        "systemPalette",
        "QVariantMap",
        "darkMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'systrayUseMonoIconsChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'systemPaletteChanged'
        QtMocHelpers::SignalData<void(const QPalette &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'darkModeChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overrideServerUrlChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'forceOverrideServerUrlChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'vfsEnabledChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startLoginFlowAutomaticallyChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOverrideServerUrl'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'setForceOverrideServerUrl'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setVfsEnabled'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setStartLoginFlowAutomatically'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'systemPaletteHasChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'getColorDarkness'
        QtMocHelpers::MethodData<double(const QColor &)>(20, 2, QMC::AccessPublic, QMetaType::Double, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Method 'isDarkColor'
        QtMocHelpers::MethodData<bool(const QColor &)>(23, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 21, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'branded'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'appNameGUI'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'appName'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stateOnlineImageSource'
        QtMocHelpers::PropertyData<QUrl>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stateOfflineImageSource'
        QtMocHelpers::PropertyData<QUrl>(29, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'statusOnlineImageSource'
        QtMocHelpers::PropertyData<QUrl>(30, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'statusDoNotDisturbImageSource'
        QtMocHelpers::PropertyData<QUrl>(31, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'statusAwayImageSource'
        QtMocHelpers::PropertyData<QUrl>(32, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'statusBusyImageSource'
        QtMocHelpers::PropertyData<QUrl>(33, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'statusInvisibleImageSource'
        QtMocHelpers::PropertyData<QUrl>(34, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'folderOfflineIcon'
        QtMocHelpers::PropertyData<QIcon>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'applicationIcon'
        QtMocHelpers::PropertyData<QIcon>(37, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'version'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'helpUrl'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'conflictHelpUrl'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'overrideServerUrl'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'forceOverrideServerUrl'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isVfsEnabled'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'startLoginFlowAutomatically'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'wizardHeaderTitleColor'
        QtMocHelpers::PropertyData<QColor>(44, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'wizardHeaderBackgroundColor'
        QtMocHelpers::PropertyData<QColor>(45, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'updateCheckUrl'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultColor'
        QtMocHelpers::PropertyData<QColor>(47, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'systemPalette'
        QtMocHelpers::PropertyData<QVariantMap>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'darkMode'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Theme, qt_meta_tag_ZN3OCC5ThemeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Theme::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ThemeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ThemeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC5ThemeE_t>.metaTypes,
    nullptr
} };

void OCC::Theme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Theme *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->systrayUseMonoIconsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->systemPaletteChanged((*reinterpret_cast<std::add_pointer_t<QPalette>>(_a[1]))); break;
        case 2: _t->darkModeChanged(); break;
        case 3: _t->overrideServerUrlChanged(); break;
        case 4: _t->forceOverrideServerUrlChanged(); break;
        case 5: _t->vfsEnabledChanged(); break;
        case 6: _t->startLoginFlowAutomaticallyChanged(); break;
        case 7: _t->setOverrideServerUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->setForceOverrideServerUrl((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setVfsEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setStartLoginFlowAutomatically((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->systemPaletteHasChanged(); break;
        case 12: { double _r = _t->getColorDarkness((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->isDarkColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Theme::*)(bool )>(_a, &Theme::systrayUseMonoIconsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)(const QPalette & )>(_a, &Theme::systemPaletteChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)()>(_a, &Theme::darkModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)()>(_a, &Theme::overrideServerUrlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)()>(_a, &Theme::forceOverrideServerUrlChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)()>(_a, &Theme::vfsEnabledChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Theme::*)()>(_a, &Theme::startLoginFlowAutomaticallyChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isBranded(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->appNameGUI(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->appName(); break;
        case 3: *reinterpret_cast<QUrl*>(_v) = _t->stateOnlineImageSource(); break;
        case 4: *reinterpret_cast<QUrl*>(_v) = _t->stateOfflineImageSource(); break;
        case 5: *reinterpret_cast<QUrl*>(_v) = _t->statusOnlineImageSource(); break;
        case 6: *reinterpret_cast<QUrl*>(_v) = _t->statusDoNotDisturbImageSource(); break;
        case 7: *reinterpret_cast<QUrl*>(_v) = _t->statusAwayImageSource(); break;
        case 8: *reinterpret_cast<QUrl*>(_v) = _t->statusBusyImageSource(); break;
        case 9: *reinterpret_cast<QUrl*>(_v) = _t->statusInvisibleImageSource(); break;
        case 10: *reinterpret_cast<QIcon*>(_v) = _t->folderOfflineIcon(); break;
        case 11: *reinterpret_cast<QIcon*>(_v) = _t->applicationIcon(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->version(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->helpUrl(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->conflictHelpUrl(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->overrideServerUrl(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->forceOverrideServerUrl(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->isVfsEnabled(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->startLoginFlowAutomatically(); break;
        case 19: *reinterpret_cast<QColor*>(_v) = _t->wizardHeaderTitleColor(); break;
        case 20: *reinterpret_cast<QColor*>(_v) = _t->wizardHeaderBackgroundColor(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->updateCheckUrl(); break;
        case 22: *reinterpret_cast<QColor*>(_v) = _t->defaultColor(); break;
        case 23: *reinterpret_cast<QVariantMap*>(_v) = _t->systemPalette(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->darkMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 15: _t->setOverrideServerUrl(*reinterpret_cast<QString*>(_v)); break;
        case 16: _t->setForceOverrideServerUrl(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setVfsEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setStartLoginFlowAutomatically(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Theme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Theme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5ThemeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Theme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void OCC::Theme::systrayUseMonoIconsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::Theme::systemPaletteChanged(const QPalette & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::Theme::darkModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::Theme::overrideServerUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::Theme::forceOverrideServerUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Theme::vfsEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::Theme::startLoginFlowAutomaticallyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
