/****************************************************************************
** Meta object code from reading C++ file 'systray.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/systray.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'systray.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC7SystrayE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Systray::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC7SystrayE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Systray",
        "currentUserChanged",
        "",
        "openAccountWizard",
        "openSettings",
        "openSettingsForSandboxReapproval",
        "openHelp",
        "shutdown",
        "showFileDetailsPage",
        "fileLocalPath",
        "OCC::Systray::FileDetailsPage",
        "page",
        "showFileDetails",
        "OCC::AccountState*",
        "accountState",
        "localPath",
        "fileDetailsPage",
        "sendChatMessage",
        "token",
        "message",
        "replyTo",
        "showErrorMessageDialog",
        "error",
        "syncIsPausedChanged",
        "anySyncFoldersChanged",
        "isOpenChanged",
        "hideSettingsDialog",
        "openUrlInBrowser",
        "QUrl",
        "url",
        "setTrayEngine",
        "QQmlApplicationEngine*",
        "trayEngine",
        "create",
        "showMessage",
        "title",
        "QSystemTrayIcon::MessageIcon",
        "icon",
        "showUpdateMessage",
        "webUrl",
        "showTalkMessage",
        "OCC::AccountStatePtr",
        "createCallDialog",
        "OCC::Activity",
        "callNotification",
        "createEditFileLocallyLoadingDialog",
        "fileName",
        "destroyEditFileLocallyLoadingDialog",
        "createResolveConflictsDialog",
        "OCC::ActivityList",
        "allConflicts",
        "createEncryptionTokenDiscoveryDialog",
        "destroyEncryptionTokenDiscoveryDialog",
        "slotCurrentUserChanged",
        "forceWindowInit",
        "QQuickWindow*",
        "window",
        "positionWindowAtTray",
        "positionWindowAtScreenCenter",
        "positionNotificationWindow",
        "destroyDialog",
        "showWindow",
        "OCC::Systray::WindowPosition",
        "position",
        "hideWindow",
        "setSyncIsPaused",
        "syncIsPaused",
        "setIsOpen",
        "isOpen",
        "createShareDialog",
        "createFileActivityDialog",
        "showFileActionsDialog",
        "presentShareViewInTray",
        "presentFileActionsViewInSystray",
        "slotUpdateSyncPausedState",
        "slotUnpauseAllFolders",
        "slotPauseAllFolders",
        "slotSyncFoldersChanged",
        "OCC::Folder::Map",
        "foldeMap",
        "windowTitle",
        "useNormalWindow",
        "anySyncFolders",
        "enableAddAccount",
        "TaskBarPosition",
        "Bottom",
        "Left",
        "Top",
        "Right",
        "NotificationPosition",
        "Default",
        "TopLeft",
        "TopRight",
        "BottomLeft",
        "BottomRight",
        "WindowPosition",
        "Center",
        "FileDetailsPage",
        "Activity",
        "Sharing"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentUserChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openAccountWizard'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openSettings'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openSettingsForSandboxReapproval'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openHelp'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shutdown'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showFileDetailsPage'
        QtMocHelpers::SignalData<void(const QString &, const OCC::Systray::FileDetailsPage)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'showFileDetails'
        QtMocHelpers::SignalData<void(OCC::AccountState *, const QString &, const OCC::Systray::FileDetailsPage)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 }, { 0x80000000 | 10, 16 },
        }}),
        // Signal 'sendChatMessage'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 20 },
        }}),
        // Signal 'showErrorMessageDialog'
        QtMocHelpers::SignalData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'syncIsPausedChanged'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anySyncFoldersChanged'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isOpenChanged'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideSettingsDialog'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openUrlInBrowser'
        QtMocHelpers::SlotData<bool(const QUrl &) const>(27, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'setTrayEngine'
        QtMocHelpers::SlotData<void(QQmlApplicationEngine *)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'create'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, QSystemTrayIcon::MessageIcon)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 19 }, { 0x80000000 | 36, 37 },
        }}),
        // Slot 'showMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(34, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 19 },
        }}),
        // Slot 'showUpdateMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QUrl &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 19 }, { 0x80000000 | 28, 39 },
        }}),
        // Slot 'showTalkMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const OCC::AccountStatePtr &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 19 }, { QMetaType::QString, 20 }, { QMetaType::QString, 18 },
            { 0x80000000 | 41, 14 },
        }}),
        // Slot 'createCallDialog'
        QtMocHelpers::SlotData<void(const OCC::Activity &, const OCC::AccountStatePtr)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 }, { 0x80000000 | 41, 14 },
        }}),
        // Slot 'createEditFileLocallyLoadingDialog'
        QtMocHelpers::SlotData<void(const QString &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 46 },
        }}),
        // Slot 'destroyEditFileLocallyLoadingDialog'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createResolveConflictsDialog'
        QtMocHelpers::SlotData<void(const OCC::ActivityList &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 49, 50 },
        }}),
        // Slot 'createEncryptionTokenDiscoveryDialog'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'destroyEncryptionTokenDiscoveryDialog'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCurrentUserChanged'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forceWindowInit'
        QtMocHelpers::SlotData<void(QQuickWindow *) const>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'positionWindowAtTray'
        QtMocHelpers::SlotData<void(QQuickWindow *) const>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'positionWindowAtScreenCenter'
        QtMocHelpers::SlotData<void(QQuickWindow *) const>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'positionNotificationWindow'
        QtMocHelpers::SlotData<void(QQuickWindow *) const>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'destroyDialog'
        QtMocHelpers::SlotData<void(QQuickWindow *) const>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'showWindow'
        QtMocHelpers::SlotData<void(OCC::Systray::WindowPosition)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 63 },
        }}),
        // Slot 'showWindow'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'hideWindow'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSyncIsPaused'
        QtMocHelpers::SlotData<void(const bool)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 66 },
        }}),
        // Slot 'setIsOpen'
        QtMocHelpers::SlotData<void(const bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 68 },
        }}),
        // Slot 'createShareDialog'
        QtMocHelpers::SlotData<void(const QString &)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'createFileActivityDialog'
        QtMocHelpers::SlotData<void(const QString &)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'showFileActionsDialog'
        QtMocHelpers::SlotData<void(const QString &)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'presentShareViewInTray'
        QtMocHelpers::SlotData<void(const QString &)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'presentFileActionsViewInSystray'
        QtMocHelpers::SlotData<void(const QString &)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'slotUpdateSyncPausedState'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUnpauseAllFolders'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPauseAllFolders'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSyncFoldersChanged'
        QtMocHelpers::SlotData<void(const OCC::Folder::Map &)>(77, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 78, 79 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'windowTitle'
        QtMocHelpers::PropertyData<QString>(80, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'useNormalWindow'
        QtMocHelpers::PropertyData<bool>(81, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'syncIsPaused'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'anySyncFolders'
        QtMocHelpers::PropertyData<bool>(82, QMetaType::Bool, QMC::DefaultPropertyFlags, 11),
        // property 'isOpen'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'enableAddAccount'
        QtMocHelpers::PropertyData<bool>(83, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TaskBarPosition'
        QtMocHelpers::EnumData<enum TaskBarPosition>(84, 84, QMC::EnumIsScoped).add({
            {   85, TaskBarPosition::Bottom },
            {   86, TaskBarPosition::Left },
            {   87, TaskBarPosition::Top },
            {   88, TaskBarPosition::Right },
        }),
        // enum 'NotificationPosition'
        QtMocHelpers::EnumData<enum NotificationPosition>(89, 89, QMC::EnumIsScoped).add({
            {   90, NotificationPosition::Default },
            {   91, NotificationPosition::TopLeft },
            {   92, NotificationPosition::TopRight },
            {   93, NotificationPosition::BottomLeft },
            {   94, NotificationPosition::BottomRight },
        }),
        // enum 'WindowPosition'
        QtMocHelpers::EnumData<enum WindowPosition>(95, 95, QMC::EnumIsScoped).add({
            {   90, WindowPosition::Default },
            {   96, WindowPosition::Center },
        }),
        // enum 'FileDetailsPage'
        QtMocHelpers::EnumData<enum FileDetailsPage>(97, 97, QMC::EnumIsScoped).add({
            {   98, FileDetailsPage::Activity },
            {   99, FileDetailsPage::Sharing },
        }),
    };
    return QtMocHelpers::metaObjectData<Systray, qt_meta_tag_ZN3OCC7SystrayE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Systray::staticMetaObject = { {
    QMetaObject::SuperData::link<QSystemTrayIcon::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7SystrayE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7SystrayE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC7SystrayE_t>.metaTypes,
    nullptr
} };

void OCC::Systray::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Systray *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentUserChanged(); break;
        case 1: _t->openAccountWizard(); break;
        case 2: _t->openSettings(); break;
        case 3: _t->openSettingsForSandboxReapproval(); break;
        case 4: _t->openHelp(); break;
        case 5: _t->shutdown(); break;
        case 6: _t->showFileDetailsPage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::Systray::FileDetailsPage>>(_a[2]))); break;
        case 7: _t->showFileDetails((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<OCC::Systray::FileDetailsPage>>(_a[3]))); break;
        case 8: _t->sendChatMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 9: _t->showErrorMessageDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->syncIsPausedChanged(); break;
        case 11: _t->anySyncFoldersChanged(); break;
        case 12: _t->isOpenChanged(); break;
        case 13: _t->hideSettingsDialog(); break;
        case 14: { bool _r = _t->openUrlInBrowser((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setTrayEngine((*reinterpret_cast<std::add_pointer_t<QQmlApplicationEngine*>>(_a[1]))); break;
        case 16: _t->create(); break;
        case 17: _t->showMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::MessageIcon>>(_a[3]))); break;
        case 18: _t->showMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 19: _t->showUpdateMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 20: _t->showTalkMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<OCC::AccountStatePtr>>(_a[5]))); break;
        case 21: _t->createCallDialog((*reinterpret_cast<std::add_pointer_t<OCC::Activity>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::AccountStatePtr>>(_a[2]))); break;
        case 22: _t->createEditFileLocallyLoadingDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->destroyEditFileLocallyLoadingDialog(); break;
        case 24: _t->createResolveConflictsDialog((*reinterpret_cast<std::add_pointer_t<OCC::ActivityList>>(_a[1]))); break;
        case 25: _t->createEncryptionTokenDiscoveryDialog(); break;
        case 26: _t->destroyEncryptionTokenDiscoveryDialog(); break;
        case 27: _t->slotCurrentUserChanged(); break;
        case 28: _t->forceWindowInit((*reinterpret_cast<std::add_pointer_t<QQuickWindow*>>(_a[1]))); break;
        case 29: _t->positionWindowAtTray((*reinterpret_cast<std::add_pointer_t<QQuickWindow*>>(_a[1]))); break;
        case 30: _t->positionWindowAtScreenCenter((*reinterpret_cast<std::add_pointer_t<QQuickWindow*>>(_a[1]))); break;
        case 31: _t->positionNotificationWindow((*reinterpret_cast<std::add_pointer_t<QQuickWindow*>>(_a[1]))); break;
        case 32: _t->destroyDialog((*reinterpret_cast<std::add_pointer_t<QQuickWindow*>>(_a[1]))); break;
        case 33: _t->showWindow((*reinterpret_cast<std::add_pointer_t<OCC::Systray::WindowPosition>>(_a[1]))); break;
        case 34: _t->showWindow(); break;
        case 35: _t->hideWindow(); break;
        case 36: _t->setSyncIsPaused((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->setIsOpen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->createShareDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->createFileActivityDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 40: _t->showFileActionsDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->presentShareViewInTray((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->presentFileActionsViewInSystray((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->slotUpdateSyncPausedState(); break;
        case 44: _t->slotUnpauseAllFolders(); break;
        case 45: _t->slotPauseAllFolders(); break;
        case 46: _t->slotSyncFoldersChanged((*reinterpret_cast<std::add_pointer_t<OCC::Folder::Map>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountStatePtr >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountStatePtr >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Activity >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ActivityList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::currentUserChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::openAccountWizard, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::openSettings, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::openSettingsForSandboxReapproval, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::openHelp, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::shutdown, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)(const QString & , const OCC::Systray::FileDetailsPage )>(_a, &Systray::showFileDetailsPage, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)(OCC::AccountState * , const QString & , const OCC::Systray::FileDetailsPage )>(_a, &Systray::showFileDetails, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)(const QString & , const QString & , const QString & )>(_a, &Systray::sendChatMessage, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)(const QString & )>(_a, &Systray::showErrorMessageDialog, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::syncIsPausedChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::anySyncFoldersChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::isOpenChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Systray::*)()>(_a, &Systray::hideSettingsDialog, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->windowTitle(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->useNormalWindow(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->syncIsPaused(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->anySyncFolders(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isOpen(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->enableAddAccount(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setSyncIsPaused(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setIsOpen(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Systray::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Systray::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC7SystrayE_t>.strings))
        return static_cast<void*>(this);
    return QSystemTrayIcon::qt_metacast(_clname);
}

int OCC::Systray::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSystemTrayIcon::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void OCC::Systray::currentUserChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Systray::openAccountWizard()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Systray::openSettings()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::Systray::openSettingsForSandboxReapproval()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::Systray::openHelp()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Systray::shutdown()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::Systray::showFileDetailsPage(const QString & _t1, const OCC::Systray::FileDetailsPage _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void OCC::Systray::showFileDetails(OCC::AccountState * _t1, const QString & _t2, const OCC::Systray::FileDetailsPage _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void OCC::Systray::sendChatMessage(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void OCC::Systray::showErrorMessageDialog(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void OCC::Systray::syncIsPausedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OCC::Systray::anySyncFoldersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void OCC::Systray::isOpenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void OCC::Systray::hideSettingsDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}
QT_WARNING_POP
