/****************************************************************************
** Meta object code from reading C++ file 'activitylistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/tray/activitylistmodel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'activitylistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC17ActivityListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ActivityListModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17ActivityListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ActivityListModel",
        "accountStateChanged",
        "",
        "hasSyncConflictsChanged",
        "allConflictsChanged",
        "activityJobStatusCode",
        "statusCode",
        "sendNotificationRequest",
        "accountName",
        "link",
        "verb",
        "row",
        "interactiveActivityReceived",
        "showSettingsDialog",
        "fetchMore",
        "QModelIndex",
        "slotRefreshActivity",
        "slotRefreshActivityInitial",
        "slotRemoveAccount",
        "slotTriggerDefaultAction",
        "activityIndex",
        "slotTriggerAction",
        "actionIndex",
        "slotTriggerDismiss",
        "addNotificationToActivityList",
        "OCC::Activity",
        "activity",
        "addErrorToActivityList",
        "OCC::ActivityListModel::ErrorType",
        "type",
        "addSyncFileItemToActivityList",
        "removeActivityFromActivityList",
        "removeOutdatedNotifications",
        "OCC::ActivityList",
        "receivedNotifications",
        "setAccountState",
        "OCC::AccountState*",
        "state",
        "setReplyMessageSent",
        "message",
        "setCurrentItem",
        "currentItem",
        "activitiesReceived",
        "QJsonDocument",
        "json",
        "setAndRefreshCurrentlyFetching",
        "value",
        "setDoneFetching",
        "setHideOldActivities",
        "setDisplayActions",
        "startFetchJob",
        "addEntriesToActivityList",
        "activityList",
        "accountStateHasChanged",
        "ingestActivities",
        "QJsonArray",
        "activities",
        "appendMoreActivitiesAvailableEntry",
        "insertOrRemoveDummyFetchingActivity",
        "triggerCaseClashAction",
        "maxActionButtons",
        "accountState",
        "AccountState*",
        "hasSyncConflicts",
        "allConflicts",
        "DataRole",
        "IconRole",
        "AccountRole",
        "ObjectTypeRole",
        "ObjectIdRole",
        "ObjectNameRole",
        "ActionsLinksRole",
        "ActionsLinksContextMenuRole",
        "ActionsLinksForActionButtonsRole",
        "ActionTextRole",
        "ActionTextColorRole",
        "ActionRole",
        "MessageRole",
        "DisplayPathRole",
        "PathRole",
        "OpenablePathRole",
        "DisplayLocationRole",
        "LinkRole",
        "PointInTimeRole",
        "AccountConnectedRole",
        "DisplayActions",
        "ShowFileDetailsRole",
        "ShareableRole",
        "DismissableRole",
        "IsCurrentUserFileActivityRole",
        "ThumbnailRole",
        "TalkNotificationConversationTokenRole",
        "TalkNotificationMessageIdRole",
        "TalkNotificationMessageSentRole",
        "TalkNotificationUserAvatarRole",
        "ActivityIndexRole",
        "ActivityRole",
        "ActivityIntegrationRole",
        "ErrorType",
        "SyncError",
        "NetworkError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountStateChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasSyncConflictsChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allConflictsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activityJobStatusCode'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'sendNotificationRequest'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QByteArray &, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QByteArray, 10 }, { QMetaType::Int, 11 },
        }}),
        // Signal 'interactiveActivityReceived'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showSettingsDialog'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchMore'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'slotRefreshActivity'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRefreshActivityInitial'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRemoveAccount'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTriggerDefaultAction'
        QtMocHelpers::SlotData<void(const int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'slotTriggerAction'
        QtMocHelpers::SlotData<void(const int, const int)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'slotTriggerDismiss'
        QtMocHelpers::SlotData<void(const int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'addNotificationToActivityList'
        QtMocHelpers::SlotData<void(const OCC::Activity &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'addErrorToActivityList'
        QtMocHelpers::SlotData<void(const OCC::Activity &, const OCC::ActivityListModel::ErrorType)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'addSyncFileItemToActivityList'
        QtMocHelpers::SlotData<void(const OCC::Activity &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'removeActivityFromActivityList'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'removeActivityFromActivityList'
        QtMocHelpers::SlotData<void(const OCC::Activity &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'removeOutdatedNotifications'
        QtMocHelpers::SlotData<void(const OCC::ActivityList &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'setAccountState'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'setReplyMessageSent'
        QtMocHelpers::SlotData<void(const int, const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::QString, 39 },
        }}),
        // Slot 'setCurrentItem'
        QtMocHelpers::SlotData<void(const int)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 41 },
        }}),
        // Slot 'activitiesReceived'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 43, 44 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'setAndRefreshCurrentlyFetching'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 46 },
        }}),
        // Slot 'setDoneFetching'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 46 },
        }}),
        // Slot 'setHideOldActivities'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 46 },
        }}),
        // Slot 'setDisplayActions'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 46 },
        }}),
        // Slot 'startFetchJob'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addEntriesToActivityList'
        QtMocHelpers::SlotData<void(const OCC::ActivityList &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 52 },
        }}),
        // Slot 'accountStateHasChanged'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ingestActivities'
        QtMocHelpers::SlotData<void(const QJsonArray &)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'appendMoreActivitiesAvailableEntry'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'insertOrRemoveDummyFetchingActivity'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'triggerCaseClashAction'
        QtMocHelpers::SlotData<void(OCC::Activity)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'maxActionButtons'
        QtMocHelpers::PropertyData<quint32>(60, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'accountState'
        QtMocHelpers::PropertyData<AccountState*>(61, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'hasSyncConflicts'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'allConflicts'
        QtMocHelpers::PropertyData<OCC::ActivityList>(64, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DataRole'
        QtMocHelpers::EnumData<enum DataRole>(65, 65, QMC::EnumFlags{}).add({
            {   66, DataRole::IconRole },
            {   67, DataRole::AccountRole },
            {   68, DataRole::ObjectTypeRole },
            {   69, DataRole::ObjectIdRole },
            {   70, DataRole::ObjectNameRole },
            {   71, DataRole::ActionsLinksRole },
            {   72, DataRole::ActionsLinksContextMenuRole },
            {   73, DataRole::ActionsLinksForActionButtonsRole },
            {   74, DataRole::ActionTextRole },
            {   75, DataRole::ActionTextColorRole },
            {   76, DataRole::ActionRole },
            {   77, DataRole::MessageRole },
            {   78, DataRole::DisplayPathRole },
            {   79, DataRole::PathRole },
            {   80, DataRole::OpenablePathRole },
            {   81, DataRole::DisplayLocationRole },
            {   82, DataRole::LinkRole },
            {   83, DataRole::PointInTimeRole },
            {   84, DataRole::AccountConnectedRole },
            {   85, DataRole::DisplayActions },
            {   86, DataRole::ShowFileDetailsRole },
            {   87, DataRole::ShareableRole },
            {   88, DataRole::DismissableRole },
            {   89, DataRole::IsCurrentUserFileActivityRole },
            {   90, DataRole::ThumbnailRole },
            {   91, DataRole::TalkNotificationConversationTokenRole },
            {   92, DataRole::TalkNotificationMessageIdRole },
            {   93, DataRole::TalkNotificationMessageSentRole },
            {   94, DataRole::TalkNotificationUserAvatarRole },
            {   95, DataRole::ActivityIndexRole },
            {   96, DataRole::ActivityRole },
            {   97, DataRole::ActivityIntegrationRole },
        }),
        // enum 'ErrorType'
        QtMocHelpers::EnumData<enum ErrorType>(98, 98, QMC::EnumIsScoped).add({
            {   99, ErrorType::SyncError },
            {  100, ErrorType::NetworkError },
        }),
    };
    return QtMocHelpers::metaObjectData<ActivityListModel, qt_meta_tag_ZN3OCC17ActivityListModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC17ActivityListModelE[] = {
    QMetaObject::SuperData::link<OCC::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::ActivityListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17ActivityListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17ActivityListModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC17ActivityListModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17ActivityListModelE_t>.metaTypes,
    nullptr
} };

void OCC::ActivityListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ActivityListModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountStateChanged(); break;
        case 1: _t->hasSyncConflictsChanged(); break;
        case 2: _t->allConflictsChanged(); break;
        case 3: _t->activityJobStatusCode((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->sendNotificationRequest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 5: _t->interactiveActivityReceived(); break;
        case 6: _t->showSettingsDialog(); break;
        case 7: _t->fetchMore((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->slotRefreshActivity(); break;
        case 9: _t->slotRefreshActivityInitial(); break;
        case 10: _t->slotRemoveAccount(); break;
        case 11: _t->slotTriggerDefaultAction((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotTriggerAction((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->slotTriggerDismiss((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->addNotificationToActivityList((*reinterpret_cast<std::add_pointer_t<OCC::Activity>>(_a[1]))); break;
        case 15: _t->addErrorToActivityList((*reinterpret_cast<std::add_pointer_t<OCC::Activity>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ActivityListModel::ErrorType>>(_a[2]))); break;
        case 16: _t->addSyncFileItemToActivityList((*reinterpret_cast<std::add_pointer_t<OCC::Activity>>(_a[1]))); break;
        case 17: _t->removeActivityFromActivityList((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->removeActivityFromActivityList((*reinterpret_cast<std::add_pointer_t<OCC::Activity>>(_a[1]))); break;
        case 19: _t->removeOutdatedNotifications((*reinterpret_cast<std::add_pointer_t<OCC::ActivityList>>(_a[1]))); break;
        case 20: _t->setAccountState((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 21: _t->setReplyMessageSent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->setCurrentItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->activitiesReceived((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 24: _t->setAndRefreshCurrentlyFetching((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->setDoneFetching((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->setHideOldActivities((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->setDisplayActions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->startFetchJob(); break;
        case 29: _t->addEntriesToActivityList((*reinterpret_cast<std::add_pointer_t<OCC::ActivityList>>(_a[1]))); break;
        case 30: _t->accountStateHasChanged(); break;
        case 31: _t->ingestActivities((*reinterpret_cast<std::add_pointer_t<QJsonArray>>(_a[1]))); break;
        case 32: _t->appendMoreActivitiesAvailableEntry(); break;
        case 33: _t->insertOrRemoveDummyFetchingActivity(); break;
        case 34: _t->triggerCaseClashAction((*reinterpret_cast<std::add_pointer_t<OCC::Activity>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Activity >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Activity >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Activity >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Activity >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ActivityList >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::ActivityList >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::Activity >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)()>(_a, &ActivityListModel::accountStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)()>(_a, &ActivityListModel::hasSyncConflictsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)()>(_a, &ActivityListModel::allConflictsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)(int )>(_a, &ActivityListModel::activityJobStatusCode, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)(const QString & , const QString & , const QByteArray & , int )>(_a, &ActivityListModel::sendNotificationRequest, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)()>(_a, &ActivityListModel::interactiveActivityReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActivityListModel::*)()>(_a, &ActivityListModel::showSettingsDialog, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< OCC::ActivityList >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<quint32*>(_v) = _t->maxActionButtons(); break;
        case 1: *reinterpret_cast<AccountState**>(_v) = _t->accountState(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasSyncConflicts(); break;
        case 3: *reinterpret_cast<OCC::ActivityList*>(_v) = _t->allConflicts(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setAccountState(*reinterpret_cast<AccountState**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::ActivityListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ActivityListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17ActivityListModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::ActivityListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OCC::ActivityListModel::accountStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ActivityListModel::hasSyncConflictsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::ActivityListModel::allConflictsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::ActivityListModel::activityJobStatusCode(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::ActivityListModel::sendNotificationRequest(const QString & _t1, const QString & _t2, const QByteArray & _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void OCC::ActivityListModel::interactiveActivityReceived()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::ActivityListModel::showSettingsDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
