/****************************************************************************
** Meta object code from reading C++ file 'foldermetadata.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/foldermetadata.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'foldermetadata.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14FolderMetadataE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderMetadata::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14FolderMetadataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderMetadata",
        "setupComplete",
        "",
        "addEncryptedFile",
        "OCC::FolderMetadata::EncryptedFile",
        "f",
        "removeEncryptedFile",
        "removeAllEncryptedFiles",
        "initMetadata",
        "initEmptyMetadata",
        "initEmptyMetadataLegacy",
        "setupExistingMetadata",
        "metadata",
        "setupExistingMetadataLegacy",
        "startFetchRootE2eeFolderMetadata",
        "path",
        "slotRootE2eeFolderMetadataReceived",
        "statusCode",
        "message",
        "updateUsersEncryptedMetadataKey",
        "createNewMetadataKeyForEncryption",
        "emitSetupComplete",
        "FolderType",
        "Nested",
        "Root",
        "CertificateType",
        "SoftwareNextcloudCertificate",
        "HardwareCertificate",
        "MetadataVersion",
        "VersionUndefined",
        "Version1",
        "Version1_2",
        "Version2_0",
        "Version2_1"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setupComplete'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addEncryptedFile'
        QtMocHelpers::SlotData<void(const OCC::FolderMetadata::EncryptedFile &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'removeEncryptedFile'
        QtMocHelpers::SlotData<void(const OCC::FolderMetadata::EncryptedFile &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'removeAllEncryptedFiles'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initMetadata'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'initEmptyMetadata'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'initEmptyMetadataLegacy'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setupExistingMetadata'
        QtMocHelpers::SlotData<void(const QByteArray &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 12 },
        }}),
        // Slot 'setupExistingMetadataLegacy'
        QtMocHelpers::SlotData<void(const QByteArray &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 12 },
        }}),
        // Slot 'startFetchRootE2eeFolderMetadata'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'slotRootE2eeFolderMetadataReceived'
        QtMocHelpers::SlotData<void(int, const QString &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'updateUsersEncryptedMetadataKey'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createNewMetadataKeyForEncryption'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'emitSetupComplete'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FolderType'
        QtMocHelpers::EnumData<enum FolderType>(22, 22, QMC::EnumIsScoped).add({
            {   23, FolderType::Nested },
            {   24, FolderType::Root },
        }),
        // enum 'CertificateType'
        QtMocHelpers::EnumData<enum CertificateType>(25, 25, QMC::EnumIsScoped).add({
            {   26, CertificateType::SoftwareNextcloudCertificate },
            {   27, CertificateType::HardwareCertificate },
        }),
        // enum 'MetadataVersion'
        QtMocHelpers::EnumData<enum MetadataVersion>(28, 28, QMC::EnumIsScoped).add({
            {   29, MetadataVersion::VersionUndefined },
            {   30, MetadataVersion::Version1 },
            {   31, MetadataVersion::Version1_2 },
            {   32, MetadataVersion::Version2_0 },
            {   33, MetadataVersion::Version2_1 },
        }),
    };
    return QtMocHelpers::metaObjectData<FolderMetadata, qt_meta_tag_ZN3OCC14FolderMetadataE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderMetadata::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14FolderMetadataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14FolderMetadataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14FolderMetadataE_t>.metaTypes,
    nullptr
} };

void OCC::FolderMetadata::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderMetadata *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setupComplete(); break;
        case 1: _t->addEncryptedFile((*reinterpret_cast<std::add_pointer_t<OCC::FolderMetadata::EncryptedFile>>(_a[1]))); break;
        case 2: _t->removeEncryptedFile((*reinterpret_cast<std::add_pointer_t<OCC::FolderMetadata::EncryptedFile>>(_a[1]))); break;
        case 3: _t->removeAllEncryptedFiles(); break;
        case 4: _t->initMetadata(); break;
        case 5: _t->initEmptyMetadata(); break;
        case 6: _t->initEmptyMetadataLegacy(); break;
        case 7: _t->setupExistingMetadata((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 8: _t->setupExistingMetadataLegacy((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 9: _t->startFetchRootE2eeFolderMetadata((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->slotRootE2eeFolderMetadataReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->updateUsersEncryptedMetadataKey(); break;
        case 12: _t->createNewMetadataKeyForEncryption(); break;
        case 13: _t->emitSetupComplete(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FolderMetadata::*)()>(_a, &FolderMetadata::setupComplete, 0))
            return;
    }
}

const QMetaObject *OCC::FolderMetadata::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderMetadata::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14FolderMetadataE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::FolderMetadata::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void OCC::FolderMetadata::setupComplete()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
