/****************************************************************************
** Meta object code from reading C++ file 'updatee2eefolderusersmetadatajob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/updatee2eefolderusersmetadatajob.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'updatee2eefolderusersmetadatajob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UpdateE2eeFolderUsersMetadataJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UpdateE2eeFolderUsersMetadataJob",
        "certificateReady",
        "",
        "finished",
        "code",
        "message",
        "folderUnlocked",
        "start",
        "keepLock",
        "setUserData",
        "OCC::UpdateE2eeFolderUsersMetadataJob::UserData",
        "userData",
        "setFolderToken",
        "folderToken",
        "setMetadataKeyForEncryption",
        "metadataKey",
        "setMetadataKeyForDecryption",
        "setKeyChecksums",
        "QSet<QByteArray>",
        "keyChecksums",
        "setSubJobSyncItems",
        "QHash<QString,OCC::SyncFileItemPtr>",
        "subJobSyncItems",
        "slotStartE2eeMetadataJobs",
        "slotFetchMetadataJobFinished",
        "statusCode",
        "slotSubJobFinished",
        "slotFolderUnlocked",
        "folderId",
        "httpStatus",
        "slotUpdateMetadataFinished",
        "slotCertificatesFetchedFromServer",
        "QHash<QString,OCC::NextcloudSslCertificate>",
        "results",
        "slotCertificateFetchedFromKeychain",
        "QSslCertificate",
        "certificate"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'certificateReady'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'folderUnlocked'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<void(const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setUserData'
        QtMocHelpers::SlotData<void(const OCC::UpdateE2eeFolderUsersMetadataJob::UserData &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setFolderToken'
        QtMocHelpers::SlotData<void(const QByteArray &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 13 },
        }}),
        // Slot 'setMetadataKeyForEncryption'
        QtMocHelpers::SlotData<void(const QByteArray &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Slot 'setMetadataKeyForDecryption'
        QtMocHelpers::SlotData<void(const QByteArray &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Slot 'setKeyChecksums'
        QtMocHelpers::SlotData<void(const QSet<QByteArray> &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'setSubJobSyncItems'
        QtMocHelpers::SlotData<void(const QHash<QString,OCC::SyncFileItemPtr> &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotStartE2eeMetadataJobs'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFetchMetadataJobFinished'
        QtMocHelpers::SlotData<void(int, const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'slotSubJobFinished'
        QtMocHelpers::SlotData<void(int, const QString &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'slotSubJobFinished'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'slotFolderUnlocked'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 28 }, { QMetaType::Int, 29 },
        }}),
        // Slot 'slotUpdateMetadataFinished'
        QtMocHelpers::SlotData<void(int, const QString &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'slotUpdateMetadataFinished'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'slotCertificatesFetchedFromServer'
        QtMocHelpers::SlotData<void(const QHash<QString,OCC::NextcloudSslCertificate> &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotCertificateFetchedFromKeychain'
        QtMocHelpers::SlotData<void(const QSslCertificate &)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UpdateE2eeFolderUsersMetadataJob, qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::UpdateE2eeFolderUsersMetadataJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t>.metaTypes,
    nullptr
} };

void OCC::UpdateE2eeFolderUsersMetadataJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UpdateE2eeFolderUsersMetadataJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->certificateReady(); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->folderUnlocked(); break;
        case 4: _t->start((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->start(); break;
        case 6: _t->setUserData((*reinterpret_cast<std::add_pointer_t<OCC::UpdateE2eeFolderUsersMetadataJob::UserData>>(_a[1]))); break;
        case 7: _t->setFolderToken((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 8: _t->setMetadataKeyForEncryption((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 9: _t->setMetadataKeyForDecryption((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 10: _t->setKeyChecksums((*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[1]))); break;
        case 11: _t->setSubJobSyncItems((*reinterpret_cast<std::add_pointer_t<QHash<QString,OCC::SyncFileItemPtr>>>(_a[1]))); break;
        case 12: _t->slotStartE2eeMetadataJobs(); break;
        case 13: _t->slotFetchMetadataJobFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->slotSubJobFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->slotSubJobFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotFolderUnlocked((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->slotUpdateMetadataFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->slotUpdateMetadataFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->slotCertificatesFetchedFromServer((*reinterpret_cast<std::add_pointer_t<QHash<QString,OCC::NextcloudSslCertificate>>>(_a[1]))); break;
        case 20: _t->slotCertificateFetchedFromKeychain((*reinterpret_cast<std::add_pointer_t<QSslCertificate>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslCertificate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UpdateE2eeFolderUsersMetadataJob::*)()>(_a, &UpdateE2eeFolderUsersMetadataJob::certificateReady, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UpdateE2eeFolderUsersMetadataJob::*)(int , const QString & )>(_a, &UpdateE2eeFolderUsersMetadataJob::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UpdateE2eeFolderUsersMetadataJob::*)()>(_a, &UpdateE2eeFolderUsersMetadataJob::folderUnlocked, 3))
            return;
    }
}

const QMetaObject *OCC::UpdateE2eeFolderUsersMetadataJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::UpdateE2eeFolderUsersMetadataJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC32UpdateE2eeFolderUsersMetadataJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::UpdateE2eeFolderUsersMetadataJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void OCC::UpdateE2eeFolderUsersMetadataJob::certificateReady()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::UpdateE2eeFolderUsersMetadataJob::finished(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::UpdateE2eeFolderUsersMetadataJob::folderUnlocked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
