/****************************************************************************
** Meta object code from reading C++ file 'activitydata.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/tray/activitydata.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'activitydata.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12ActivityLinkE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ActivityLink::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12ActivityLinkE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ActivityLink",
        "imageSource",
        "imageSourceHovered",
        "label",
        "link",
        "verb",
        "primary"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'imageSource'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'imageSourceHovered'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'link'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'verb'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'primary'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ActivityLink, qt_meta_tag_ZN3OCC12ActivityLinkE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ActivityLink::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ActivityLinkE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ActivityLinkE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12ActivityLinkE_t>.metaTypes,
    nullptr
} };

void OCC::ActivityLink::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ActivityLink *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->_imageSource; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->_imageSourceHovered; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->_label; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->_link; break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->_verb; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->_primary; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->_imageSource, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->_imageSourceHovered, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->_label, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->_link, *reinterpret_cast<QString*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->_verb, *reinterpret_cast<QByteArray*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->_primary, *reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC11PreviewDataE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::PreviewData::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC11PreviewDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::PreviewData",
        "source",
        "link",
        "mimeType",
        "fileId",
        "view",
        "isMimeTypeIcon",
        "filename"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'source'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'link'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'mimeType'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'fileId'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'view'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'isMimeTypeIcon'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'filename'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PreviewData, qt_meta_tag_ZN3OCC11PreviewDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::PreviewData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11PreviewDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC11PreviewDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC11PreviewDataE_t>.metaTypes,
    nullptr
} };

void OCC::PreviewData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PreviewData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->_source; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->_link; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->_mimeType; break;
        case 3: *reinterpret_cast<int*>(_v) = _t->_fileId; break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->_view; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->_isMimeTypeIcon; break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->_filename; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->_source, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->_link, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->_mimeType, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->_fileId, *reinterpret_cast<int*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->_view, *reinterpret_cast<QString*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->_isMimeTypeIcon, *reinterpret_cast<bool*>(_v)); break;
        case 6: QtMocHelpers::setProperty(_t->_filename, *reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC20RichSubjectParameterE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::RichSubjectParameter::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC20RichSubjectParameterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::RichSubjectParameter",
        "type",
        "id",
        "name",
        "path",
        "link",
        "QUrl"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'id'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'path'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'link'
        QtMocHelpers::PropertyData<QUrl>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RichSubjectParameter, qt_meta_tag_ZN3OCC20RichSubjectParameterE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::RichSubjectParameter::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20RichSubjectParameterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20RichSubjectParameterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC20RichSubjectParameterE_t>.metaTypes,
    nullptr
} };

void OCC::RichSubjectParameter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RichSubjectParameter *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->type; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->id; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->name; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->path; break;
        case 4: *reinterpret_cast<QUrl*>(_v) = _t->link; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->type, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->id, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->name, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->path, *reinterpret_cast<QString*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->link, *reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC20TalkNotificationDataE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::TalkNotificationData::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC20TalkNotificationDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::TalkNotificationData",
        "conversationToken",
        "messageId",
        "messageSent",
        "userAvatar"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'conversationToken'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'messageId'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'messageSent'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'userAvatar'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TalkNotificationData, qt_meta_tag_ZN3OCC20TalkNotificationDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::TalkNotificationData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20TalkNotificationDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20TalkNotificationDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC20TalkNotificationDataE_t>.metaTypes,
    nullptr
} };

void OCC::TalkNotificationData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TalkNotificationData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->conversationToken; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->messageId; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->messageSent; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->userAvatar; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->conversationToken, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->messageId, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->messageSent, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->userAvatar, *reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC8ActivityE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Activity::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC8ActivityE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Activity",
        "type",
        "OCC::Activity::Type",
        "talkNotificationData",
        "OCC::TalkNotificationData",
        "subjectRichParameters",
        "QVariantMap",
        "Type",
        "DummyFetchingActivityType",
        "OpenSettingsNotificationType",
        "NotificationType",
        "SyncResultType",
        "SyncFileItemType",
        "ActivityType",
        "DummyMoreActivitiesAvailableType"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<OCC::Activity::Type>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'talkNotificationData'
        QtMocHelpers::PropertyData<OCC::TalkNotificationData>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'subjectRichParameters'
        QtMocHelpers::PropertyData<QVariantMap>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(7, 7, QMC::EnumFlags{}).add({
            {    8, Type::DummyFetchingActivityType },
            {    9, Type::OpenSettingsNotificationType },
            {   10, Type::NotificationType },
            {   11, Type::SyncResultType },
            {   12, Type::SyncFileItemType },
            {   13, Type::ActivityType },
            {   14, Type::DummyMoreActivitiesAvailableType },
        }),
    };
    return QtMocHelpers::metaObjectData<Activity, qt_meta_tag_ZN3OCC8ActivityE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC8ActivityE[] = {
    QMetaObject::SuperData::link<OCC::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::Activity::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC8ActivityE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC8ActivityE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC8ActivityE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC8ActivityE_t>.metaTypes,
    nullptr
} };

void OCC::Activity::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Activity *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< OCC::Activity::Type >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<OCC::Activity::Type*>(_v) = _t->_type; break;
        case 1: *reinterpret_cast<OCC::TalkNotificationData*>(_v) = _t->_talkNotificationData; break;
        case 2: *reinterpret_cast<QVariantMap*>(_v) = _t->_subjectRichParameters; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->_type, *reinterpret_cast<OCC::Activity::Type*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->_talkNotificationData, *reinterpret_cast<OCC::TalkNotificationData*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->_subjectRichParameters, *reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
