/****************************************************************************
** Meta object code from reading C++ file 'encryptedfoldermetadatahandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/encryptedfoldermetadatahandler.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'encryptedfoldermetadatahandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::EncryptedFolderMetadataHandler::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::EncryptedFolderMetadataHandler",
        "fetchFinished",
        "",
        "code",
        "message",
        "uploadFinished",
        "folderUnlocked",
        "folderId",
        "httpStatus",
        "slotFolderEncryptedIdReceived",
        "list",
        "slotFolderEncryptedIdError",
        "QNetworkReply*",
        "reply",
        "slotMetadataReceived",
        "QJsonDocument",
        "json",
        "statusCode",
        "slotMetadataReceivedError",
        "httpReturnCode",
        "slotFolderLockedSuccessfully",
        "token",
        "slotFolderLockedError",
        "httpErrorCode",
        "slotUploadMetadataSuccess",
        "slotUploadMetadataError",
        "slotEmitUploadSuccess",
        "slotEmitUploadError",
        "FetchMode",
        "NonEmptyMetadata",
        "AllowEmptyMetadata",
        "UploadMode",
        "DoNotKeepLock",
        "KeepLock",
        "UnlockFolderWithResult",
        "Success",
        "Failure"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fetchFinished'
        QtMocHelpers::SignalData<void(int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'fetchFinished'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'uploadFinished'
        QtMocHelpers::SignalData<void(int, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'uploadFinished'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'folderUnlocked'
        QtMocHelpers::SignalData<void(const QByteArray &, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'slotFolderEncryptedIdReceived'
        QtMocHelpers::SlotData<void(const QStringList &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 10 },
        }}),
        // Slot 'slotFolderEncryptedIdError'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotMetadataReceived'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotMetadataReceivedError'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'slotFolderLockedSuccessfully'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 }, { QMetaType::QByteArray, 21 },
        }}),
        // Slot 'slotFolderLockedError'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 }, { QMetaType::Int, 23 },
        }}),
        // Slot 'slotUploadMetadataSuccess'
        QtMocHelpers::SlotData<void(const QByteArray &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 },
        }}),
        // Slot 'slotUploadMetadataError'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'slotEmitUploadSuccess'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEmitUploadError'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FetchMode'
        QtMocHelpers::EnumData<enum FetchMode>(28, 28, QMC::EnumIsScoped).add({
            {   29, FetchMode::NonEmptyMetadata },
            {   30, FetchMode::AllowEmptyMetadata },
        }),
        // enum 'UploadMode'
        QtMocHelpers::EnumData<enum UploadMode>(31, 31, QMC::EnumIsScoped).add({
            {   32, UploadMode::DoNotKeepLock },
            {   33, UploadMode::KeepLock },
        }),
        // enum 'UnlockFolderWithResult'
        QtMocHelpers::EnumData<enum UnlockFolderWithResult>(34, 34, QMC::EnumIsScoped).add({
            {   35, UnlockFolderWithResult::Success },
            {   36, UnlockFolderWithResult::Failure },
        }),
    };
    return QtMocHelpers::metaObjectData<EncryptedFolderMetadataHandler, qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::EncryptedFolderMetadataHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t>.metaTypes,
    nullptr
} };

void OCC::EncryptedFolderMetadataHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EncryptedFolderMetadataHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fetchFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->fetchFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->uploadFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->uploadFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->folderUnlocked((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->slotFolderEncryptedIdReceived((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->slotFolderEncryptedIdError((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 7: _t->slotMetadataReceived((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->slotMetadataReceivedError((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->slotFolderLockedSuccessfully((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 10: _t->slotFolderLockedError((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->slotUploadMetadataSuccess((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 12: _t->slotUploadMetadataError((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->slotEmitUploadSuccess(); break;
        case 14: _t->slotEmitUploadError(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EncryptedFolderMetadataHandler::*)(int , const QString & )>(_a, &EncryptedFolderMetadataHandler::fetchFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptedFolderMetadataHandler::*)(int , const QString & )>(_a, &EncryptedFolderMetadataHandler::uploadFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptedFolderMetadataHandler::*)(const QByteArray & , int )>(_a, &EncryptedFolderMetadataHandler::folderUnlocked, 4))
            return;
    }
}

const QMetaObject *OCC::EncryptedFolderMetadataHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::EncryptedFolderMetadataHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC30EncryptedFolderMetadataHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::EncryptedFolderMetadataHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void OCC::EncryptedFolderMetadataHandler::fetchFinished(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::EncryptedFolderMetadataHandler::uploadFinished(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 4
void OCC::EncryptedFolderMetadataHandler::folderUnlocked(const QByteArray & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
