/****************************************************************************
** Meta object code from reading C++ file 'webflowcredentials.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/creds/webflowcredentials.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'webflowcredentials.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::WebFlowCredentials::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::WebFlowCredentials",
        "slotAuthentication",
        "",
        "QNetworkReply*",
        "reply",
        "QAuthenticator*",
        "authenticator",
        "slotFinished",
        "slotAskFromUserCredentialsProvided",
        "user",
        "pass",
        "host",
        "slotAskFromUserCancelled",
        "slotReadClientCertPEMJobDone",
        "OCC::KeychainChunk::ReadJob*",
        "readJob",
        "slotReadClientKeyPEMJobDone",
        "slotReadClientCaCertsPEMJobDone",
        "slotReadPasswordJobDone",
        "QKeychain::Job*",
        "incomingJob",
        "slotWriteClientCertPEMJobDone",
        "OCC::KeychainChunk::WriteJob*",
        "writeJob",
        "slotWriteClientKeyPEMJobDone",
        "slotWriteClientCaCertsPEMJobDone",
        "slotWriteJobDone"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotAuthentication'
        QtMocHelpers::SlotData<void(QNetworkReply *, QAuthenticator *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotAskFromUserCredentialsProvided'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slotAskFromUserCancelled'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotReadClientCertPEMJobDone'
        QtMocHelpers::SlotData<void(OCC::KeychainChunk::ReadJob *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotReadClientKeyPEMJobDone'
        QtMocHelpers::SlotData<void(OCC::KeychainChunk::ReadJob *)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotReadClientCaCertsPEMJobDone'
        QtMocHelpers::SlotData<void(OCC::KeychainChunk::ReadJob *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotReadPasswordJobDone'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotWriteClientCertPEMJobDone'
        QtMocHelpers::SlotData<void(OCC::KeychainChunk::WriteJob *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotWriteClientKeyPEMJobDone'
        QtMocHelpers::SlotData<void(OCC::KeychainChunk::WriteJob *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotWriteClientCaCertsPEMJobDone'
        QtMocHelpers::SlotData<void(OCC::KeychainChunk::WriteJob *)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotWriteJobDone'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WebFlowCredentials, qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::WebFlowCredentials::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractCredentials::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t>.metaTypes,
    nullptr
} };

void OCC::WebFlowCredentials::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WebFlowCredentials *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotAuthentication((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAuthenticator*>>(_a[2]))); break;
        case 1: _t->slotFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 2: _t->slotAskFromUserCredentialsProvided((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 3: _t->slotAskFromUserCancelled(); break;
        case 4: _t->slotReadClientCertPEMJobDone((*reinterpret_cast<std::add_pointer_t<OCC::KeychainChunk::ReadJob*>>(_a[1]))); break;
        case 5: _t->slotReadClientKeyPEMJobDone((*reinterpret_cast<std::add_pointer_t<OCC::KeychainChunk::ReadJob*>>(_a[1]))); break;
        case 6: _t->slotReadClientCaCertsPEMJobDone((*reinterpret_cast<std::add_pointer_t<OCC::KeychainChunk::ReadJob*>>(_a[1]))); break;
        case 7: _t->slotReadPasswordJobDone((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        case 8: _t->slotWriteClientCertPEMJobDone((*reinterpret_cast<std::add_pointer_t<OCC::KeychainChunk::WriteJob*>>(_a[1]))); break;
        case 9: _t->slotWriteClientKeyPEMJobDone((*reinterpret_cast<std::add_pointer_t<OCC::KeychainChunk::WriteJob*>>(_a[1]))); break;
        case 10: _t->slotWriteClientCaCertsPEMJobDone((*reinterpret_cast<std::add_pointer_t<OCC::KeychainChunk::WriteJob*>>(_a[1]))); break;
        case 11: _t->slotWriteJobDone((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::WebFlowCredentials::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::WebFlowCredentials::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18WebFlowCredentialsE_t>.strings))
        return static_cast<void*>(this);
    return AbstractCredentials::qt_metacast(_clname);
}

int OCC::WebFlowCredentials::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractCredentials::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
