/****************************************************************************
** Meta object code from reading C++ file 'bulkpropagatorjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/bulkpropagatorjob.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bulkpropagatorjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::BulkPropagatorJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::BulkPropagatorJob",
        "startUploadFile",
        "",
        "OCC::SyncFileItemPtr",
        "item",
        "OCC::BulkPropagatorJob::UploadFileInfo",
        "fileToUpload",
        "slotComputeTransmissionChecksum",
        "slotComputeMd5Checksum",
        "transmissionChecksumType",
        "transmissionChecksum",
        "slotStartUpload",
        "md5Checksum",
        "slotOnErrorStartFolderUnlock",
        "OCC::SyncFileItem::Status",
        "status",
        "errorString",
        "OCC::ErrorCategory",
        "errorCategory",
        "slotPutFinished",
        "slotUploadProgress",
        "sent",
        "total",
        "slotJobDestroyed",
        "job"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'startUploadFile'
        QtMocHelpers::SlotData<void(OCC::SyncFileItemPtr, OCC::BulkPropagatorJob::UploadFileInfo)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotComputeTransmissionChecksum'
        QtMocHelpers::SlotData<void(OCC::SyncFileItemPtr, OCC::BulkPropagatorJob::UploadFileInfo)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotComputeMd5Checksum'
        QtMocHelpers::SlotData<void(OCC::SyncFileItemPtr, OCC::BulkPropagatorJob::UploadFileInfo, const QByteArray &, const QByteArray &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::QByteArray, 9 }, { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'slotStartUpload'
        QtMocHelpers::SlotData<void(OCC::SyncFileItemPtr, OCC::BulkPropagatorJob::UploadFileInfo, const QByteArray &, const QByteArray &, const QByteArray &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::QByteArray, 9 }, { QMetaType::QByteArray, 10 },
            { QMetaType::QByteArray, 12 },
        }}),
        // Slot 'slotOnErrorStartFolderUnlock'
        QtMocHelpers::SlotData<void(OCC::SyncFileItemPtr, const OCC::SyncFileItem::Status, const QString &, const OCC::ErrorCategory)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotPutFinished'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUploadProgress'
        QtMocHelpers::SlotData<void(OCC::SyncFileItemPtr, qint64, qint64)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::LongLong, 21 }, { QMetaType::LongLong, 22 },
        }}),
        // Slot 'slotJobDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BulkPropagatorJob, qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::BulkPropagatorJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PropagatorJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t>.metaTypes,
    nullptr
} };

void OCC::BulkPropagatorJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BulkPropagatorJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->startUploadFile((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::BulkPropagatorJob::UploadFileInfo>>(_a[2]))); break;
        case 1: _t->slotComputeTransmissionChecksum((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::BulkPropagatorJob::UploadFileInfo>>(_a[2]))); break;
        case 2: _t->slotComputeMd5Checksum((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::BulkPropagatorJob::UploadFileInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4]))); break;
        case 3: _t->slotStartUpload((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::BulkPropagatorJob::UploadFileInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5]))); break;
        case 4: _t->slotOnErrorStartFolderUnlock((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[4]))); break;
        case 5: _t->slotPutFinished(); break;
        case 6: _t->slotUploadProgress((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[3]))); break;
        case 7: _t->slotJobDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OCC::BulkPropagatorJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::BulkPropagatorJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17BulkPropagatorJobE_t>.strings))
        return static_cast<void*>(this);
    return PropagatorJob::qt_metacast(_clname);
}

int OCC::BulkPropagatorJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PropagatorJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
