/****************************************************************************
** Meta object code from reading C++ file 'progressdispatcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/progressdispatcher.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'progressdispatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12ProgressInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ProgressInfo::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12ProgressInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ProgressInfo",
        "updateEstimates",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'updateEstimates'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProgressInfo, qt_meta_tag_ZN3OCC12ProgressInfoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ProgressInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ProgressInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ProgressInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12ProgressInfoE_t>.metaTypes,
    nullptr
} };

void OCC::ProgressInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProgressInfo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateEstimates(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *OCC::ProgressInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ProgressInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12ProgressInfoE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ProgressInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3OCC18ProgressDispatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ProgressDispatcher::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC18ProgressDispatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ProgressDispatcher",
        "progressInfo",
        "",
        "folder",
        "OCC::ProgressInfo",
        "progress",
        "itemCompleted",
        "OCC::SyncFileItemPtr",
        "item",
        "OCC::ErrorCategory",
        "category",
        "syncError",
        "message",
        "addErrorToGui",
        "OCC::SyncFileItem::Status",
        "status",
        "errorMessage",
        "subject",
        "folderConflicts",
        "conflictPaths"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'progressInfo'
        QtMocHelpers::SignalData<void(const QString &, const OCC::ProgressInfo &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'itemCompleted'
        QtMocHelpers::SignalData<void(const QString &, const OCC::SyncFileItemPtr &, const OCC::ErrorCategory)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'syncError'
        QtMocHelpers::SignalData<void(const QString &, const QString &, OCC::ErrorCategory)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 12 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'addErrorToGui'
        QtMocHelpers::SignalData<void(const QString &, const OCC::SyncFileItem::Status, const QString &, const QString &, const OCC::ErrorCategory)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'folderConflicts'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProgressDispatcher, qt_meta_tag_ZN3OCC18ProgressDispatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ProgressDispatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18ProgressDispatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18ProgressDispatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC18ProgressDispatcherE_t>.metaTypes,
    nullptr
} };

void OCC::ProgressDispatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProgressDispatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->progressInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::ProgressInfo>>(_a[2]))); break;
        case 1: _t->itemCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[3]))); break;
        case 2: _t->syncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[3]))); break;
        case 3: _t->addErrorToGui((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItem::Status>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<OCC::ErrorCategory>>(_a[5]))); break;
        case 4: _t->folderConflicts((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProgressDispatcher::*)(const QString & , const OCC::ProgressInfo & )>(_a, &ProgressDispatcher::progressInfo, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProgressDispatcher::*)(const QString & , const OCC::SyncFileItemPtr & , const OCC::ErrorCategory )>(_a, &ProgressDispatcher::itemCompleted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProgressDispatcher::*)(const QString & , const QString & , OCC::ErrorCategory )>(_a, &ProgressDispatcher::syncError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProgressDispatcher::*)(const QString & , const OCC::SyncFileItem::Status , const QString & , const QString & , const OCC::ErrorCategory )>(_a, &ProgressDispatcher::addErrorToGui, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProgressDispatcher::*)(const QString & , const QStringList & )>(_a, &ProgressDispatcher::folderConflicts, 4))
            return;
    }
}

const QMetaObject *OCC::ProgressDispatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ProgressDispatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC18ProgressDispatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ProgressDispatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::ProgressDispatcher::progressInfo(const QString & _t1, const OCC::ProgressInfo & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::ProgressDispatcher::itemCompleted(const QString & _t1, const OCC::SyncFileItemPtr & _t2, const OCC::ErrorCategory _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void OCC::ProgressDispatcher::syncError(const QString & _t1, const QString & _t2, OCC::ErrorCategory _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void OCC::ProgressDispatcher::addErrorToGui(const QString & _t1, const OCC::SyncFileItem::Status _t2, const QString & _t3, const QString & _t4, const OCC::ErrorCategory _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 4
void OCC::ProgressDispatcher::folderConflicts(const QString & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN3OCCE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::qt_create_metaobjectdata<qt_meta_tag_ZN3OCCE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3OCCE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN3OCCE =
    OCC::qt_create_metaobjectdata<qt_meta_tag_ZN3OCCE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3OCCE =
    qt_staticMetaObjectContent_ZN3OCCE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3OCCE =
    qt_staticMetaObjectContent_ZN3OCCE.relocatingData;

Q_CONSTINIT const QMetaObject OCC::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3OCCE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3OCCE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3OCCE.metaTypes,
    nullptr
} };

QT_WARNING_POP
