/****************************************************************************
** Meta object code from reading C++ file 'discovery.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/discovery.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'discovery.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ProcessDirectoryJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ProcessDirectoryJob",
        "finished",
        "",
        "etag",
        "time",
        "updatedRootFolderQuota",
        "int64_t",
        "bytesUsed",
        "bytesAvailable",
        "rootFileIdReceived",
        "fileId",
        "setFolderQuota",
        "OCC::FolderQuota",
        "folderQuota",
        "QueryMode",
        "NormalQuery",
        "ParentDontExist",
        "ParentNotChanged",
        "InBlackList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'etag'
        QtMocHelpers::SignalData<void(const QByteArray &, const QDateTime &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QDateTime, 4 },
        }}),
        // Signal 'updatedRootFolderQuota'
        QtMocHelpers::SignalData<void(const int64_t &, const int64_t &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 },
        }}),
        // Signal 'rootFileIdReceived'
        QtMocHelpers::SignalData<void(qint64)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 10 },
        }}),
        // Slot 'setFolderQuota'
        QtMocHelpers::SlotData<void(const OCC::FolderQuota &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'QueryMode'
        QtMocHelpers::EnumData<enum QueryMode>(14, 14, QMC::EnumFlags{}).add({
            {   15, QueryMode::NormalQuery },
            {   16, QueryMode::ParentDontExist },
            {   17, QueryMode::ParentNotChanged },
            {   18, QueryMode::InBlackList },
        }),
    };
    return QtMocHelpers::metaObjectData<ProcessDirectoryJob, qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ProcessDirectoryJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t>.metaTypes,
    nullptr
} };

void OCC::ProcessDirectoryJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProcessDirectoryJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->etag((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        case 2: _t->updatedRootFolderQuota((*reinterpret_cast<std::add_pointer_t<int64_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int64_t>>(_a[2]))); break;
        case 3: _t->rootFileIdReceived((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 4: _t->setFolderQuota((*reinterpret_cast<std::add_pointer_t<OCC::FolderQuota>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProcessDirectoryJob::*)()>(_a, &ProcessDirectoryJob::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDirectoryJob::*)(const QByteArray & , const QDateTime & )>(_a, &ProcessDirectoryJob::etag, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDirectoryJob::*)(const int64_t & , const int64_t & )>(_a, &ProcessDirectoryJob::updatedRootFolderQuota, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDirectoryJob::*)(qint64 )>(_a, &ProcessDirectoryJob::rootFileIdReceived, 3))
            return;
    }
}

const QMetaObject *OCC::ProcessDirectoryJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ProcessDirectoryJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19ProcessDirectoryJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ProcessDirectoryJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::ProcessDirectoryJob::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ProcessDirectoryJob::etag(const QByteArray & _t1, const QDateTime & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OCC::ProcessDirectoryJob::updatedRootFolderQuota(const int64_t & _t1, const int64_t & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::ProcessDirectoryJob::rootFileIdReceived(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
