/****************************************************************************
** Meta object code from reading C++ file 'folderstatusmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/folderstatusmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folderstatusmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC17FolderStatusModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderStatusModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderStatusModel",
        "dirtyChanged",
        "",
        "suggestExpand",
        "QModelIndex",
        "slotUpdateFolderState",
        "OCC::Folder*",
        "slotApplySelectiveSync",
        "resetFolders",
        "slotSyncAllPendingBigFolders",
        "slotSyncNoPendingBigFolders",
        "slotSetProgress",
        "OCC::ProgressInfo",
        "progress",
        "e2eInitializationFinished",
        "isNewMnemonicGenerated",
        "slotUpdateDirectories",
        "slotGatherPermissions",
        "name",
        "QMap<QString,QString>",
        "properties",
        "slotGatherEncryptionStatus",
        "href",
        "slotLscolFinishedWithError",
        "QNetworkReply*",
        "r",
        "slotFolderSyncStateChange",
        "f",
        "slotFolderScheduleQueueChanged",
        "slotNewBigFolder",
        "slotShowFetchProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dirtyChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'suggestExpand'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'slotUpdateFolderState'
        QtMocHelpers::SlotData<void(OCC::Folder *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotApplySelectiveSync'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetFolders'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncAllPendingBigFolders'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSyncNoPendingBigFolders'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetProgress'
        QtMocHelpers::SlotData<void(const OCC::ProgressInfo &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'e2eInitializationFinished'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotUpdateDirectories'
        QtMocHelpers::SlotData<void(const QStringList &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 2 },
        }}),
        // Slot 'slotGatherPermissions'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotGatherEncryptionStatus'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotLscolFinishedWithError'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'slotFolderSyncStateChange'
        QtMocHelpers::SlotData<void(OCC::Folder *)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 27 },
        }}),
        // Slot 'slotFolderScheduleQueueChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewBigFolder'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowFetchProgress'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FolderStatusModel, qt_meta_tag_ZN3OCC17FolderStatusModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::FolderStatusModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.metaTypes,
    nullptr
} };

void OCC::FolderStatusModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderStatusModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dirtyChanged(); break;
        case 1: _t->suggestExpand((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->slotUpdateFolderState((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1]))); break;
        case 3: _t->slotApplySelectiveSync(); break;
        case 4: _t->resetFolders(); break;
        case 5: _t->slotSyncAllPendingBigFolders(); break;
        case 6: _t->slotSyncNoPendingBigFolders(); break;
        case 7: _t->slotSetProgress((*reinterpret_cast<std::add_pointer_t<OCC::ProgressInfo>>(_a[1]))); break;
        case 8: _t->e2eInitializationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotUpdateDirectories((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 10: _t->slotGatherPermissions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 11: _t->slotGatherEncryptionStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 12: _t->slotLscolFinishedWithError((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 13: _t->slotFolderSyncStateChange((*reinterpret_cast<std::add_pointer_t<OCC::Folder*>>(_a[1]))); break;
        case 14: _t->slotFolderScheduleQueueChanged(); break;
        case 15: _t->slotNewBigFolder(); break;
        case 16: _t->slotShowFetchProgress(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FolderStatusModel::*)()>(_a, &FolderStatusModel::dirtyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FolderStatusModel::*)(const QModelIndex & )>(_a, &FolderStatusModel::suggestExpand, 1))
            return;
    }
}

const QMetaObject *OCC::FolderStatusModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderStatusModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int OCC::FolderStatusModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void OCC::FolderStatusModel::dirtyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::FolderStatusModel::suggestExpand(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
