/********************************************************************************
** Form generated from reading UI file 'proxysettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROXYSETTINGS_H
#define UI_PROXYSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ProxySettings
{
public:
    QGridLayout *gridLayout_3;
    QGroupBox *proxyGroupBox;
    QGridLayout *gridLayout_2;
    QRadioButton *manualProxyRadioButton;
    QHBoxLayout *horizontalLayout_9;
    QWidget *manualSettings;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_10;
    QComboBox *typeComboBox;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *horizontalLayout_3;
    QLabel *hostLabel;
    QLineEdit *hostLineEdit;
    QLabel *label;
    QSpinBox *portSpinBox;
    QCheckBox *authRequiredcheckBox;
    QWidget *authWidgets;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *userLineEdit;
    QLineEdit *passwordLineEdit;
    QLabel *labelLocalhost;
    QSpacerItem *horizontalSpacer_6;
    QRadioButton *systemProxyRadioButton;
    QRadioButton *noProxyRadioButton;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *ProxySettings)
    {
        if (ProxySettings->objectName().isEmpty())
            ProxySettings->setObjectName("ProxySettings");
        ProxySettings->resize(481, 296);
        gridLayout_3 = new QGridLayout(ProxySettings);
        gridLayout_3->setObjectName("gridLayout_3");
        proxyGroupBox = new QGroupBox(ProxySettings);
        proxyGroupBox->setObjectName("proxyGroupBox");
        proxyGroupBox->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(proxyGroupBox->sizePolicy().hasHeightForWidth());
        proxyGroupBox->setSizePolicy(sizePolicy);
        gridLayout_2 = new QGridLayout(proxyGroupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        manualProxyRadioButton = new QRadioButton(proxyGroupBox);
        manualProxyRadioButton->setObjectName("manualProxyRadioButton");

        gridLayout_2->addWidget(manualProxyRadioButton, 2, 0, 1, 1);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        manualSettings = new QWidget(proxyGroupBox);
        manualSettings->setObjectName("manualSettings");
        manualSettings->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(manualSettings->sizePolicy().hasHeightForWidth());
        manualSettings->setSizePolicy(sizePolicy1);
        verticalLayout_2 = new QVBoxLayout(manualSettings);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(-1, -1, 0, -1);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        typeComboBox = new QComboBox(manualSettings);
        typeComboBox->setObjectName("typeComboBox");
        typeComboBox->setEnabled(true);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(typeComboBox->sizePolicy().hasHeightForWidth());
        typeComboBox->setSizePolicy(sizePolicy2);

        horizontalLayout_10->addWidget(typeComboBox);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_5);


        verticalLayout_2->addLayout(horizontalLayout_10);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        hostLabel = new QLabel(manualSettings);
        hostLabel->setObjectName("hostLabel");

        horizontalLayout_3->addWidget(hostLabel);

        hostLineEdit = new QLineEdit(manualSettings);
        hostLineEdit->setObjectName("hostLineEdit");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(hostLineEdit->sizePolicy().hasHeightForWidth());
        hostLineEdit->setSizePolicy(sizePolicy3);

        horizontalLayout_3->addWidget(hostLineEdit);

        label = new QLabel(manualSettings);
        label->setObjectName("label");
        label->setText(QString::fromUtf8(":"));

        horizontalLayout_3->addWidget(label);

        portSpinBox = new QSpinBox(manualSettings);
        portSpinBox->setObjectName("portSpinBox");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(portSpinBox->sizePolicy().hasHeightForWidth());
        portSpinBox->setSizePolicy(sizePolicy4);
        portSpinBox->setMinimum(1);
        portSpinBox->setMaximum(65535);
        portSpinBox->setValue(8080);

        horizontalLayout_3->addWidget(portSpinBox);


        verticalLayout_2->addLayout(horizontalLayout_3);

        authRequiredcheckBox = new QCheckBox(manualSettings);
        authRequiredcheckBox->setObjectName("authRequiredcheckBox");
        authRequiredcheckBox->setEnabled(false);

        verticalLayout_2->addWidget(authRequiredcheckBox);

        authWidgets = new QWidget(manualSettings);
        authWidgets->setObjectName("authWidgets");
        authWidgets->setEnabled(true);
        horizontalLayout_4 = new QHBoxLayout(authWidgets);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        userLineEdit = new QLineEdit(authWidgets);
        userLineEdit->setObjectName("userLineEdit");
        userLineEdit->setEnabled(false);
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy5.setHorizontalStretch(1);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(userLineEdit->sizePolicy().hasHeightForWidth());
        userLineEdit->setSizePolicy(sizePolicy5);

        horizontalLayout_4->addWidget(userLineEdit);

        passwordLineEdit = new QLineEdit(authWidgets);
        passwordLineEdit->setObjectName("passwordLineEdit");
        passwordLineEdit->setEnabled(false);
        sizePolicy3.setHeightForWidth(passwordLineEdit->sizePolicy().hasHeightForWidth());
        passwordLineEdit->setSizePolicy(sizePolicy3);
        passwordLineEdit->setEchoMode(QLineEdit::EchoMode::Password);

        horizontalLayout_4->addWidget(passwordLineEdit);


        verticalLayout_2->addWidget(authWidgets);

        labelLocalhost = new QLabel(manualSettings);
        labelLocalhost->setObjectName("labelLocalhost");

        verticalLayout_2->addWidget(labelLocalhost);


        horizontalLayout_9->addWidget(manualSettings);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_6);


        gridLayout_2->addLayout(horizontalLayout_9, 3, 0, 1, 3);

        systemProxyRadioButton = new QRadioButton(proxyGroupBox);
        systemProxyRadioButton->setObjectName("systemProxyRadioButton");

        gridLayout_2->addWidget(systemProxyRadioButton, 1, 0, 1, 1);

        noProxyRadioButton = new QRadioButton(proxyGroupBox);
        noProxyRadioButton->setObjectName("noProxyRadioButton");
        noProxyRadioButton->setChecked(true);

        gridLayout_2->addWidget(noProxyRadioButton, 0, 0, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_4, 2, 2, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 4, 0, 1, 1);


        gridLayout_3->addWidget(proxyGroupBox, 0, 0, 1, 1);

        buttonBox = new QDialogButtonBox(ProxySettings);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel|QDialogButtonBox::StandardButton::Ok);

        gridLayout_3->addWidget(buttonBox, 1, 0, 1, 1);


        retranslateUi(ProxySettings);

        QMetaObject::connectSlotsByName(ProxySettings);
    } // setupUi

    void retranslateUi(QWidget *ProxySettings)
    {
        ProxySettings->setWindowTitle(QCoreApplication::translate("ProxySettings", "Form", nullptr));
        proxyGroupBox->setTitle(QCoreApplication::translate("ProxySettings", "Proxy Settings", nullptr));
        manualProxyRadioButton->setText(QCoreApplication::translate("ProxySettings", "Manually specify proxy", nullptr));
        hostLabel->setText(QCoreApplication::translate("ProxySettings", "Host", nullptr));
        hostLineEdit->setText(QString());
        authRequiredcheckBox->setText(QCoreApplication::translate("ProxySettings", "Proxy server requires authentication", nullptr));
        userLineEdit->setText(QString());
        passwordLineEdit->setText(QString());
        labelLocalhost->setText(QCoreApplication::translate("ProxySettings", "Note: proxy settings have no effects for accounts on localhost", nullptr));
        systemProxyRadioButton->setText(QCoreApplication::translate("ProxySettings", "Use system proxy", nullptr));
        noProxyRadioButton->setText(QCoreApplication::translate("ProxySettings", "No proxy", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ProxySettings: public Ui_ProxySettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROXYSETTINGS_H
