/****************************************************************************
** Meta object code from reading C++ file 'datefieldbackend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/filedetails/datefieldbackend.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'datefieldbackend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Quick::DateFieldBackend::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Quick::DateFieldBackend",
        "dateChanged",
        "",
        "dateMsecsChanged",
        "dateStringChanged",
        "minimumDateChanged",
        "minimumDateMsecsChanged",
        "maximumDateChanged",
        "maximumDateMsecsChanged",
        "validDateChanged",
        "setDate",
        "date",
        "setDateMsecs",
        "dateMsecs",
        "setDateString",
        "dateString",
        "setMinimumDate",
        "minimumDate",
        "setMinimumDateMsecs",
        "minimumDateMsecs",
        "setMaximumDate",
        "maximumDate",
        "setMaximumDateMsecs",
        "maximumDateMsecs",
        "validDate"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dateChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dateMsecsChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dateStringChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumDateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumDateMsecsChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximumDateChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximumDateMsecsChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'validDateChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setDate'
        QtMocHelpers::SlotData<void(const QDate &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 11 },
        }}),
        // Slot 'setDateMsecs'
        QtMocHelpers::SlotData<void(const qint64)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 13 },
        }}),
        // Slot 'setDateString'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'setMinimumDate'
        QtMocHelpers::SlotData<void(const QDate &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 17 },
        }}),
        // Slot 'setMinimumDateMsecs'
        QtMocHelpers::SlotData<void(const qint64)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 19 },
        }}),
        // Slot 'setMaximumDate'
        QtMocHelpers::SlotData<void(const QDate &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 21 },
        }}),
        // Slot 'setMaximumDateMsecs'
        QtMocHelpers::SlotData<void(const qint64)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'date'
        QtMocHelpers::PropertyData<QDate>(11, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'dateMsecs'
        QtMocHelpers::PropertyData<qint64>(13, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'dateString'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'minimumDate'
        QtMocHelpers::PropertyData<QDate>(17, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'minimumDateMsecs'
        QtMocHelpers::PropertyData<qint64>(19, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'maximumDate'
        QtMocHelpers::PropertyData<QDate>(21, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'maximumDateMsecs'
        QtMocHelpers::PropertyData<qint64>(23, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'validDate'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DateFieldBackend, qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Quick::DateFieldBackend::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t>.metaTypes,
    nullptr
} };

void OCC::Quick::DateFieldBackend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DateFieldBackend *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dateChanged(); break;
        case 1: _t->dateMsecsChanged(); break;
        case 2: _t->dateStringChanged(); break;
        case 3: _t->minimumDateChanged(); break;
        case 4: _t->minimumDateMsecsChanged(); break;
        case 5: _t->maximumDateChanged(); break;
        case 6: _t->maximumDateMsecsChanged(); break;
        case 7: _t->validDateChanged(); break;
        case 8: _t->setDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 9: _t->setDateMsecs((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 10: _t->setDateString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->setMinimumDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 12: _t->setMinimumDateMsecs((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 13: _t->setMaximumDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 14: _t->setMaximumDateMsecs((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::dateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::dateMsecsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::dateStringChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::minimumDateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::minimumDateMsecsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::maximumDateChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::maximumDateMsecsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DateFieldBackend::*)()>(_a, &DateFieldBackend::validDateChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDate*>(_v) = _t->date(); break;
        case 1: *reinterpret_cast<qint64*>(_v) = _t->dateMsecs(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->dateString(); break;
        case 3: *reinterpret_cast<QDate*>(_v) = _t->minimumDate(); break;
        case 4: *reinterpret_cast<qint64*>(_v) = _t->minimumDateMsecs(); break;
        case 5: *reinterpret_cast<QDate*>(_v) = _t->maximumDate(); break;
        case 6: *reinterpret_cast<qint64*>(_v) = _t->maximumDateMsecs(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->validDate(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDate(*reinterpret_cast<QDate*>(_v)); break;
        case 1: _t->setDateMsecs(*reinterpret_cast<qint64*>(_v)); break;
        case 2: _t->setDateString(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setMinimumDate(*reinterpret_cast<QDate*>(_v)); break;
        case 4: _t->setMinimumDateMsecs(*reinterpret_cast<qint64*>(_v)); break;
        case 5: _t->setMaximumDate(*reinterpret_cast<QDate*>(_v)); break;
        case 6: _t->setMaximumDateMsecs(*reinterpret_cast<qint64*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Quick::DateFieldBackend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Quick::DateFieldBackend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC5Quick16DateFieldBackendE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Quick::DateFieldBackend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void OCC::Quick::DateFieldBackend::dateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Quick::DateFieldBackend::dateMsecsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Quick::DateFieldBackend::dateStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::Quick::DateFieldBackend::minimumDateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::Quick::DateFieldBackend::minimumDateMsecsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Quick::DateFieldBackend::maximumDateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::Quick::DateFieldBackend::maximumDateMsecsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::Quick::DateFieldBackend::validDateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
