/****************************************************************************
** Meta object code from reading C++ file 'accountstate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/accountstate.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountstate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC12AccountStateE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountState::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC12AccountStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountState",
        "stateChanged",
        "",
        "OCC::AccountState::State",
        "state",
        "isConnectedChanged",
        "hasFetchedNavigationApps",
        "statusChanged",
        "desktopNotificationsAllowedChanged",
        "termsOfServiceChanged",
        "OCC::AccountPtr",
        "account",
        "checkConnectivity",
        "slotConnectionValidatorResult",
        "OCC::ConnectionValidator::Status",
        "status",
        "errors",
        "slotHandleRemoteWipeCheck",
        "slotCredentialsFetched",
        "OCC::AbstractCredentials*",
        "creds",
        "slotCredentialsAsked",
        "slotNavigationAppsFetched",
        "QJsonDocument",
        "reply",
        "statusCode",
        "slotEtagResponseHeaderReceived",
        "value",
        "slotOcsError",
        "message",
        "slotCheckConnection",
        "slotCheckServerAvailibility",
        "slotPushNotificationsReady",
        "slotServerUserStatusChanged",
        "AccountPtr",
        "State"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(OCC::AccountState::State)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'isConnectedChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasFetchedNavigationApps'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'desktopNotificationsAllowedChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'termsOfServiceChanged'
        QtMocHelpers::SignalData<void(OCC::AccountPtr, OCC::AccountState::State)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'checkConnectivity'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotConnectionValidatorResult'
        QtMocHelpers::SlotData<void(OCC::ConnectionValidator::Status, const QStringList &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QStringList, 16 },
        }}),
        // Slot 'slotHandleRemoteWipeCheck'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCredentialsFetched'
        QtMocHelpers::SlotData<void(OCC::AbstractCredentials *)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotCredentialsAsked'
        QtMocHelpers::SlotData<void(OCC::AbstractCredentials *)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotNavigationAppsFetched'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'slotEtagResponseHeaderReceived'
        QtMocHelpers::SlotData<void(const QByteArray &, int)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QByteArray, 27 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'slotOcsError'
        QtMocHelpers::SlotData<void(int, const QString &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 25 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'slotCheckConnection'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCheckServerAvailibility'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPushNotificationsReady'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotServerUserStatusChanged'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'account'
        QtMocHelpers::PropertyData<AccountPtr>(11, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(4, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountState, qt_meta_tag_ZN3OCC12AccountStateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::AccountState::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.metaTypes,
    nullptr
} };

void OCC::AccountState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountState *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<OCC::AccountState::State>>(_a[1]))); break;
        case 1: _t->isConnectedChanged(); break;
        case 2: _t->hasFetchedNavigationApps(); break;
        case 3: _t->statusChanged(); break;
        case 4: _t->desktopNotificationsAllowedChanged(); break;
        case 5: _t->termsOfServiceChanged((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::AccountState::State>>(_a[2]))); break;
        case 6: _t->checkConnectivity(); break;
        case 7: _t->slotConnectionValidatorResult((*reinterpret_cast<std::add_pointer_t<OCC::ConnectionValidator::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 8: _t->slotHandleRemoteWipeCheck(); break;
        case 9: _t->slotCredentialsFetched((*reinterpret_cast<std::add_pointer_t<OCC::AbstractCredentials*>>(_a[1]))); break;
        case 10: _t->slotCredentialsAsked((*reinterpret_cast<std::add_pointer_t<OCC::AbstractCredentials*>>(_a[1]))); break;
        case 11: _t->slotNavigationAppsFetched((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->slotEtagResponseHeaderReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->slotOcsError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->slotCheckConnection(); break;
        case 15: _t->slotCheckServerAvailibility(); break;
        case 16: _t->slotPushNotificationsReady(); break;
        case 17: _t->slotServerUserStatusChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AbstractCredentials* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AbstractCredentials* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)(OCC::AccountState::State )>(_a, &AccountState::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)()>(_a, &AccountState::isConnectedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)()>(_a, &AccountState::hasFetchedNavigationApps, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)()>(_a, &AccountState::statusChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)()>(_a, &AccountState::desktopNotificationsAllowedChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountState::*)(OCC::AccountPtr , OCC::AccountState::State )>(_a, &AccountState::termsOfServiceChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountPtr*>(_v) = _t->_account; break;
        case 1: *reinterpret_cast<enum State*>(_v) = _t->state(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->_account, *reinterpret_cast<AccountPtr*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::AccountState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC12AccountStateE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QSharedData"))
        return static_cast< QSharedData*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::AccountState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OCC::AccountState::stateChanged(OCC::AccountState::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::AccountState::isConnectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::AccountState::hasFetchedNavigationApps()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::AccountState::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::AccountState::desktopNotificationsAllowedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::AccountState::termsOfServiceChanged(OCC::AccountPtr _t1, OCC::AccountState::State _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN3OCC10AccountAppE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountApp::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10AccountAppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountApp"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountApp, qt_meta_tag_ZN3OCC10AccountAppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::AccountApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10AccountAppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10AccountAppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10AccountAppE_t>.metaTypes,
    nullptr
} };

void OCC::AccountApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountApp *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OCC::AccountApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10AccountAppE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::AccountApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
