/****************************************************************************
** Meta object code from reading C++ file 'userstatusselectormodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/userstatusselectormodel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'userstatusselectormodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UserStatusSelectorModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UserStatusSelectorModel",
        "userIndexChanged",
        "",
        "errorMessageChanged",
        "busyStatusSupportedChanged",
        "userStatusChanged",
        "clearAtDisplayStringChanged",
        "predefinedStatusesChanged",
        "finished",
        "setUserIndex",
        "userIndex",
        "setUserStatus",
        "clearUserStatus",
        "setClearAt",
        "OCC::UserStatusSelectorModel::ClearStageType",
        "clearStageType",
        "setPredefinedStatus",
        "OCC::UserStatus",
        "predefinedStatus",
        "clearAtReadable",
        "status",
        "userStatusMessage",
        "userStatusEmoji",
        "onlineStatus",
        "OCC::UserStatus::OnlineStatus",
        "predefinedStatuses",
        "QList<OCC::UserStatus>",
        "clearStageTypes",
        "QVariantList",
        "clearAtDisplayString",
        "errorMessage",
        "busyStatusSupported",
        "onlineIcon",
        "QUrl",
        "awayIcon",
        "dndIcon",
        "busyIcon",
        "invisibleIcon",
        "ClearStageType",
        "DontClear",
        "HalfHour",
        "OneHour",
        "FourHour",
        "Today",
        "Week"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'userIndexChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorMessageChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'busyStatusSupportedChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userStatusChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clearAtDisplayStringChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'predefinedStatusesChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setUserIndex'
        QtMocHelpers::SlotData<void(const int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setUserStatus'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearUserStatus'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setClearAt'
        QtMocHelpers::SlotData<void(const OCC::UserStatusSelectorModel::ClearStageType)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'setPredefinedStatus'
        QtMocHelpers::SlotData<void(const OCC::UserStatus &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'clearAtReadable'
        QtMocHelpers::MethodData<QString(const OCC::UserStatus &) const>(19, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 17, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'userIndex'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'userStatusMessage'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'userStatusEmoji'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'onlineStatus'
        QtMocHelpers::PropertyData<OCC::UserStatus::OnlineStatus>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'predefinedStatuses'
        QtMocHelpers::PropertyData<QList<OCC::UserStatus>>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'clearStageTypes'
        QtMocHelpers::PropertyData<QVariantList>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'clearAtDisplayString'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'errorMessage'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'busyStatusSupported'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'onlineIcon'
        QtMocHelpers::PropertyData<QUrl>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'awayIcon'
        QtMocHelpers::PropertyData<QUrl>(34, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'dndIcon'
        QtMocHelpers::PropertyData<QUrl>(35, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'busyIcon'
        QtMocHelpers::PropertyData<QUrl>(36, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'invisibleIcon'
        QtMocHelpers::PropertyData<QUrl>(37, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ClearStageType'
        QtMocHelpers::EnumData<enum ClearStageType>(38, 38, QMC::EnumIsScoped).add({
            {   39, ClearStageType::DontClear },
            {   40, ClearStageType::HalfHour },
            {   41, ClearStageType::OneHour },
            {   42, ClearStageType::FourHour },
            {   43, ClearStageType::Today },
            {   44, ClearStageType::Week },
        }),
    };
    return QtMocHelpers::metaObjectData<UserStatusSelectorModel, qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC23UserStatusSelectorModelE[] = {
    QMetaObject::SuperData::link<OCC::UserStatus::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::UserStatusSelectorModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC23UserStatusSelectorModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t>.metaTypes,
    nullptr
} };

void OCC::UserStatusSelectorModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UserStatusSelectorModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->userIndexChanged(); break;
        case 1: _t->errorMessageChanged(); break;
        case 2: _t->busyStatusSupportedChanged(); break;
        case 3: _t->userStatusChanged(); break;
        case 4: _t->clearAtDisplayStringChanged(); break;
        case 5: _t->predefinedStatusesChanged(); break;
        case 6: _t->finished(); break;
        case 7: _t->setUserIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->setUserStatus(); break;
        case 9: _t->clearUserStatus(); break;
        case 10: _t->setClearAt((*reinterpret_cast<std::add_pointer_t<OCC::UserStatusSelectorModel::ClearStageType>>(_a[1]))); break;
        case 11: _t->setPredefinedStatus((*reinterpret_cast<std::add_pointer_t<OCC::UserStatus>>(_a[1]))); break;
        case 12: { QString _r = _t->clearAtReadable((*reinterpret_cast<std::add_pointer_t<OCC::UserStatus>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::UserStatus >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::UserStatus >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::userIndexChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::errorMessageChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::busyStatusSupportedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::userStatusChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::clearAtDisplayStringChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::predefinedStatusesChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusSelectorModel::*)()>(_a, &UserStatusSelectorModel::finished, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<OCC::UserStatus> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->userIndex(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->userStatusMessage(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->userStatusEmoji(); break;
        case 3: *reinterpret_cast<OCC::UserStatus::OnlineStatus*>(_v) = _t->onlineStatus(); break;
        case 4: *reinterpret_cast<QList<OCC::UserStatus>*>(_v) = _t->predefinedStatuses(); break;
        case 5: *reinterpret_cast<QVariantList*>(_v) = _t->clearStageTypes(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->clearAtDisplayString(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->errorMessage(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->busyStatusSupported(); break;
        case 9: *reinterpret_cast<QUrl*>(_v) = _t->onlineIcon(); break;
        case 10: *reinterpret_cast<QUrl*>(_v) = _t->awayIcon(); break;
        case 11: *reinterpret_cast<QUrl*>(_v) = _t->dndIcon(); break;
        case 12: *reinterpret_cast<QUrl*>(_v) = _t->busyIcon(); break;
        case 13: *reinterpret_cast<QUrl*>(_v) = _t->invisibleIcon(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUserIndex(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setUserStatusMessage(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setUserStatusEmoji(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setOnlineStatus(*reinterpret_cast<OCC::UserStatus::OnlineStatus*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::UserStatusSelectorModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::UserStatusSelectorModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC23UserStatusSelectorModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::UserStatusSelectorModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void OCC::UserStatusSelectorModel::userIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::UserStatusSelectorModel::errorMessageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::UserStatusSelectorModel::busyStatusSupportedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::UserStatusSelectorModel::userStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::UserStatusSelectorModel::clearAtDisplayStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::UserStatusSelectorModel::predefinedStatusesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::UserStatusSelectorModel::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
