/********************************************************************************
** Form generated from reading UI file 'accountsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTSETTINGS_H
#define UI_ACCOUNTSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "folderstatusview.h"
#include "sslbutton.h"

namespace OCC {

class Ui_AccountSettings
{
public:
    QVBoxLayout *verticalLayout_2;
    QFrame *accountStatusPanel;
    QVBoxLayout *accountStatusLayout;
    QWidget *accountStatus;
    QGridLayout *gridLayout_2;
    SslButton *sslButton;
    QLabel *connectLabel;
    QWidget *encryptionMessage;
    QVBoxLayout *encryptionMessageLayout;
    QHBoxLayout *encryptionMessageHeaderLayout;
    QLabel *encryptionMessageIcon;
    QLabel *encryptionMessageLabel;
    QHBoxLayout *encryptionMessageButtonsLayout;
    QFrame *fileProviderPanel;
    QVBoxLayout *fileProviderPanelLayout;
    QLabel *fileProviderPanelTitle;
    QWidget *fileProviderPanelContents;
    QFrame *syncFoldersPanel;
    QVBoxLayout *syncFoldersPanelLayout;
    QLabel *syncFoldersPanelTitle;
    QWidget *syncFoldersPanelContents;
    QVBoxLayout *syncFoldersLayout;
    OCC::FolderStatusView *_folderList;
    QWidget *selectiveSyncStatus;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *verticalLayout_3;
    QLabel *selectiveSyncLabel;
    QWidget *selectiveSyncButtons;
    QHBoxLayout *horizontalLayout;
    QPushButton *selectiveSyncCancel;
    QPushButton *selectiveSyncApply;
    QWidget *bigFolderUi;
    QVBoxLayout *verticalLayout;
    QLabel *selectiveSyncNotification;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *bigFolderSyncAll;
    QPushButton *bigFolderSyncNone;
    QPushButton *bigFolderApply;
    QFrame *connectionSettingsPanel;
    QVBoxLayout *connectionSettingsPanelLayout;
    QLabel *connectionSettingsPanelTitle;
    QWidget *connectionSettingsPanelContents;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *OCC__AccountSettings)
    {
        if (OCC__AccountSettings->objectName().isEmpty())
            OCC__AccountSettings->setObjectName("OCC__AccountSettings");
        OCC__AccountSettings->resize(1028, 871);
        OCC__AccountSettings->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout_2 = new QVBoxLayout(OCC__AccountSettings);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        accountStatusPanel = new QFrame(OCC__AccountSettings);
        accountStatusPanel->setObjectName("accountStatusPanel");
        accountStatusPanel->setFrameShape(QFrame::Shape::NoFrame);
        accountStatusLayout = new QVBoxLayout(accountStatusPanel);
        accountStatusLayout->setObjectName("accountStatusLayout");
        accountStatusLayout->setContentsMargins(0, 0, 0, 0);
        accountStatus = new QWidget(accountStatusPanel);
        accountStatus->setObjectName("accountStatus");
        gridLayout_2 = new QGridLayout(accountStatus);
        gridLayout_2->setObjectName("gridLayout_2");
        sslButton = new SslButton(accountStatus);
        sslButton->setObjectName("sslButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(sslButton->sizePolicy().hasHeightForWidth());
        sslButton->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(sslButton, 0, 0, 1, 1);

        connectLabel = new QLabel(accountStatus);
        connectLabel->setObjectName("connectLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(connectLabel->sizePolicy().hasHeightForWidth());
        connectLabel->setSizePolicy(sizePolicy1);
        connectLabel->setTextFormat(Qt::TextFormat::RichText);
        connectLabel->setWordWrap(true);
        connectLabel->setOpenExternalLinks(true);

        gridLayout_2->addWidget(connectLabel, 0, 1, 1, 1);


        accountStatusLayout->addWidget(accountStatus);

        encryptionMessage = new QWidget(accountStatusPanel);
        encryptionMessage->setObjectName("encryptionMessage");
        encryptionMessageLayout = new QVBoxLayout(encryptionMessage);
        encryptionMessageLayout->setObjectName("encryptionMessageLayout");
        encryptionMessageHeaderLayout = new QHBoxLayout();
        encryptionMessageHeaderLayout->setObjectName("encryptionMessageHeaderLayout");
        encryptionMessageIcon = new QLabel(encryptionMessage);
        encryptionMessageIcon->setObjectName("encryptionMessageIcon");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(encryptionMessageIcon->sizePolicy().hasHeightForWidth());
        encryptionMessageIcon->setSizePolicy(sizePolicy2);
        encryptionMessageIcon->setAlignment(Qt::AlignmentFlag::AlignHCenter|Qt::AlignmentFlag::AlignTop);

        encryptionMessageHeaderLayout->addWidget(encryptionMessageIcon);

        encryptionMessageLabel = new QLabel(encryptionMessage);
        encryptionMessageLabel->setObjectName("encryptionMessageLabel");
        sizePolicy1.setHeightForWidth(encryptionMessageLabel->sizePolicy().hasHeightForWidth());
        encryptionMessageLabel->setSizePolicy(sizePolicy1);
        encryptionMessageLabel->setWordWrap(true);

        encryptionMessageHeaderLayout->addWidget(encryptionMessageLabel);


        encryptionMessageLayout->addLayout(encryptionMessageHeaderLayout);

        encryptionMessageButtonsLayout = new QHBoxLayout();
        encryptionMessageButtonsLayout->setObjectName("encryptionMessageButtonsLayout");

        encryptionMessageLayout->addLayout(encryptionMessageButtonsLayout);


        accountStatusLayout->addWidget(encryptionMessage);


        verticalLayout_2->addWidget(accountStatusPanel);

        fileProviderPanel = new QFrame(OCC__AccountSettings);
        fileProviderPanel->setObjectName("fileProviderPanel");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(fileProviderPanel->sizePolicy().hasHeightForWidth());
        fileProviderPanel->setSizePolicy(sizePolicy3);
        fileProviderPanel->setFrameShape(QFrame::Shape::NoFrame);
        fileProviderPanelLayout = new QVBoxLayout(fileProviderPanel);
        fileProviderPanelLayout->setObjectName("fileProviderPanelLayout");
        fileProviderPanelLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        fileProviderPanelTitle = new QLabel(fileProviderPanel);
        fileProviderPanelTitle->setObjectName("fileProviderPanelTitle");
        QFont font;
        font.setBold(true);
        fileProviderPanelTitle->setFont(font);

        fileProviderPanelLayout->addWidget(fileProviderPanelTitle);

        fileProviderPanelContents = new QWidget(fileProviderPanel);
        fileProviderPanelContents->setObjectName("fileProviderPanelContents");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(fileProviderPanelContents->sizePolicy().hasHeightForWidth());
        fileProviderPanelContents->setSizePolicy(sizePolicy4);

        fileProviderPanelLayout->addWidget(fileProviderPanelContents);


        verticalLayout_2->addWidget(fileProviderPanel);

        syncFoldersPanel = new QFrame(OCC__AccountSettings);
        syncFoldersPanel->setObjectName("syncFoldersPanel");
        sizePolicy3.setHeightForWidth(syncFoldersPanel->sizePolicy().hasHeightForWidth());
        syncFoldersPanel->setSizePolicy(sizePolicy3);
        syncFoldersPanel->setFrameShape(QFrame::Shape::NoFrame);
        syncFoldersPanelLayout = new QVBoxLayout(syncFoldersPanel);
        syncFoldersPanelLayout->setObjectName("syncFoldersPanelLayout");
        syncFoldersPanelLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        syncFoldersPanelTitle = new QLabel(syncFoldersPanel);
        syncFoldersPanelTitle->setObjectName("syncFoldersPanelTitle");
        sizePolicy4.setHeightForWidth(syncFoldersPanelTitle->sizePolicy().hasHeightForWidth());
        syncFoldersPanelTitle->setSizePolicy(sizePolicy4);
        syncFoldersPanelTitle->setFont(font);

        syncFoldersPanelLayout->addWidget(syncFoldersPanelTitle);

        syncFoldersPanelContents = new QWidget(syncFoldersPanel);
        syncFoldersPanelContents->setObjectName("syncFoldersPanelContents");
        sizePolicy4.setHeightForWidth(syncFoldersPanelContents->sizePolicy().hasHeightForWidth());
        syncFoldersPanelContents->setSizePolicy(sizePolicy4);
        syncFoldersLayout = new QVBoxLayout(syncFoldersPanelContents);
        syncFoldersLayout->setObjectName("syncFoldersLayout");
        syncFoldersLayout->setContentsMargins(0, 0, 0, 0);
        _folderList = new OCC::FolderStatusView(syncFoldersPanelContents);
        _folderList->setObjectName("_folderList");
        sizePolicy4.setHeightForWidth(_folderList->sizePolicy().hasHeightForWidth());
        _folderList->setSizePolicy(sizePolicy4);
        _folderList->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        _folderList->setAutoFillBackground(false);
        _folderList->setFrameShape(QFrame::Shape::NoFrame);
        _folderList->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContents);
        _folderList->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        _folderList->setAnimated(true);

        syncFoldersLayout->addWidget(_folderList);

        selectiveSyncStatus = new QWidget(syncFoldersPanelContents);
        selectiveSyncStatus->setObjectName("selectiveSyncStatus");
        horizontalLayout_3 = new QHBoxLayout(selectiveSyncStatus);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        selectiveSyncLabel = new QLabel(selectiveSyncStatus);
        selectiveSyncLabel->setObjectName("selectiveSyncLabel");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Maximum);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(selectiveSyncLabel->sizePolicy().hasHeightForWidth());
        selectiveSyncLabel->setSizePolicy(sizePolicy5);
        selectiveSyncLabel->setTextFormat(Qt::TextFormat::RichText);
        selectiveSyncLabel->setWordWrap(true);
        selectiveSyncLabel->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(selectiveSyncLabel);


        horizontalLayout_3->addLayout(verticalLayout_3);

        selectiveSyncButtons = new QWidget(selectiveSyncStatus);
        selectiveSyncButtons->setObjectName("selectiveSyncButtons");
        horizontalLayout = new QHBoxLayout(selectiveSyncButtons);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        selectiveSyncCancel = new QPushButton(selectiveSyncButtons);
        selectiveSyncCancel->setObjectName("selectiveSyncCancel");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(selectiveSyncCancel->sizePolicy().hasHeightForWidth());
        selectiveSyncCancel->setSizePolicy(sizePolicy6);

        horizontalLayout->addWidget(selectiveSyncCancel);

        selectiveSyncApply = new QPushButton(selectiveSyncButtons);
        selectiveSyncApply->setObjectName("selectiveSyncApply");
        sizePolicy6.setHeightForWidth(selectiveSyncApply->sizePolicy().hasHeightForWidth());
        selectiveSyncApply->setSizePolicy(sizePolicy6);

        horizontalLayout->addWidget(selectiveSyncApply);


        horizontalLayout_3->addWidget(selectiveSyncButtons);


        syncFoldersLayout->addWidget(selectiveSyncStatus);

        bigFolderUi = new QWidget(syncFoldersPanelContents);
        bigFolderUi->setObjectName("bigFolderUi");
        verticalLayout = new QVBoxLayout(bigFolderUi);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        selectiveSyncNotification = new QLabel(bigFolderUi);
        selectiveSyncNotification->setObjectName("selectiveSyncNotification");
        selectiveSyncNotification->setStyleSheet(QString::fromUtf8("color: red"));
        selectiveSyncNotification->setWordWrap(true);

        verticalLayout->addWidget(selectiveSyncNotification);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        bigFolderSyncAll = new QPushButton(bigFolderUi);
        bigFolderSyncAll->setObjectName("bigFolderSyncAll");

        horizontalLayout_2->addWidget(bigFolderSyncAll);

        bigFolderSyncNone = new QPushButton(bigFolderUi);
        bigFolderSyncNone->setObjectName("bigFolderSyncNone");

        horizontalLayout_2->addWidget(bigFolderSyncNone);

        bigFolderApply = new QPushButton(bigFolderUi);
        bigFolderApply->setObjectName("bigFolderApply");

        horizontalLayout_2->addWidget(bigFolderApply);


        verticalLayout->addLayout(horizontalLayout_2);


        syncFoldersLayout->addWidget(bigFolderUi);


        syncFoldersPanelLayout->addWidget(syncFoldersPanelContents);


        verticalLayout_2->addWidget(syncFoldersPanel);

        connectionSettingsPanel = new QFrame(OCC__AccountSettings);
        connectionSettingsPanel->setObjectName("connectionSettingsPanel");
        sizePolicy3.setHeightForWidth(connectionSettingsPanel->sizePolicy().hasHeightForWidth());
        connectionSettingsPanel->setSizePolicy(sizePolicy3);
        connectionSettingsPanel->setFrameShape(QFrame::Shape::NoFrame);
        connectionSettingsPanelLayout = new QVBoxLayout(connectionSettingsPanel);
        connectionSettingsPanelLayout->setObjectName("connectionSettingsPanelLayout");
        connectionSettingsPanelTitle = new QLabel(connectionSettingsPanel);
        connectionSettingsPanelTitle->setObjectName("connectionSettingsPanelTitle");
        connectionSettingsPanelTitle->setFont(font);

        connectionSettingsPanelLayout->addWidget(connectionSettingsPanelTitle);

        connectionSettingsPanelContents = new QWidget(connectionSettingsPanel);
        connectionSettingsPanelContents->setObjectName("connectionSettingsPanelContents");
        sizePolicy4.setHeightForWidth(connectionSettingsPanelContents->sizePolicy().hasHeightForWidth());
        connectionSettingsPanelContents->setSizePolicy(sizePolicy4);

        connectionSettingsPanelLayout->addWidget(connectionSettingsPanelContents);


        verticalLayout_2->addWidget(connectionSettingsPanel);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(OCC__AccountSettings);

        QMetaObject::connectSlotsByName(OCC__AccountSettings);
    } // setupUi

    void retranslateUi(QWidget *OCC__AccountSettings)
    {
        sslButton->setText(QString());
        connectLabel->setText(QCoreApplication::translate("OCC::AccountSettings", "Connected with <server> as <user>", nullptr));
        encryptionMessageIcon->setText(QString());
        encryptionMessageLabel->setText(QString());
        fileProviderPanelTitle->setText(QCoreApplication::translate("OCC::AccountSettings", "Virtual files", nullptr));
        syncFoldersPanelTitle->setText(QCoreApplication::translate("OCC::AccountSettings", "Classic sync", nullptr));
        selectiveSyncLabel->setText(QCoreApplication::translate("OCC::AccountSettings", "Unchecked folders will be <b>removed</b> from your local file system and will not be synchronized to this computer anymore", nullptr));
        selectiveSyncCancel->setText(QCoreApplication::translate("OCC::AccountSettings", "Cancel", nullptr));
        selectiveSyncApply->setText(QCoreApplication::translate("OCC::AccountSettings", "Apply", nullptr));
        selectiveSyncNotification->setText(QString());
        bigFolderSyncAll->setText(QCoreApplication::translate("OCC::AccountSettings", "Synchronize all", nullptr));
        bigFolderSyncNone->setText(QCoreApplication::translate("OCC::AccountSettings", "Synchronize none", nullptr));
        bigFolderApply->setText(QCoreApplication::translate("OCC::AccountSettings", "Apply manual changes", nullptr));
        connectionSettingsPanelTitle->setText(QCoreApplication::translate("OCC::AccountSettings", "Connection settings", nullptr));
        (void)OCC__AccountSettings;
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class AccountSettings: public Ui_AccountSettings {};
} // namespace Ui
} // namespace OCC

#endif // UI_ACCOUNTSETTINGS_H
