/****************************************************************************
** Meta object code from reading C++ file 'editlocallyjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/editlocallyjob.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editlocallyjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14EditLocallyJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::EditLocallyJob::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14EditLocallyJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::EditLocallyJob",
        "error",
        "",
        "message",
        "informativeText",
        "finished",
        "callShowError",
        "startSetup",
        "startEditLocally",
        "fetchRemoteFileParentInfo",
        "startSyncBeforeOpening",
        "proceedWithSetup",
        "findAfolderAndConstructPaths",
        "showError",
        "slotItemDiscovered",
        "OCC::SyncFileItemPtr",
        "item",
        "slotItemCompleted",
        "slotLsColJobFinishedWithError",
        "QNetworkReply*",
        "reply",
        "slotDirectoryListingIterated",
        "name",
        "QMap<QString,QString>",
        "properties",
        "processLocalItem",
        "openFile",
        "lockFile",
        "etag",
        "fileAlreadyLocked",
        "fileLockSuccess",
        "fileLockError",
        "errorMessage",
        "fileLockProcedureComplete",
        "notificationTitle",
        "notificationMessage",
        "success",
        "disconnectFolderSignals"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'callShowError'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'startSetup'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startEditLocally'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchRemoteFileParentInfo'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'startSyncBeforeOpening'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'proceedWithSetup'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'findAfolderAndConstructPaths'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showError'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'slotItemDiscovered'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotLsColJobFinishedWithError'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotDirectoryListingIterated'
        QtMocHelpers::SlotData<void(const QString &, const QMap<QString,QString> &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'processLocalItem'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openFile'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'lockFile'
        QtMocHelpers::SlotData<void(const QString &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'fileAlreadyLocked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'fileLockSuccess'
        QtMocHelpers::SlotData<void(const OCC::SyncFileItemPtr &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'fileLockError'
        QtMocHelpers::SlotData<void(const QString &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Slot 'fileLockProcedureComplete'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const bool)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 34 }, { QMetaType::QString, 35 }, { QMetaType::Bool, 36 },
        }}),
        // Slot 'disconnectFolderSignals'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EditLocallyJob, qt_meta_tag_ZN3OCC14EditLocallyJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::EditLocallyJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14EditLocallyJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14EditLocallyJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14EditLocallyJobE_t>.metaTypes,
    nullptr
} };

void OCC::EditLocallyJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EditLocallyJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->finished(); break;
        case 2: _t->callShowError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->startSetup(); break;
        case 4: _t->startEditLocally(); break;
        case 5: _t->fetchRemoteFileParentInfo(); break;
        case 6: _t->startSyncBeforeOpening(); break;
        case 7: _t->proceedWithSetup(); break;
        case 8: _t->findAfolderAndConstructPaths(); break;
        case 9: _t->showError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->slotItemDiscovered((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 11: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 12: _t->slotLsColJobFinishedWithError((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 13: _t->slotDirectoryListingIterated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[2]))); break;
        case 14: _t->processLocalItem(); break;
        case 15: _t->openFile(); break;
        case 16: _t->lockFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->fileAlreadyLocked(); break;
        case 18: _t->fileLockSuccess((*reinterpret_cast<std::add_pointer_t<OCC::SyncFileItemPtr>>(_a[1]))); break;
        case 19: _t->fileLockError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->fileLockProcedureComplete((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 21: _t->disconnectFolderSignals(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EditLocallyJob::*)(const QString & , const QString & )>(_a, &EditLocallyJob::error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditLocallyJob::*)()>(_a, &EditLocallyJob::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditLocallyJob::*)(const QString & , const QString & )>(_a, &EditLocallyJob::callShowError, 2))
            return;
    }
}

const QMetaObject *OCC::EditLocallyJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::EditLocallyJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14EditLocallyJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::EditLocallyJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void OCC::EditLocallyJob::error(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OCC::EditLocallyJob::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::EditLocallyJob::callShowError(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
