/****************************************************************************
** Meta object code from reading C++ file 'unifiedsearchresultslistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/tray/unifiedsearchresultslistmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'unifiedsearchresultslistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UnifiedSearchResultsListModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UnifiedSearchResultsListModel",
        "currentFetchMoreInProgressProviderIdChanged",
        "",
        "isSearchInProgressChanged",
        "errorStringChanged",
        "searchTermChanged",
        "waitingForSearchTermEditEndChanged",
        "setSearchTerm",
        "term",
        "slotSearchTermEditingFinished",
        "slotFetchProvidersFinished",
        "QJsonDocument",
        "json",
        "statusCode",
        "slotSearchForProviderFinished",
        "resultClicked",
        "providerId",
        "QUrl",
        "resourceUrl",
        "fetchMoreTriggerClicked",
        "isSearchInProgress",
        "currentFetchMoreInProgressProviderId",
        "errorString",
        "searchTerm",
        "waitingForSearchTermEditEnd"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentFetchMoreInProgressProviderIdChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isSearchInProgressChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorStringChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'searchTermChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waitingForSearchTermEditEndChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSearchTerm'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'slotSearchTermEditingFinished'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFetchProvidersFinished'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot 'slotSearchForProviderFinished'
        QtMocHelpers::SlotData<void(const QJsonDocument &, int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 },
        }}),
        // Method 'resultClicked'
        QtMocHelpers::MethodData<void(const QString &, const QUrl &) const>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Method 'fetchMoreTriggerClicked'
        QtMocHelpers::MethodData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isSearchInProgress'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'currentFetchMoreInProgressProviderId'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'searchTerm'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'waitingForSearchTermEditEnd'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UnifiedSearchResultsListModel, qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::UnifiedSearchResultsListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t>.metaTypes,
    nullptr
} };

void OCC::UnifiedSearchResultsListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UnifiedSearchResultsListModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentFetchMoreInProgressProviderIdChanged(); break;
        case 1: _t->isSearchInProgressChanged(); break;
        case 2: _t->errorStringChanged(); break;
        case 3: _t->searchTermChanged(); break;
        case 4: _t->waitingForSearchTermEditEndChanged(); break;
        case 5: _t->setSearchTerm((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotSearchTermEditingFinished(); break;
        case 7: _t->slotFetchProvidersFinished((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->slotSearchForProviderFinished((*reinterpret_cast<std::add_pointer_t<QJsonDocument>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->resultClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 10: _t->fetchMoreTriggerClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UnifiedSearchResultsListModel::*)()>(_a, &UnifiedSearchResultsListModel::currentFetchMoreInProgressProviderIdChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnifiedSearchResultsListModel::*)()>(_a, &UnifiedSearchResultsListModel::isSearchInProgressChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnifiedSearchResultsListModel::*)()>(_a, &UnifiedSearchResultsListModel::errorStringChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnifiedSearchResultsListModel::*)()>(_a, &UnifiedSearchResultsListModel::searchTermChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnifiedSearchResultsListModel::*)()>(_a, &UnifiedSearchResultsListModel::waitingForSearchTermEditEndChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isSearchInProgress(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->currentFetchMoreInProgressProviderId(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->searchTerm(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->waitingForSearchTermEditEnd(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setSearchTerm(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::UnifiedSearchResultsListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::UnifiedSearchResultsListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC29UnifiedSearchResultsListModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::UnifiedSearchResultsListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OCC::UnifiedSearchResultsListModel::currentFetchMoreInProgressProviderIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::UnifiedSearchResultsListModel::isSearchInProgressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::UnifiedSearchResultsListModel::errorStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::UnifiedSearchResultsListModel::searchTermChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::UnifiedSearchResultsListModel::waitingForSearchTermEditEndChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
