/****************************************************************************
** Meta object code from reading C++ file 'conflictsolver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/conflictsolver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'conflictsolver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14ConflictSolverE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ConflictSolver::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14ConflictSolverE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ConflictSolver",
        "localVersionFilenameChanged",
        "",
        "remoteVersionFilenameChanged",
        "isBulkSolutionChanged",
        "yesToAllRequestedChanged",
        "setLocalVersionFilename",
        "localVersionFilename",
        "setRemoteVersionFilename",
        "remoteVersionFilename",
        "setIsBulkSolution",
        "isBulkSolution",
        "setYesToAllRequested",
        "yesToAllRequested",
        "Solution",
        "KeepLocalVersion",
        "KeepRemoteVersion",
        "KeepBothVersions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'localVersionFilenameChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'remoteVersionFilenameChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isBulkSolutionChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'yesToAllRequestedChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLocalVersionFilename'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setRemoteVersionFilename'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'setIsBulkSolution'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'setYesToAllRequested'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'localVersionFilename'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'remoteVersionFilename'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isBulkSolution'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'yesToAllRequested'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Solution'
        QtMocHelpers::EnumData<enum Solution>(14, 14, QMC::EnumFlags{}).add({
            {   15, Solution::KeepLocalVersion },
            {   16, Solution::KeepRemoteVersion },
            {   17, Solution::KeepBothVersions },
        }),
    };
    return QtMocHelpers::metaObjectData<ConflictSolver, qt_meta_tag_ZN3OCC14ConflictSolverE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ConflictSolver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14ConflictSolverE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14ConflictSolverE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14ConflictSolverE_t>.metaTypes,
    nullptr
} };

void OCC::ConflictSolver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConflictSolver *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->localVersionFilenameChanged(); break;
        case 1: _t->remoteVersionFilenameChanged(); break;
        case 2: _t->isBulkSolutionChanged(); break;
        case 3: _t->yesToAllRequestedChanged(); break;
        case 4: _t->setLocalVersionFilename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->setRemoteVersionFilename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->setIsBulkSolution((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setYesToAllRequested((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConflictSolver::*)()>(_a, &ConflictSolver::localVersionFilenameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConflictSolver::*)()>(_a, &ConflictSolver::remoteVersionFilenameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConflictSolver::*)()>(_a, &ConflictSolver::isBulkSolutionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConflictSolver::*)()>(_a, &ConflictSolver::yesToAllRequestedChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->localVersionFilename(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->remoteVersionFilename(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isBulkSolution(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->yesToAllRequested(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLocalVersionFilename(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setRemoteVersionFilename(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setIsBulkSolution(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setYesToAllRequested(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::ConflictSolver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ConflictSolver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14ConflictSolverE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ConflictSolver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OCC::ConflictSolver::localVersionFilenameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::ConflictSolver::remoteVersionFilenameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::ConflictSolver::isBulkSolutionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::ConflictSolver::yesToAllRequestedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
