/****************************************************************************
** Meta object code from reading C++ file 'syncstatussummary.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/tray/syncstatussummary.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncstatussummary.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::SyncStatusSummary::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::SyncStatusSummary",
        "syncProgressChanged",
        "",
        "syncIconChanged",
        "syncingChanged",
        "syncStatusStringChanged",
        "syncStatusDetailStringChanged",
        "totalFilesChanged",
        "needsSandboxReapprovalChanged",
        "load",
        "syncProgress",
        "syncIcon",
        "QUrl",
        "syncing",
        "syncStatusString",
        "syncStatusDetailString",
        "totalFiles",
        "needsSandboxReapproval"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'syncProgressChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncIconChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncingChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncStatusStringChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncStatusDetailStringChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'totalFilesChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'needsSandboxReapprovalChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'load'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'syncProgress'
        QtMocHelpers::PropertyData<double>(10, QMetaType::Double, QMC::DefaultPropertyFlags, 0),
        // property 'syncIcon'
        QtMocHelpers::PropertyData<QUrl>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'syncing'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'syncStatusString'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'syncStatusDetailString'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'totalFiles'
        QtMocHelpers::PropertyData<qint64>(16, QMetaType::LongLong, QMC::DefaultPropertyFlags, 5),
        // property 'needsSandboxReapproval'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SyncStatusSummary, qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::SyncStatusSummary::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t>.metaTypes,
    nullptr
} };

void OCC::SyncStatusSummary::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncStatusSummary *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->syncProgressChanged(); break;
        case 1: _t->syncIconChanged(); break;
        case 2: _t->syncingChanged(); break;
        case 3: _t->syncStatusStringChanged(); break;
        case 4: _t->syncStatusDetailStringChanged(); break;
        case 5: _t->totalFilesChanged(); break;
        case 6: _t->needsSandboxReapprovalChanged(); break;
        case 7: _t->load(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::syncProgressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::syncIconChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::syncingChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::syncStatusStringChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::syncStatusDetailStringChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::totalFilesChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncStatusSummary::*)()>(_a, &SyncStatusSummary::needsSandboxReapprovalChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->syncProgress(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->syncIcon(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->syncing(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->syncStatusString(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->syncStatusDetailString(); break;
        case 5: *reinterpret_cast<qint64*>(_v) = _t->totalFiles(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->needsSandboxReapproval(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::SyncStatusSummary::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::SyncStatusSummary::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17SyncStatusSummaryE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::SyncStatusSummary::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OCC::SyncStatusSummary::syncProgressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::SyncStatusSummary::syncIconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::SyncStatusSummary::syncingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::SyncStatusSummary::syncStatusStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::SyncStatusSummary::syncStatusDetailStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::SyncStatusSummary::totalFilesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::SyncStatusSummary::needsSandboxReapprovalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
