/****************************************************************************
** Meta object code from reading C++ file 'userstatusconnector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/userstatusconnector.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'userstatusconnector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC10UserStatusE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UserStatus::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC10UserStatusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UserStatus",
        "id",
        "message",
        "icon",
        "state",
        "OnlineStatus",
        "Online",
        "DoNotDisturb",
        "Away",
        "Busy",
        "Offline",
        "Invisible"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'id'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'message'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'icon'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'state'
        QtMocHelpers::PropertyData<enum OnlineStatus>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'OnlineStatus'
        QtMocHelpers::EnumData<enum OnlineStatus>(5, 5, QMC::EnumIsScoped).add({
            {    6, OnlineStatus::Online },
            {    7, OnlineStatus::DoNotDisturb },
            {    8, OnlineStatus::Away },
            {    9, OnlineStatus::Busy },
            {   10, OnlineStatus::Offline },
            {   11, OnlineStatus::Invisible },
        }),
    };
    return QtMocHelpers::metaObjectData<UserStatus, qt_meta_tag_ZN3OCC10UserStatusE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::UserStatus::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10UserStatusE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC10UserStatusE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC10UserStatusE_t>.metaTypes,
    nullptr
} };

void OCC::UserStatus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<UserStatus *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->_id; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->_message; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->_icon; break;
        case 3: *reinterpret_cast<enum OnlineStatus*>(_v) = _t->_state; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->_id, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->_message, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->_icon, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->_state, *reinterpret_cast<enum OnlineStatus*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN3OCC19UserStatusConnectorE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::UserStatusConnector::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19UserStatusConnectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::UserStatusConnector",
        "userStatusFetched",
        "",
        "OCC::UserStatus",
        "userStatus",
        "predefinedStatusesFetched",
        "QList<OCC::UserStatus>",
        "statuses",
        "userStatusSet",
        "serverUserStatusChanged",
        "messageCleared",
        "error",
        "OCC::UserStatusConnector::Error",
        "Error",
        "CouldNotFetchUserStatus",
        "CouldNotFetchPredefinedUserStatuses",
        "UserStatusNotSupported",
        "EmojisNotSupported",
        "CouldNotSetUserStatus",
        "CouldNotClearMessage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'userStatusFetched'
        QtMocHelpers::SignalData<void(const OCC::UserStatus &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'predefinedStatusesFetched'
        QtMocHelpers::SignalData<void(const QVector<OCC::UserStatus> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'userStatusSet'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverUserStatusChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'messageCleared'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'error'
        QtMocHelpers::SignalData<void(OCC::UserStatusConnector::Error)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(13, 13, QMC::EnumIsScoped).add({
            {   14, Error::CouldNotFetchUserStatus },
            {   15, Error::CouldNotFetchPredefinedUserStatuses },
            {   16, Error::UserStatusNotSupported },
            {   17, Error::EmojisNotSupported },
            {   18, Error::CouldNotSetUserStatus },
            {   19, Error::CouldNotClearMessage },
        }),
    };
    return QtMocHelpers::metaObjectData<UserStatusConnector, qt_meta_tag_ZN3OCC19UserStatusConnectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::UserStatusConnector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19UserStatusConnectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19UserStatusConnectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC19UserStatusConnectorE_t>.metaTypes,
    nullptr
} };

void OCC::UserStatusConnector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UserStatusConnector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->userStatusFetched((*reinterpret_cast<std::add_pointer_t<OCC::UserStatus>>(_a[1]))); break;
        case 1: _t->predefinedStatusesFetched((*reinterpret_cast<std::add_pointer_t<QList<OCC::UserStatus>>>(_a[1]))); break;
        case 2: _t->userStatusSet(); break;
        case 3: _t->serverUserStatusChanged(); break;
        case 4: _t->messageCleared(); break;
        case 5: _t->error((*reinterpret_cast<std::add_pointer_t<OCC::UserStatusConnector::Error>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::UserStatus >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<OCC::UserStatus> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UserStatusConnector::*)(const OCC::UserStatus & )>(_a, &UserStatusConnector::userStatusFetched, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusConnector::*)(const QVector<OCC::UserStatus> & )>(_a, &UserStatusConnector::predefinedStatusesFetched, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusConnector::*)()>(_a, &UserStatusConnector::userStatusSet, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusConnector::*)()>(_a, &UserStatusConnector::serverUserStatusChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusConnector::*)()>(_a, &UserStatusConnector::messageCleared, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserStatusConnector::*)(OCC::UserStatusConnector::Error )>(_a, &UserStatusConnector::error, 5))
            return;
    }
}

const QMetaObject *OCC::UserStatusConnector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::UserStatusConnector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19UserStatusConnectorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::UserStatusConnector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void OCC::UserStatusConnector::userStatusFetched(const OCC::UserStatus & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::UserStatusConnector::predefinedStatusesFetched(const QVector<OCC::UserStatus> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::UserStatusConnector::userStatusSet()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::UserStatusConnector::serverUserStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::UserStatusConnector::messageCleared()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::UserStatusConnector::error(OCC::UserStatusConnector::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
