/****************************************************************************
** Meta object code from reading C++ file 'clientsideencryption.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/libsync/clientsideencryption.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'clientsideencryption.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::ClientSideEncryption::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::ClientSideEncryption",
        "initializationFinished",
        "",
        "isNewMnemonicGenerated",
        "sensitiveDataForgotten",
        "privateKeyDeleted",
        "certificateDeleted",
        "mnemonicDeleted",
        "publicKeyDeleted",
        "certificateFetchedFromKeychain",
        "QSslCertificate",
        "certificate",
        "certificatesFetchedFromServer",
        "QHash<QString,OCC::NextcloudSslCertificate>",
        "results",
        "certificateWriteComplete",
        "startingDiscoveryEncryptionUsbToken",
        "finishedDiscoveryEncryptionUsbToken",
        "canEncryptChanged",
        "canDecryptChanged",
        "userCertificateNeedsMigrationChanged",
        "initialize",
        "QWidget*",
        "settingsDialog",
        "initializeHardwareTokenEncryption",
        "addExtraRootCertificates",
        "forgetSensitiveData",
        "getUsersPublicKeyFromServer",
        "userIds",
        "fetchCertificateFromKeyChain",
        "userId",
        "writeCertificate",
        "migrateCertificate",
        "generateKeyPair",
        "encryptPrivateKey",
        "publicCertificateFetched",
        "QKeychain::Job*",
        "incoming",
        "publicKeyFetched",
        "publicKeyFetchedForUserId",
        "privateKeyFetched",
        "mnemonicKeyFetched",
        "handlePrivateKeyDeleted",
        "const QKeychain::Job*",
        "handleCertificateDeleted",
        "handleMnemonicDeleted",
        "handlePublicKeyDeleted",
        "checkAllSensitiveDataDeleted",
        "getPrivateKeyFromServer",
        "getPublicKeyFromServer",
        "fetchAndValidatePublicKeyFromServer",
        "decryptPrivateKey",
        "key",
        "fetchPublicKeyFromKeyChain",
        "writePrivateKey",
        "completeHardwareTokenInitialization",
        "setMnemonic",
        "mnemonic",
        "canEncrypt",
        "canDecrypt",
        "userCertificateNeedsMigration",
        "EncryptionErrorType",
        "RetryOnError",
        "FatalError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'initializationFinished'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'initializationFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Signal 'sensitiveDataForgotten'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'privateKeyDeleted'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateDeleted'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mnemonicDeleted'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'publicKeyDeleted'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateFetchedFromKeychain'
        QtMocHelpers::SignalData<void(QSslCertificate)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'certificatesFetchedFromServer'
        QtMocHelpers::SignalData<void(const QHash<QString,OCC::NextcloudSslCertificate> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'certificateWriteComplete'
        QtMocHelpers::SignalData<void(const QSslCertificate &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'startingDiscoveryEncryptionUsbToken'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finishedDiscoveryEncryptionUsbToken'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canEncryptChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canDecryptChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userCertificateNeedsMigrationChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initialize'
        QtMocHelpers::SlotData<void(QWidget *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'initializeHardwareTokenEncryption'
        QtMocHelpers::SlotData<void(QWidget *)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'addExtraRootCertificates'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forgetSensitiveData'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getUsersPublicKeyFromServer'
        QtMocHelpers::SlotData<void(const QStringList &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 28 },
        }}),
        // Slot 'fetchCertificateFromKeyChain'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Slot 'writeCertificate'
        QtMocHelpers::SlotData<void(const QString &, const QSslCertificate &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'migrateCertificate'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'generateKeyPair'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'encryptPrivateKey'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'publicCertificateFetched'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'publicKeyFetched'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'publicKeyFetchedForUserId'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'privateKeyFetched'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'mnemonicKeyFetched'
        QtMocHelpers::SlotData<void(QKeychain::Job *)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'handlePrivateKeyDeleted'
        QtMocHelpers::SlotData<void(const QKeychain::Job * const)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 37 },
        }}),
        // Slot 'handleCertificateDeleted'
        QtMocHelpers::SlotData<void(const QKeychain::Job * const)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 37 },
        }}),
        // Slot 'handleMnemonicDeleted'
        QtMocHelpers::SlotData<void(const QKeychain::Job * const)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 37 },
        }}),
        // Slot 'handlePublicKeyDeleted'
        QtMocHelpers::SlotData<void(const QKeychain::Job * const)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 37 },
        }}),
        // Slot 'checkAllSensitiveDataDeleted'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'getPrivateKeyFromServer'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'getPublicKeyFromServer'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'fetchAndValidatePublicKeyFromServer'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'decryptPrivateKey'
        QtMocHelpers::SlotData<void(const QByteArray &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 52 },
        }}),
        // Slot 'fetchCertificateFromKeyChain'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'fetchPublicKeyFromKeyChain'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'writePrivateKey'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'writeCertificate'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'completeHardwareTokenInitialization'
        QtMocHelpers::SlotData<void(QWidget *)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'setMnemonic'
        QtMocHelpers::SlotData<void(const QString &)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 57 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'canEncrypt'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 12),
        // property 'canDecrypt'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 13),
        // property 'userCertificateNeedsMigration'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 14),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EncryptionErrorType'
        QtMocHelpers::EnumData<enum EncryptionErrorType>(61, 61, QMC::EnumIsScoped).add({
            {   62, EncryptionErrorType::RetryOnError },
            {   63, EncryptionErrorType::FatalError },
        }),
    };
    return QtMocHelpers::metaObjectData<ClientSideEncryption, qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::ClientSideEncryption::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t>.metaTypes,
    nullptr
} };

void OCC::ClientSideEncryption::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ClientSideEncryption *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->initializationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->initializationFinished(); break;
        case 2: _t->sensitiveDataForgotten(); break;
        case 3: _t->privateKeyDeleted(); break;
        case 4: _t->certificateDeleted(); break;
        case 5: _t->mnemonicDeleted(); break;
        case 6: _t->publicKeyDeleted(); break;
        case 7: _t->certificateFetchedFromKeychain((*reinterpret_cast<std::add_pointer_t<QSslCertificate>>(_a[1]))); break;
        case 8: _t->certificatesFetchedFromServer((*reinterpret_cast<std::add_pointer_t<QHash<QString,OCC::NextcloudSslCertificate>>>(_a[1]))); break;
        case 9: _t->certificateWriteComplete((*reinterpret_cast<std::add_pointer_t<QSslCertificate>>(_a[1]))); break;
        case 10: _t->startingDiscoveryEncryptionUsbToken(); break;
        case 11: _t->finishedDiscoveryEncryptionUsbToken(); break;
        case 12: _t->canEncryptChanged(); break;
        case 13: _t->canDecryptChanged(); break;
        case 14: _t->userCertificateNeedsMigrationChanged(); break;
        case 15: _t->initialize((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 16: _t->initializeHardwareTokenEncryption((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 17: _t->addExtraRootCertificates(); break;
        case 18: _t->forgetSensitiveData(); break;
        case 19: _t->getUsersPublicKeyFromServer((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 20: _t->fetchCertificateFromKeyChain((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->writeCertificate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSslCertificate>>(_a[2]))); break;
        case 22: _t->migrateCertificate(); break;
        case 23: _t->generateKeyPair(); break;
        case 24: _t->encryptPrivateKey(); break;
        case 25: _t->publicCertificateFetched((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        case 26: _t->publicKeyFetched((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        case 27: _t->publicKeyFetchedForUserId((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        case 28: _t->privateKeyFetched((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        case 29: _t->mnemonicKeyFetched((*reinterpret_cast<std::add_pointer_t<QKeychain::Job*>>(_a[1]))); break;
        case 30: _t->handlePrivateKeyDeleted((*reinterpret_cast<std::add_pointer_t<const QKeychain::Job*>>(_a[1]))); break;
        case 31: _t->handleCertificateDeleted((*reinterpret_cast<std::add_pointer_t<const QKeychain::Job*>>(_a[1]))); break;
        case 32: _t->handleMnemonicDeleted((*reinterpret_cast<std::add_pointer_t<const QKeychain::Job*>>(_a[1]))); break;
        case 33: _t->handlePublicKeyDeleted((*reinterpret_cast<std::add_pointer_t<const QKeychain::Job*>>(_a[1]))); break;
        case 34: _t->checkAllSensitiveDataDeleted(); break;
        case 35: _t->getPrivateKeyFromServer(); break;
        case 36: _t->getPublicKeyFromServer(); break;
        case 37: _t->fetchAndValidatePublicKeyFromServer(); break;
        case 38: _t->decryptPrivateKey((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 39: _t->fetchCertificateFromKeyChain(); break;
        case 40: _t->fetchPublicKeyFromKeyChain(); break;
        case 41: _t->writePrivateKey(); break;
        case 42: _t->writeCertificate(); break;
        case 43: _t->completeHardwareTokenInitialization((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 44: _t->setMnemonic((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslCertificate >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslCertificate >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslCertificate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)(bool )>(_a, &ClientSideEncryption::initializationFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::sensitiveDataForgotten, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::privateKeyDeleted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::certificateDeleted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::mnemonicDeleted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::publicKeyDeleted, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)(QSslCertificate )>(_a, &ClientSideEncryption::certificateFetchedFromKeychain, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)(const QHash<QString,OCC::NextcloudSslCertificate> & )>(_a, &ClientSideEncryption::certificatesFetchedFromServer, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)(const QSslCertificate & )>(_a, &ClientSideEncryption::certificateWriteComplete, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::startingDiscoveryEncryptionUsbToken, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::finishedDiscoveryEncryptionUsbToken, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::canEncryptChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::canDecryptChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientSideEncryption::*)()>(_a, &ClientSideEncryption::userCertificateNeedsMigrationChanged, 14))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->canEncrypt(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->canDecrypt(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->userCertificateNeedsMigration(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::ClientSideEncryption::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::ClientSideEncryption::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC20ClientSideEncryptionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::ClientSideEncryption::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void OCC::ClientSideEncryption::initializationFinished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 2
void OCC::ClientSideEncryption::sensitiveDataForgotten()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::ClientSideEncryption::privateKeyDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::ClientSideEncryption::certificateDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::ClientSideEncryption::mnemonicDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::ClientSideEncryption::publicKeyDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::ClientSideEncryption::certificateFetchedFromKeychain(QSslCertificate _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void OCC::ClientSideEncryption::certificatesFetchedFromServer(const QHash<QString,OCC::NextcloudSslCertificate> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void OCC::ClientSideEncryption::certificateWriteComplete(const QSslCertificate & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void OCC::ClientSideEncryption::startingDiscoveryEncryptionUsbToken()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OCC::ClientSideEncryption::finishedDiscoveryEncryptionUsbToken()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void OCC::ClientSideEncryption::canEncryptChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void OCC::ClientSideEncryption::canDecryptChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void OCC::ClientSideEncryption::userCertificateNeedsMigrationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}
QT_WARNING_POP
