/****************************************************************************
** Meta object code from reading C++ file 'accountmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/accountmanager.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14AccountManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountManager::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14AccountManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountManager",
        "accountAdded",
        "",
        "OCC::AccountState*",
        "account",
        "accountRemoved",
        "accountSyncConnectionRemoved",
        "removeAccountFolders",
        "forceLegacyImportChanged",
        "capabilitiesChanged",
        "accountListInitialized",
        "saveAccount",
        "OCC::AccountPtr",
        "newAccountData",
        "saveAccountState",
        "a",
        "save",
        "saveCredentials",
        "deleteAccount",
        "shutdown",
        "setForceLegacyImport",
        "forceLegacyImport",
        "AccountsRestoreResult",
        "AccountsRestoreFailure",
        "AccountsNotFound",
        "AccountsRestoreSuccess",
        "AccountsRestoreSuccessFromLegacyVersion",
        "AccountsRestoreSuccessWithSkipped"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountAdded'
        QtMocHelpers::SignalData<void(OCC::AccountState *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'accountRemoved'
        QtMocHelpers::SignalData<void(OCC::AccountState *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'accountSyncConnectionRemoved'
        QtMocHelpers::SignalData<void(OCC::AccountState *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'removeAccountFolders'
        QtMocHelpers::SignalData<void(OCC::AccountState *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'forceLegacyImportChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'capabilitiesChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountListInitialized'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAccount'
        QtMocHelpers::SlotData<void(const OCC::AccountPtr &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'saveAccountState'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 15 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'deleteAccount'
        QtMocHelpers::SlotData<void(OCC::AccountState *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setForceLegacyImport'
        QtMocHelpers::SlotData<void(const bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'forceLegacyImport'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AccountsRestoreResult'
        QtMocHelpers::EnumData<enum AccountsRestoreResult>(22, 22, QMC::EnumFlags{}).add({
            {   23, AccountsRestoreResult::AccountsRestoreFailure },
            {   24, AccountsRestoreResult::AccountsNotFound },
            {   25, AccountsRestoreResult::AccountsRestoreSuccess },
            {   26, AccountsRestoreResult::AccountsRestoreSuccessFromLegacyVersion },
            {   27, AccountsRestoreResult::AccountsRestoreSuccessWithSkipped },
        }),
    };
    return QtMocHelpers::metaObjectData<AccountManager, qt_meta_tag_ZN3OCC14AccountManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::AccountManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14AccountManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14AccountManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14AccountManagerE_t>.metaTypes,
    nullptr
} };

void OCC::AccountManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountAdded((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 1: _t->accountRemoved((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 2: _t->accountSyncConnectionRemoved((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 3: _t->removeAccountFolders((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 4: _t->forceLegacyImportChanged(); break;
        case 5: _t->capabilitiesChanged(); break;
        case 6: _t->accountListInitialized(); break;
        case 7: _t->saveAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountPtr>>(_a[1]))); break;
        case 8: _t->saveAccountState((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 9: _t->save((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->save(); break;
        case 11: _t->deleteAccount((*reinterpret_cast<std::add_pointer_t<OCC::AccountState*>>(_a[1]))); break;
        case 12: _t->shutdown(); break;
        case 13: _t->setForceLegacyImport((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountPtr >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::AccountState* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)(OCC::AccountState * )>(_a, &AccountManager::accountAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)(OCC::AccountState * )>(_a, &AccountManager::accountRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)(OCC::AccountState * )>(_a, &AccountManager::accountSyncConnectionRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)(OCC::AccountState * )>(_a, &AccountManager::removeAccountFolders, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)()>(_a, &AccountManager::forceLegacyImportChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)()>(_a, &AccountManager::capabilitiesChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountManager::*)()>(_a, &AccountManager::accountListInitialized, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->forceLegacyImport(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setForceLegacyImport(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::AccountManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14AccountManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::AccountManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OCC::AccountManager::accountAdded(OCC::AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OCC::AccountManager::accountRemoved(OCC::AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::AccountManager::accountSyncConnectionRemoved(OCC::AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OCC::AccountManager::removeAccountFolders(OCC::AccountState * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OCC::AccountManager::forceLegacyImportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::AccountManager::capabilitiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::AccountManager::accountListInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
