/****************************************************************************
** Meta object code from reading C++ file 'sharee.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/gui/sharee.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sharee.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC6ShareeE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Sharee::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC6ShareeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Sharee",
        "format",
        "shareWith",
        "displayName",
        "iconUrlColoured",
        "type",
        "Type",
        "Invalid",
        "User",
        "Group",
        "Email",
        "Federated",
        "Team",
        "Room",
        "LookupServerSearch",
        "LookupServerSearchResults"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'format'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'shareWith'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'displayName'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'iconUrlColoured'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'type'
        QtMocHelpers::PropertyData<enum Type>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(6, 6, QMC::EnumFlags{}).add({
            {    7, Type::Invalid },
            {    8, Type::User },
            {    9, Type::Group },
            {   10, Type::Email },
            {   11, Type::Federated },
            {   12, Type::Team },
            {   13, Type::Room },
            {   14, Type::LookupServerSearch },
            {   15, Type::LookupServerSearchResults },
        }),
    };
    return QtMocHelpers::metaObjectData<Sharee, qt_meta_tag_ZN3OCC6ShareeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Sharee::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6ShareeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6ShareeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC6ShareeE_t>.metaTypes,
    nullptr
} };

void OCC::Sharee::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Sharee *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->format(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->_shareWith; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->_displayName; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->_iconUrlColoured; break;
        case 4: *reinterpret_cast<enum Type*>(_v) = _t->_type; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: QtMocHelpers::setProperty(_t->_shareWith, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->_displayName, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->_iconUrlColoured, *reinterpret_cast<QString*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->_type, *reinterpret_cast<enum Type*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
