/****************************************************************************
** Meta object code from reading C++ file 'vfs.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../nextcloud-client/src/common/vfs.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vfs.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC3VfsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Vfs::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC3VfsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Vfs",
        "beginHydrating",
        "",
        "doneHydrating",
        "failureHydrating",
        "errorCode",
        "statusCode",
        "errorString",
        "fileName",
        "fileStatusChanged",
        "systemFileName",
        "OCC::SyncFileStatus",
        "fileStatus",
        "Mode",
        "Off",
        "WithSuffix",
        "WindowsCfApi",
        "XAttr",
        "ConvertToPlaceholderResult",
        "Error",
        "Ok",
        "Locked",
        "UpdateMetadataType",
        "DatabaseMetadata",
        "FileMetadata",
        "AllMetadata",
        "AvailabilityError",
        "DbError",
        "NoSuchItem",
        "AvailabilityRecursivity",
        "RecursiveAvailability",
        "NotRecursiveAvailability"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'beginHydrating'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'doneHydrating'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'failureHydrating'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Int, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'fileStatusChanged'
        QtMocHelpers::SlotData<void(const QString &, OCC::SyncFileStatus)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Mode'
        QtMocHelpers::EnumData<enum Mode>(13, 13, QMC::EnumFlags{}).add({
            {   14, Mode::Off },
            {   15, Mode::WithSuffix },
            {   16, Mode::WindowsCfApi },
            {   17, Mode::XAttr },
        }),
        // enum 'ConvertToPlaceholderResult'
        QtMocHelpers::EnumData<enum ConvertToPlaceholderResult>(18, 18, QMC::EnumIsScoped).add({
            {   19, ConvertToPlaceholderResult::Error },
            {   20, ConvertToPlaceholderResult::Ok },
            {   21, ConvertToPlaceholderResult::Locked },
        }),
        // enum 'UpdateMetadataType'
        QtMocHelpers::EnumData<enum UpdateMetadataType>(22, 22, QMC::EnumFlags{}).add({
            {   23, UpdateMetadataType::DatabaseMetadata },
            {   24, UpdateMetadataType::FileMetadata },
            {   25, UpdateMetadataType::AllMetadata },
        }),
        // enum 'AvailabilityError'
        QtMocHelpers::EnumData<enum AvailabilityError>(26, 26, QMC::EnumIsScoped).add({
            {   27, AvailabilityError::DbError },
            {   28, AvailabilityError::NoSuchItem },
        }),
        // enum 'AvailabilityRecursivity'
        QtMocHelpers::EnumData<enum AvailabilityRecursivity>(29, 29, QMC::EnumIsScoped).add({
            {   30, AvailabilityRecursivity::RecursiveAvailability },
            {   31, AvailabilityRecursivity::NotRecursiveAvailability },
        }),
    };
    return QtMocHelpers::metaObjectData<Vfs, qt_meta_tag_ZN3OCC3VfsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Vfs::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC3VfsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC3VfsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC3VfsE_t>.metaTypes,
    nullptr
} };

void OCC::Vfs::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Vfs *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->beginHydrating(); break;
        case 1: _t->doneHydrating(); break;
        case 2: _t->failureHydrating((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 3: _t->fileStatusChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::SyncFileStatus>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileStatus >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Vfs::*)()>(_a, &Vfs::beginHydrating, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Vfs::*)()>(_a, &Vfs::doneHydrating, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Vfs::*)(int , int , const QString & , const QString & )>(_a, &Vfs::failureHydrating, 2))
            return;
    }
}

const QMetaObject *OCC::Vfs::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Vfs::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC3VfsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Vfs::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OCC::Vfs::beginHydrating()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Vfs::doneHydrating()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::Vfs::failureHydrating(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}
namespace {
struct qt_meta_tag_ZN3OCC6VfsOffE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::VfsOff::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC6VfsOffE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::VfsOff",
        "fileStatusChanged",
        "",
        "OCC::SyncFileStatus"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'fileStatusChanged'
        QtMocHelpers::SlotData<void(const QString &, OCC::SyncFileStatus)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VfsOff, qt_meta_tag_ZN3OCC6VfsOffE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::VfsOff::staticMetaObject = { {
    QMetaObject::SuperData::link<Vfs::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6VfsOffE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6VfsOffE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC6VfsOffE_t>.metaTypes,
    nullptr
} };

void OCC::VfsOff::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VfsOff *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileStatusChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OCC::SyncFileStatus>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OCC::SyncFileStatus >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OCC::VfsOff::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::VfsOff::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6VfsOffE_t>.strings))
        return static_cast<void*>(this);
    return Vfs::qt_metacast(_clname);
}

int OCC::VfsOff::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Vfs::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
